function p_1 = choke_gas(C_d,d_ch,gamma,p_4,q_sc,rho_sc,T_1)
% function p_1 = choke_gas(C_d,d_ch,gamma,p_4,q_sc,rho_sc,T_1)
%
% Computes the upstream pressure p_1 for single-phase gas flow through a choke for a given
% downstream pressure p_4 and upstream temperature T_1. In case of critical flow, the result
% becomes independent of the downstream pressure.      
%
% C_d = discharge coefficient, -
% d_ch = choke diameter, m
% gamma = ratio of specific heat capacities, -
% p_1 = upstream pressure, Pa
% p_4 = downstream pressure, Pa
% q_sc = [q_g_sc,q_o_sc,q_w_sc], m^3/s
%   q_g_sc = gas flow rate at standard conditions, m^3/s. 
%   q_o_sc = oil flow rate at standard conditions, m^3/s. Not relevant.
%   q_w_sc = water flow rate at standard conditions, m^3/s. Not relevant.
% rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc], kg/m^3 
%   rho_g_sc = gas density at standard conditions, kg/m^3.
%   rho_o_sc = oil density at standard conditions, kg/m^3. Not relevant.
%   rho_w_sc = water density at standard conditions, kg/m^3. Not relevant.
% T_1 = upstream temperature, deg. C

% JDJ, 08-05-11, last revised 01-05-12

% Retrieve data:
rho_g_sc = rho_sc(1); 
q_g_sc = q_sc(1); 
q_o_sc = q_sc(2);
q_w_sc = q_sc(3); 

% Check input:
if (q_o_sc ~= 0)
    error('Nonzero oil rate. Single-phase gas solution not valid.')
end
if (q_w_sc ~= 0)
    error('Nonzero water rate. Single-phase gas solution not valid.')
end

% Set iteration parameters:
max_iter = 1000;
tol = 1e-6; % relative tolerance pressure iterations, - 

% Compute auxiliary variables:
A_2 = pi*d_ch^2/4; % choke throat surface area, m^2
T_1_abs = T_1 + 273.15; % temperature, K
p_sc = 1e5;
T_sc_abs = 15 + 273.15;
p_pc = pres_pseu_crit_Sutton(rho_g_sc);
T_pc = temp_pseu_crit_Sutton(rho_g_sc);

% Assume critical flow and compute pressure drop:
r_crit = (2/(1+gamma))^(gamma/(gamma-1)); % critical pressure ratio (p_4/p_1)_crit, -
p_1 = p_4 / r_crit; % first guess
repeat = 1;
iter = 0;
while repeat == 1
    iter = iter + 1;
    if iter > max_iter
        error('Maximum number of iterations exceeded.')
    end
    p_1_old = p_1;
    p_1_pr = p_1/p_pc;
    T_1_pr = T_1_abs/T_pc;
    Z_1 = Z_factor_DAK(p_1_pr,T_1_pr);
    help_1 = 2/(1+gamma);
    help_2 = help_1^(2/(gamma-1));
    help_3 = help_1^((1+gamma)/(gamma-1));
    help_4 = (gamma/(gamma-1)) * (help_2-help_3);
    help_5 = 2*T_sc_abs*help_4/(p_sc*T_1_abs*Z_1*rho_g_sc);
    p_1 = abs(q_g_sc)/(C_d*A_2*sqrt(help_5));
    if p_1_old ~= 0
        err_rel = abs((p_1-p_1_old)/p_1_old); % relative error, -
        if err_rel < tol % converged
            repeat = 0;
        end
    else
        repeat = 1;
    end
end

% Recompute p_1 in case of subcritical flow:
p_1_crit = p_4 / r_crit;
if p_1 < p_1_crit % subcritical flow
    p_1 = p_1_crit; % first guess
    repeat = 1;
    iter = 0;
    while repeat == 1
        iter = iter + 1;
        if iter > max_iter
            error('Maximum number of iterations exceeded.')
        end
        p_1_old = p_1;
        p_1_pr = p_1/p_pc;
        T_1_pr = T_1_abs/T_pc;
        Z_1 = Z_factor_DAK(p_1_pr,T_1_pr);
        B_g_1 = gas_form_vol_fact(p_1,T_1_abs,Z_1);
        B_g_4 = (p_4/p_1)^(-1/gamma) * B_g_1;
        help_1 = (gamma-1)/gamma;
        help_2 = (rho_g_sc*q_g_sc^2*B_g_4^2)/(2*A_2^2*C_d^2);
        p_1 = (1/B_g_1) * (B_g_4*p_4 + help_1*help_2);
        err_rel = abs((p_1-p_1_old)/p_1_old); % relative error, -
        if err_rel < tol % converged
            repeat = 0;
        end
    end
end

