function [p_1,r_crit] = choke_multi_phase(C_d,c_l,c_p,c_v,d_ch,oil,p_4,q_sc,rho_sc,T_1)
% function [p_1,r_crit] = choke_multi_phase(C_d,c_l,c_p,c_v,d_ch,oil,p_4,q_sc,rho_sc,T_1)
%
% Computes the upstream pressure p_1 for multi-phase flow through a choke for a given
% downstream pressure p_4 and upstream temperature T_1. In case of critical flow, the result
% becomes independent of the downstream pressure.      
%
% C_d = discharge coefficient, -
% c_l = specific heat capacity of liquid at constant volume, J/(kg K)
% c_p = specific heat capacity of gas at constant pressure, J/(kg K)
% c_v = specific heat capacity of gas at constant volume, J/(kg K)
% d_ch = choke diameter, m
% oil = parameter to select black oil model or volatile oil table, -  
%   oil = 1: black oil; parameters computed with the aid of Standing correlations
%   oil = 2: black oil; parameters computed with the aid of Glaso correlations
%   oil = 3: volatile oil; parameters read from file 'vol_oil_table_01'
% p_1 = upstream pressure, Pa
% p_4 = downstream pressure, Pa
% q_sc = [q_g_sc,q_o_sc,q_w_sc], m^3/s
%   q_g_sc = gas flow rate at standard conditions, m^3/s 
%   q_o_sc = oil flow rate at standard conditions, m^3/s
%   q_w_sc = water flow rate at standard conditions, m^3/s
% r_crit = (p_4/p_1)_crit = critical pressure ratio, -
% rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc], kg/m^3 
%   rho_g_sc = gas density at standard conditions, kg/m^3
%   rho_o_sc = oil density at standard conditions, kg/m^3
%   rho_w_sc = water density at standard conditions, kg/m^3
% T_1 = upstream temperature, deg. C

% JDJ, 01-05-12, last revised 31-01-15

% Retrieve data:
q_g_sc = q_sc(1);
q_o_sc = q_sc(2);
q_w_sc = q_sc(3);
rho_g_sc = rho_sc(1);
rho_o_sc = rho_sc(2);

% Check input:
if q_g_sc == 0
    error('Zero gas rate. Multi-phase solution not valid.')
end
if q_o_sc == 0
    error('Zero oil rate. Multi-phase solution not valid.')
end
if q_g_sc > 0 | q_o_sc > 0 | q_w_sc > 0
    q_g_sc
    q_o_sc
    q_w_sc
    error('Injection. Multi-phase solution not valid.')
end

% Compute auxiliary variables:
A_2 = pi*d_ch^2/4; % choke throat surface area, m^2
R_go = q_g_sc / q_o_sc; % producing gas-oil ratio, m^3/m^3
R_sb = R_go; % solution gas-oil ratio at bubble point of oil in choke, m^3/m^3
switch oil
    case 1
        p_b = pres_bub_Standing(R_sb,rho_g_sc,rho_o_sc,T_1); % bubble point pressure, Pa
    case 2
        p_b = pres_bub_Glaso(R_sb,rho_g_sc,rho_o_sc,T_1);
    case 3
        p_b = pres_bub_volatile_oil(T_1);
end

% Assume subcritical flow and compute pressure drop:
p_1_min = p_4; % lower boundary for fzero, Pa
p_1_max = p_b; % upper boundary for fzero, Pa
options = []; % options for fzero
p_1 = fminbnd(@(p_1) f_min_sub_crit_multi_phase(A_2,C_d,c_l,c_p,c_v,oil,p_1,p_4,q_sc,...
    R_sb,rho_sc,T_1),p_1_min,p_1_max,options);

% Recompute p_1 in case of critical flow:
r_crit = choke_multi_phase_boundary(c_l,c_p,c_v,oil,p_4,q_sc,rho_sc,T_1); % crit.pres.ratio., -
p_1_crit = p_4 / r_crit;
if p_1 > p_1_crit % critical flow
    p_1 = fminbnd(@(p_1) f_min_crit_multi_phase(A_2,C_d,c_l,c_p,c_v,oil,p_1,q_sc,...
    r_crit,R_sb,rho_sc,T_1),p_1_min,p_1_max,options);
end

% Check result:
if p_1 >= p_b
    error('Upstream pressure exceeds bubble point pressure. Use of choke_multi_phase not valid.')
end

% Auxiliary function for fminbnd (cost function to be minimized); critical flow:
function y = f_min_crit_multi_phase(A_2,C_d,c_l,c_p,c_v,oil,p_1,q_sc,r_crit,R_sb,rho_sc,T_1)

% Compute function value y:
[q,rho] = local_q_and_rho(oil,p_1,q_sc,R_sb,rho_sc,T_1);
q_g = q(1); % local gas rate, m^3/s
q_o = q(2); % local oil rate, m^3/s
q_w = q(3); % local water rate, m^3/s
q_l = q_o + q_w; % local liquid flow rate, m^3/s
q_m = q_g + q_l; % local mixture flow rate, m^3/s
f_o = q_o / q_l; % oil volume fraction, -
f_w = q_w / q_l; % water volume fraction, -
lambda_g = q_g / q_m; % gas volume fraction, -
lambda_l = q_l / q_m; % liquid volume fraction, -
rho_g = rho(1); % local gas density, kg/m^3
rho_o = rho(2); % local oil density, kg/m^3
rho_w = rho(3); % local water density, kg/m^3
rho_l = f_o * rho_o + f_w * rho_w; % local no-slip liquid density, kg/m^3
rho_mn = lambda_g * rho_g + lambda_l * rho_l; % local no-slip mixture density, kg/m^3
x_g = q_g*rho_g / (q_g*rho_g + q_o*rho_o + q_w*rho_w); % local gas mass fraction, -
x_l = (q_o*rho_o + q_w*rho_w) / (q_g*rho_g + q_o*rho_o + q_w*rho_w); % local liquid mass fr., -
n = 1 + (x_g * (c_p-c_v)) / (x_g*c_v + x_l*c_l); % polytropic expansion coefficient -
eps = q_l / q_g; % local liquid-gas ratio, m^3/m^3
help_1 = (1 + eps) * r_crit^(1/n) / (1 + eps * r_crit^(1/n));
help_2 = (n/(n-1) + eps * r_crit^(1/n)) / (1 + eps * r_crit^(1/n));
help_3 = (n/(n-1) + eps) * r_crit^(1/n) / (n/(n-1) + eps * r_crit^(1/n));
Theta_crit = help_1 * help_2 * (help_3 - r_crit);
y = (p_1 - (rho_mn * q_m^2 / (2 * C_d^2 * A_2^2 * Theta_crit)))^2;
    
% Auxiliary function for fminbnd (cost function to be minimized); sub-critical flow:
function y = f_min_sub_crit_multi_phase(A_2,C_d,c_l,c_p,c_v,oil,p_1,p_4,q_sc,R_sb,rho_sc,T_1)

% Compute function value y:
[q,rho] = local_q_and_rho(oil,p_1,q_sc,R_sb,rho_sc,T_1);
q_g = q(1); % local gas rate, m^3/s
q_o = q(2); % local oil rate, m^3/s
q_w = q(3); % local water rate, m^3/s
q_l = q_o + q_w; % local liquid flow rate, m^3/s
q_m = q_g + q_l; % local mixture flow rate, m^3/s
f_o = q_o / q_l; % oil volume fraction, -
f_w = q_w / q_l; % water volume fraction, -
lambda_g = q_g / q_m; % gas volume fraction, -
lambda_l = q_l / q_m; % liquid volume fraction, -
rho_g = rho(1); % local gas density, kg/m^3
rho_o = rho(2); % local oil density, kg/m^3
rho_w = rho(3); % local water density, kg/m^3
rho_l = f_o * rho_o + f_w * rho_w; % local no-slip liquid density, kg/m^3
rho_mn = lambda_g * rho_g + lambda_l * rho_l; % local no-slip mixture density, kg/m^3
x_g = q_g*rho_g / (q_g*rho_g + q_o*rho_o + q_w*rho_w); % local gas mass fraction, -
x_l = (q_o*rho_o + q_w*rho_w) / (q_g*rho_g + q_o*rho_o + q_w*rho_w); % local liquid mass fr., -
n = 1 + (x_g * (c_p-c_v)) / (x_g*c_v + x_l*c_l); % polytropic expansion coefficient -
eps = q_l / q_g; % local liquid-gas ratio, m^3/m^3
r = p_4 / p_1; % pressure ratio, -
help_1 = (1 + eps) * r^(1/n) / (1 + eps * r^(1/n));
help_2 = (n/(n-1) + eps * r^(1/n)) / (1 + eps * r^(1/n));
help_3 = (n/(n-1) + eps) * r^(1/n) / (n/(n-1) + eps * r^(1/n));
Theta = help_1 * help_2 * (help_3 - r);
y = (p_1 - (rho_mn * q_m^2 / (2 * C_d^2 * A_2^2 * Theta)))^2;
    

