function r_crit = choke_multi_phase_boundary(c_l,c_p,c_v,oil,p_4,q_sc,rho_sc,T_1)
% function r_crit = choke_multi_phase_boundary(c_l,c_p,c_v,oil,p_4,q_sc,rho_sc,T_1)
%
% Computes the critical pressure ratio r_crit = (p_4/p_1)_crit for multi-phase flow through a
% choke. This is done through computing the pressure ratio r = p_4/p_1 for which the auxiliary
% variable y achieves a minimum, with the aid of MATLAB function fminbnd. The auxiliary
% variable y is the negative of the mass flow rate through the choke (up to a constant) for a
% given value of r. During the process the upstream pressure p_1 is updated iteratively.
%
% c_l = specific heat capacity of liquid at constant volume, J/(kg K)
% c_p = specific heat capacity of gas at constant pressure, J/(kg K)
% c_v = specific heat capacity of gas at constant volume, J/(kg K)
% oil = parameter to select black oil model or volatile oil table, -  
%   oil = 1: black oil; parameters computed with the aid of Standing correlations
%   oil = 2: black oil; parameters computed with the aid of Glaso correlations
%   oil = 3: volatile oil; parameters read from file 'vol_oil_table_01'
% p_4 = downstream pressure, Pa
% r_crit = (p_4/p_1)_crit = critical pressure ratio, -
% q_sc = [q_g_sc,q_o_sc,q_w_sc], m^3/s
%   q_g_sc = gas flow rate at standard conditions, m^3/s 
%   q_o_sc = oil flow rate at standard conditions, m^3/s
%   q_w_sc = water flow rate at standard conditions, m^3/s
% rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc], kg/m^3 
%   rho_g_sc = gas density at standard conditions, kg/m^3
%   rho_o_sc = oil density at standard conditions, kg/m^3 
%   rho_w_sc = water density at standard conditions, kg/m^3
% T_1 = upstream temperature, deg. C

% JDJ, 13-02-12, last revised 31-08-14

% Retrieve data and compute auxiliary variables::
q_g_sc = q_sc(1);
q_o_sc = q_sc(2);
R_go = q_g_sc / q_o_sc; % producing gas-oil ratio, m^3/m^3
R_sb = R_go; % apparent solution gas-oil ratio at bubble point of oil in choke, m^3/m^3
 
% Set iteration parameters:
max_iter = 1000;
tol = 1e-6; % relative tolerance pressure iterations, - 
f = 0.5; % damping factor, -

% Compute r_crit, while iterating on p_1:
p_1 = p_4; % initial guess for p_1, Pa
repeat = 1;
iter = 0;
while repeat == 1
    iter = iter + 1;
    if iter > max_iter
        error('Maximum number of iterations exceeded.')
    end
    p_1_old = p_1;
    % Compute auxiliary variables:
    [q,rho] = local_q_and_rho(oil,p_1,q_sc,R_sb,rho_sc,T_1);
    q_g = q(1); % local gas rate, m^3/s
    q_o = q(2); % local oil rate, m^3/s
    q_w = q(3); % local water rate, m^3/s
    rho_g = rho(1); % local gas density, kg/m^3
    rho_o = rho(2); % local oil density, kg/m^3
    rho_w = rho(3); % local water density, kg/m^3
    x_g = q_g*rho_g / (q_g*rho_g + q_o*rho_o + q_w*rho_w); % local gas mass fraction, -
    x_l = (q_o*rho_o + q_w*rho_w) / (q_g*rho_g + q_o*rho_o + q_w*rho_w); % local l.m.fr., -
    n = 1 + (x_g * (c_p-c_v)) / (x_g*c_v + x_l*c_l); % polytropic expansion coefficient -
    eps = (q_o + q_w) / q_g; % local liquid-gas ratio, m^3/m^3 
    r_min = 0.1; % lower boundary for fminbnd, -
    r_max = 10; % upper boundary for fminbnd, -
    options = []; % options for fminbnd
    r_crit = fminbnd(@(r) f_min_gas(eps,n,r),r_min,r_max,options);
    p_1 = (f-1) * p_1_old + f * (p_4/r_crit);
    if p_1_old ~= 0
        err_rel = abs((p_1-p_1_old)/p_1_old); % relative error, -
        if err_rel < tol % converged
            repeat = 0;
        end
    else
        repeat = 1;
    end
end
    
% Auxiliary function for fminbnd (cost function to be minimized):
function y = f_min_gas(eps,n,r)

% Compute function value y:
help_1 = (1 + eps) * r^(1/n) / (1 + eps * r^(1/n));
help_2 = (n/(n-1) + eps * r^(1/n)) / (1 + eps * r^(1/n));
help_3 = (n/(n-1) + eps) * r^(1/n) / (n/(n-1) + eps * r^(1/n));
Theta = help_1 * help_2 * (help_3 - r); 
y = -Theta;
    
