function p_1 = choke_multi_phase_simp(choke_model,d_ch,p_4,q_sc)
% function p_1 = choke_multi_phase_simp(choke_model,d_ch,p_4,q_sc)
%
% Computes the upstream pressure p_1 for critical or sub-critical gas-liquid flow through a
% choke using one of four correlations depending on the value of the parameter 'choke_model' as
% follows: 
%
% choke_model = 1 : (extended) Gilbert (1954)
% choke_model = 2 : (extended) Ros (1960)
% choke_model = 3 : (extended) Baxendell
% choke_model = 4 : (extended) Achong
%
% d_ch = choke diameter, m
% p_1 = upstream pressure, Pa
% p_4 = downstream pressure, Pa
% q_sc = [q_g_sc,q_o_sc,q_w_sc]
%   q_g_sc = gas flow rate at standard conditions, m^3/s
%   q_o_sc = oil flow rate at standard conditions, m^3/s
%   q_w_sc = water flow rate at standard conditions, m^3/s
%
% Note 1: Expects negative values for all flow rates, in line with the convention that flow
% rates in production wells are negative.    
%
% Note 2: For values of p_1 larger than 1.7 * p_4 the choke operates in the critical regime.
% For lower values of p_1 the choke operates in the sub-critical regime.  
%
% Note 3: All model coefficients are given in SI units.
%
% Note 4: This model is based on a heuristic extension of file choke_critical.m
% and uses a fitted third-order polynomial for the sub-critical region. 
%
% JDJ 30-01-15, last revised 22-06-16

q_g_sc = q_sc(1);
q_o_sc = q_sc(2);
q_w_sc = q_sc(3);

q_l_sc = q_o_sc + q_w_sc; % liquid rate at standard conditions, m^3/s
R_gl = q_g_sc / q_l_sc; % gas-liquid ratio as measured at surface, m^3/m^3

% Coefficients:
if choke_model == 1 % Gilbert
    A = 3.75e10;
    B = 0.546;
    C = 1.89;
end
if choke_model == 2 % Ros 
    A = 6.52e10;
    B = 0.500;
    C = 2.00;
end
if choke_model == 3 % Baxendell 
    A = 3.58e10;
    B = 0.546;
    C = 1.93;
end
if choke_model == 4 % Achong
    A = 1.43e10;
    B = 0.650;
    C = 1.88;
end

% Coefficients shared by all models;
D = 1.01e5;
E = 5.61;
F = 2.52e3;

p_1 = -A * q_l_sc * (E*R_gl)^B / (F*d_ch)^C + D;

if p_1 < 1.7 * p_4 % sub-critical flow
    p_1_crit = 1.7 * p_4;
    q_l_sc_crit = (D - p_1_crit) * (F*d_ch)^C / (A * (E*R_gl)^B);
    dpdq_crit = -A * (E*R_gl)^B / (F*d_ch)^C;
    c_2 = dpdq_crit / (2*q_l_sc_crit) + ...
        3 * (p_1_crit - dpdq_crit*q_l_sc_crit/2 - p_4) / q_l_sc_crit^2;
    c_3 = -2 * (p_1_crit - dpdq_crit*q_l_sc_crit/2 - p_4) / q_l_sc_crit^3;
    p_1 = p_4 + c_2 * q_l_sc^2 + c_3 * q_l_sc^3;
end