function p_out = choke_oil(C_d,d_ch,dir_p,oil,p_in,q_sc,rho_sc,T)
% function p_out = choke_oil(C_d,d_ch,dir_p,oil,p_in,q_sc,rho_sc,T)
%
% Computes the pressure drop for single-phase oil flow through a choke. If the input pressure
% p_in is taken as the upstream pressure p_1, the output pressure p_out is equal to the
% downstream pressure p_4. Conversely, if p_in is taken as the downstream pressure p_4, p_out
% is equal to the upstream pressure p_1. The the sign of the flow rate q_sc determines the
% choice between injection or production: positive means injection; negative means production.
% The sign of parameter dir_p determines the computation direction: +1 means from the separator
% to the reservoir; -1 means from the reservoir to the separator.
%
% C_d = discharge coefficient, -
% d_ch = choke diameter, m
% oil = parameter to select black oil model or volatile oil table, -  
%   oil = 1: black oil; parameters computed with the aid of Standing correlations
%   oil = 2: black oil; parameters computed with the aid of Glaso correlations
%   oil = 3: volatile oil; parameters read from file 'vol_oil_table_01'
% p_in = input pressure, Pa
% dir_p = flag for computation direction (note: may be different from flow direction)
%   dir_p = 1 : from separator to reservoir
%   dir_p = -1: from reservoir to separator
% p_out = ouput pressure, Pa
% q_sc = [q_g_sc,q_o_sc,q_w_sc], m^3/s
%   q_g_sc = gas flow rate at standard conditions, m^3/s. 
%   q_o_sc = oil flow rate at standard conditions, m^3/s. 
%   q_w_sc = water flow rate at standard conditions, m^3/s. Not relevant.
% rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc], kg/m^3 
%   rho_g_sc = gas density at standard conditions, kg/m^3.
%   rho_o_sc = oil density at standard conditions, kg/m^3. 
%   rho_w_sc = water density at standard conditions, kg/m^3. Not relevant.
% T = temperature, deg. C

% JDJ, 02-03-05, last revised 10-05-13

% Densities and flow rates at standard conditions:
rho_g_sc = rho_sc(1); % gas density, kg/m^3
rho_o_sc = rho_sc(2); % oil density, kg/m^3
q_g_sc = q_sc(1); % gas flow rate, m^3/s
q_o_sc = q_sc(2); % oil flow rate, m^3/s
q_w_sc = q_sc(3); % water flow rate, m^3/s

% Check input:
if (q_w_sc ~= 0)
    error('Nonzero water rate. Single-phase oil solution not valid.')
end

% Compute local oil properties:
R_go = q_g_sc/q_o_sc; % producing GOR as observed at surface, m^3/m^3
R_sb = R_go; % bubble point GOR for the oil in the choke. 
[q,rho] = local_q_and_rho(oil,p_in,q_sc,R_sb,rho_sc,T);
rho_o = rho(2); % local oil density, kg/m^3
q_o = q(2); % local oil rate, m^2/s

% Compute pressure drop:
A_3 = pi*d_ch^2/4; % choke throat surface area, m^2 
Delta_p = (rho_o*q_o^2)/(2*C_d^2*A_3^2);
p_out = p_in - sign(q_o)*sign(dir_p)*Delta_p;
    
% Check if pressure is above bubble point pressure: 
switch oil
    case 1
        p_b = pres_bub_Standing(R_go,rho_g_sc,rho_o_sc,T);
    case 2
        p_b = pres_bub_Glaso(R_go,rho_g_sc,rho_o_sc,T);
    case 3
        p_b = pres_bub_volatile_oil(T);
end
if min(p_in,p_out) < p_b
    error('Pressure below bubble point pressure. Use of choke_oil not valid.')
end
