% Script file example_choke_performance_curve
%
% Creates a choke performance curve.
%
% JDJ, 12-02-12, last revised 25-05-17

clear variables
close all

% ---------------------------------------------------------------------------------------------
% Input data:
% ---------------------------------------------------------------------------------------------
fluid = 3; % fluid = 1: single-phase oil flow
%            fluid = 2: single-phase gas flow
%            fluid >= 3: multi-phase gas-oil-water flow
f_w_sc = 0.0; % = q_w_sc/(q_w_sc+q_o_sc) = water cut, -. Not relevant for fluids 1 and 2
oil = 1; % parameter to select black oil model or volatile oil table, -. Not relevant for
%          fluids 1 and 2
%          oil = 1: black oil; parameters computed with the aid of Standing correlations
%          oil = 2: black oil; parameters computed with the aid of Glaso correlations
%          oil = 3: volatile oil; parameters read from file 'vol_oil_table_01'
%          Note: When using a volatile oil table, the input parameters R_sb and rho_sc should
%          be consistent with those used to generate the tabulated values. For
%          vol_oil_table_01: rho_g_sc = 0.80 kg/m^3, rho_o_sc = 800 kg/m^3, R_sb = 450 m^3/m^3.  
d = from_in_to_m(2.453); % flowline or tubing diameter
d_ch = from_in_to_m(16/64); % choke throat diameter, m
C_d = 0.7; % discharge coefficient, -
c_l = 0.80; % specific heat capacity of liquid at constant volume, J/(kg K)
c_p = 0.25; % specific heat capacity of gas at constant pressure, J/(kg K)
c_v = 0.20; % specific heat capacity of gas at constant volume, J/(kg K)
n_pt = 100; % number of points in plot, -
p_fl = 1.4e6; % downstream pressure, Pa; Use this value for multi-phase flow. 
%p_fl = 15e6; % downstream pressure, Pa; Use this value for single-phase oil and gas flow. 
q_g_sc_max = -2; % maximum gas rate, m^3/s. Not relevant for fluids 1 and 3
q_o_sc_max = -2.0e-3; % maximum oil rate, m^3/s. Not relevant for fluid 2
R_go = 50; % producing GOR as observed at surface, m^3/m^3. Not relevant for fluid 2
rho_g_sc = 0.95; % gas density at standard conditions, kg/m^3
rho_o_sc = 850;  % oil density at st. conditions, kg/m^3. Not relevant for fluid 2
rho_w_sc = 1000; % water density at st. conditions, kg/m^3. Not relevant for fluids 1 and 2
T_tf = 30; % FTHT, deg. C
dir_p = 1; % flag for computation direction pressure (may be different from flow direction);
%            only relevant for fluid 1, -  
%            dir_p =  1: from separator to reservoir
%            dir_p = -1: from reservoir to separator
%
% ---------------------------------------------------------------------------------------------
% End of input data
% ---------------------------------------------------------------------------------------------

% Compute auxiliary variables:
gamma = c_p/c_v; % ratio of specific heat capacities, -
rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc];
p_tf = p_fl;
R_sb = R_go; % solution gas-oil ratio at bubble point of oil in choke, m^3/m^3
switch oil
    case 1
        p_b = pres_bub_Standing(R_sb,rho_g_sc,rho_o_sc,T_tf); % bubble point pressure, Pa
    case 2
        p_b = pres_bub_Glaso(R_sb,rho_g_sc,rho_o_sc,T_tf);
    case 3
        p_b = pres_bub_volatile_oil(T_tf);
end

% Compute and plot intake curve:
switch fluid
    case 1 % oil
        delta_q_o_sc = q_o_sc_max/n_pt; % oil rate increment, m^3/s
        results = zeros(n_pt,2);
        for i=1:n_pt
            q_o_sc = i * delta_q_o_sc; % oil rate, m^3/s
            q_g_sc = R_go * q_o_sc; % gas rate, m^3/s
            q_w_sc = 0; % water rate, m^3/s
            q_sc = [q_g_sc,q_o_sc,q_w_sc];
            p_tf = choke_oil(C_d,d_ch,dir_p,oil,p_fl,q_sc,rho_sc,T_tf);
            results(i,1:2) = [-q_o_sc p_tf];
        end
        plot(results(:,1)*1e3,results(:,2)/1e6,'LineWidth',1)
        xlabel('Oil Flow Rate,\it q_{o,sc}\rm (10^{-3} m^3/s)')
        ylabel('Upstream Pressure,\it p_{\rm1}\rm (MPa)')
        grid on
    case 2 % gas
        r_crit = (2/(1+gamma))^(gamma/(gamma-1)); % critical pressure ratio (p_4/p_1)_crit, -
        delta_q_g_sc = q_g_sc_max/n_pt; % gas rate increment, m^3/s
        results = zeros(n_pt,3);
        for i=1:n_pt
            q_g_sc = i * delta_q_g_sc; % gas rate, m^3/s
            q_sc = [q_g_sc,0,0];
            p_tf = choke_gas(C_d,d_ch,gamma,p_fl,q_sc,rho_sc,T_tf);
            results(i,1:3)= [-q_g_sc p_tf (1/r_crit)*p_fl];
        end
        plot(results(:,1),results(:,2)/1e6,'LineWidth',1)
        hold on
        plot(results(:,1),results(:,3)/1e6,'--r','LineWidth',1)
        hold off
        xlabel('Gas Flow Rate,\it -q_{g,sc}\rm (m^3/s)')
        ylabel('Upstream Pressure,\it p_{\rm1}\rm (MPa)')
        grid on
        legend('Choke performance curve','Critical flow boundary','Location','NorthWest')
    otherwise % multi-phase
        delta_q_o_sc = q_o_sc_max/n_pt; % oil rate increment, m^3/s
        results = zeros(n_pt,3);
        for i=1:n_pt
            q_o_sc = i * delta_q_o_sc; % oil rate, m^3/s
            q_g_sc = R_go * q_o_sc; % gas rate, m^3/s
            q_w_sc = (f_w_sc/(1-f_w_sc)) * q_o_sc; % water rate, m^3/s
            q_sc = [q_g_sc,q_o_sc,q_w_sc];
            p_old = p_tf;
            [p_tf,r_crit] = choke_multi_phase(C_d,c_l,c_p,c_v,d_ch,oil,p_fl,q_sc,rho_sc,T_tf);
            results(i,1:3)= [-q_o_sc p_tf (1/r_crit)*p_fl];
            Delta_p = p_old - p_tf;
            if p_tf > p_b + Delta_p % to avoid pressures above bubble point
                break
            end
        end
        nonzero_rows = results(:,1) ~= 0; % select non-zero rows
        results = results(nonzero_rows,:); % remove zero rows
        plot(results(:,1)*1e3,results(:,2)/1e6,'LineWidth',1)
        hold on
        plot(results(:,1)*1e3,results(:,3)/1e6,'--r','LineWidth',1)
        hold off
        xlabel('Oil Flow Rate,\it q_{o,sc}\rm (10^{-3} m^3/s)')
        ylabel('Upstream Pressure,\it p_{\rm1}\rm (MPa)')
        grid on
        legend('Choke performance curve','Critical flow boundary','location','NorthWest')
end
