function results_ds = system_01_downstream_GUI(p_sep)
% function results_ds = system_01_downstream_GUI(p_sep)
%
% Computes the pressure drop downstream of the analysis node through a simple system consisting
% of a near-well bore area, a tubing, a choke and a flow line. The analysis node is at the top
% of the tubing. 
%
% Modified version of system_01.m adapted to run with GUI "nodal".
%
% JDJ, 23-10-12, last revised 06-02-15

% ---------------------------------------------------------------------------------------------
% Input data not provided through the GUI:
% ---------------------------------------------------------------------------------------------

alpha_fl = from_deg_to_rad(90); % flow line inclination, rad
choke_model = 1; % Gilbert (1954)
d_fl = 0.10053; % inside diameter of a 4.5 inch flow line, m
d_ch = from_in_to_m(40/64); % inside diameter of a 40/64 inch choke, m
e_fl = 30e-6; % flow line roughness, m
f_w = 0.0; % water cut, - 
fluid = 4; % multi-phase flow using the Mukherjee and Brill correlation
l_fl = 500; % flow line length, m
n_pt = 50; % number of points in plot, - 
oil = 1; % black oil; parameters computed with the aid of Standing correlations
q_o_sc_max = -2.5e-3; % maximum oil rate, m^3/s. 
R_sb = 50; % solution GOR at bubble point, m^3/m^3 
rho_g_sc = 0.95; % gas density at standard conditions, kg/m^3
rho_o_sc = 850;  % oil density at standard conditions, kg/m^3
rho_w_sc = 1000; % water density at standard conditions, kg/m^3
T_sep = 60; % separator temperature, deg. C
T_tf = 70; % tubing head temperature, deg. C

% ---------------------------------------------------------------------------------------------
% End of input data
% ---------------------------------------------------------------------------------------------

% Compute auxiliary variables:
R_go = R_sb; % no free gas 
rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc];
delta_q_o_sc = q_o_sc_max/n_pt; % oil rate increment, m^3/s

% Initialize output:
results_ds = zeros(n_pt,2);

for i=1:n_pt
    q_o_sc = i * delta_q_o_sc; % oil rate, m^3/s 
    q_g_sc = R_go * q_o_sc; % gas rate, m^3/s
    q_w_sc = (f_w/(1-f_w)) * q_o_sc; % water rate, m^3/s
    q_sc = [q_g_sc,q_o_sc,q_w_sc];
    
    % Compute downstream performance curve:
    % 1) Compute pressure drop over the flow line:
    p_fl = pipe(alpha_fl,d_fl,e_fl,fluid,oil,p_sep,q_sc,rho_sc,0,l_fl,T_sep,T_tf); % FLP  
    % 2) Compute pressure drop over the choke: 
    p_tf_ds = choke_multi_phase_simp(choke_model,d_ch,p_fl,q_sc); % THP
    results_ds(i,1)= -q_o_sc; % flow rate
    results_ds(i,2)= p_tf_ds; % downstream pressure 
end
