function results_us = system_01_upstream_GUI(p_R)
% function results_us = system_01_upstream_GUI(p_R)
%
% Computes the pressure drop upstream of the analysis node through a simple system consisting
% of a near-well bore area, a tubing, a choke and a flow line. The analysis node is at the top
% of the tubing. 
%
% Modified version of system_01.m adapted to run with GUI "nodal".
%
% JDJ, 23-10-12, last revised 06-02-15
% ---------------------------------------------------------------------------------------------
% Input data not provided through the GUI:
% ---------------------------------------------------------------------------------------------

alpha_w = 0; % wellbore inclination, rad
av = 0; % use reservoir pressure at external boundary
d_w = 0.07600; % inside diameter of a 3.5 inch, 9.30 lbs/ft tubing, m
e_w = 30e-6; % tubing roughness, m
f_w = 0.0; % water cut, - 
fluid = 4; % multi-phase flow using the Mukherjee and Brill correlation 
h = 20; % reservoir hight, m
k = 0.6e-13; % permeability, m^2
k_ro_0 = 0.9; % end-point relative permeability to oil, -
n_pt = 50; % number of points in plot, - 
oil = 1; % black oil; parameters computed with the aid of Standing correlations
q_o_sc_max = -2.5e-3; % maximum oil rate, m^3/s. 
r_e = 800; % radius external drainage boundary, m
r_w = 0.1778; % wellbore radius, m
R_sb = 50; % solution GOR at bubble point, m^3/m^3 
rho_g_sc = 0.95; % gas density at standard conditions, kg/m^3
rho_o_sc = 850;  % oil density at standard conditions, kg/m^3
rho_w_sc = 1000; % water density at standard conditions, kg/m^3
S = 0; % skin, -
semi = 0; % use steady-state conditions
s_tot = 3000; % total along-hole well depth, m 
T_tf = 70; % tubing head temperature, deg. C
T_wf = 120; % bottomhole temperature, deg. C

% ---------------------------------------------------------------------------------------------
% End of input data
% ---------------------------------------------------------------------------------------------

% Compute auxiliary variables:
R_go = R_sb; % no free gas 
k_o = k * k_ro_0; % effective permeability to oil
rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc];
delta_q_o_sc = q_o_sc_max/n_pt; % oil rate increment, m^3/s

% Initialize output:
results_us = zeros(n_pt,2);

for i=1:n_pt
    q_o_sc = i * delta_q_o_sc; % oil rate, m^3/s 
    q_g_sc = R_go * q_o_sc; % gas rate, m^3/s
    q_w_sc = (f_w/(1-f_w)) * q_o_sc; % water rate, m^3/s
    q_sc = [q_g_sc,q_o_sc,q_w_sc];
    
    % Compute upstream performance curve:
    % 1) Compute pressure drop over the reservoir:
    p_wf = res_oil_simp(av,h,k_o,oil,p_R,q_sc,r_e,r_w,rho_sc,semi,S,T_wf); % BHP  
    % 2) Compute pressure drop over the wellbore: 
    p_tf_us = pipe(alpha_w,d_w,e_w,fluid,oil,p_wf,q_sc,rho_sc,s_tot,0,T_wf,T_tf); % THP
    results_us(i,1)= -q_o_sc; % flow rate
    results_us(i,2)= p_tf_us; % upstream pressure
end
