% Script file Eclipse_lift_table_deviated
%
% Identical to Eclipse_lift_table.m but now with survey file input. 
%
% Computes a lift table in Eclipse format for a deviated multi-phase gas-oil-water well, using
% the Hagedorn and Brown (1965), Mukherjee_and_Brill (1985) or Beggs and Brill (1973) pressure
% drop correlation.   
%
% Has the possibility to "regularize" the curves by replacing the part to the left of the
% minimum FBHP (the "severe slugging regime") by a horizontal line. Regularization may avoid
% convergence problems in the reservoir simulator.    
%
% Note: All flow rates have negative values, because the flow is from bottom to surface,
% whereas the positive s co-ordinate runs from surface to bottom.   
%
% JDJ, 20-10-03, last revised 26-05-17

clear variables
close all

% ---------------------------------------------------------------------------------------------
% Input data:
% ---------------------------------------------------------------------------------------------

% Lift table number:
lift_tab_nr = 1;  

% Well and fluid property input in SI units:
fid = fopen('deviated_well_01.txt'); % survey file identifier
d = 0.13; % tubing inside diameter, m 
e = 3e-5; % tubing roughness, m
fluid = 4; % fluid = 1: single-phase oil flow
%           fluid = 2: single-phase gas flow
%           fluid = 3: multi-phase gas-oil-water flow, Hagedorn and Brown correlation
%           fluid = 4: multi-phase gas-oil-water flow, Mukherjee and Brill correlation
%           fluid = 5: multi-phase gas-oil-water flow, Beggs and Brill correlation     
%           fluid = 6: multi-phase gas-oil-water flow, Shi et al. drift flux model (exercise)
oil = 1; % parameter to select black oil model or volatile oil table, -. Not relevant for
%          fluids 1 and 2
%          oil = 1: black oil; parameters computed with the aid of Standing correlations
%          oil = 2: black oil; parameters computed with the aid of Glaso correlations
%          oil = 3: volatile oil; parameters read from file 'vol_oil_table_01'
%          Note: When using a volatile oil table, the input parameters R_sb and rho_sc should
%          be consistent with those used to generate the tabulated values. For
%          vol_oil_table_01: rho_g_sc = 0.80 kg/m^3, rho_o_sc = 800 kg/m^3, R_sb = 450 m^3/m^3. 
rho_g_sc = 0.98; % gas density at standard conditions, kg/m^3
rho_o_sc = 735; % oil density at standard conditions, kg/m^3
rho_w_sc = 1066; % water density at standard conditions, kg/m^3
T_wf = 76; % bottomhole temperature, deg. C
T_tf = 25; % tubing head temperature, deg. C

% Flag to indicate if regularization is to be used:
regularize = 1; % 0 = no, 1 = yes

% Lift table input in 'allowable' SI units:
q_o_sc_ASI = [-100 -300 -600 -900 -1200]; % oil rate, m^3/d; maximum 6 flow
%                                           rates 
% Note: negative value, because oil flows from bottom to surface, whereas
% positive s co-ordinate runs from surface to bottom.
R_go = [10 50 100]; % producing gas-oil ratio, m^3/m^3
f_w_sc = [0.0 0.5 0.9]; % water cut (note: expressed as fraction), -
p_tf_ASI = [4000]; % tubing head pressure, kPa. 
z_tvd = 2500; % true-vertical depth, m

% ---------------------------------------------------------------------------------------------
% End of input data
% ---------------------------------------------------------------------------------------------
% Retrieve data from survey file:
[ahd,inc,azi,nor,eas,tvd] = evaluate_survey(fid);

% Create matrix with AHD and inclination data: 
survey = [ahd,inc];

% Retrieve total well length:
n_sur = length(survey);
s_tot = survey(n_sur,1); % total along-hole well depth, m

% Compute auxiliary variables and convert to SI units:
p_tf = 1000 * p_tf_ASI; % flowing tubing head pressure, Pa
q_o_sc = q_o_sc_ASI/(24*3600); % oil rate, m^3/s

rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc];
n_f_w_sc = length(f_w_sc);
n_q_o_sc = length(q_o_sc);
n_R_go = length(R_go);
n_p_tf = length(p_tf);

fid = fopen('Eclipse_lift_table.txt','w');
fprintf(fid,'VFPPROD\n');
fprintf(fid,' %2.0f',lift_tab_nr);
fprintf(fid,'    %13.5E   ',s_tot);
fprintf(fid,'\''OIL''     ');
fprintf(fid,'\''WCT''     ');
fprintf(fid,'\''GOR''     ');
% fprintf(fid,'\'' ''     ');
% fprintf(fid,'\''METRIC''     ');
% fprintf(fid,'\''BHP''     ');
fprintf(fid,'\n/\n');
for i=1:n_q_o_sc
    fprintf(fid,' %13.5E',-q_o_sc(i)*24*3600); % m^3/d
end  
fprintf(fid,'\n/\n');
for m=1:n_p_tf
    fprintf(fid,' %13.5E',p_tf(m)/1e5); % bar
end
fprintf(fid,'\n/\n');
for k=1:n_f_w_sc
    fprintf(fid,' %13.5E',f_w_sc(k)); % m^3/m^3
end   
fprintf(fid,'\n/\n');
for j=1:n_R_go
    fprintf(fid,' %13.5E',R_go(j)); % m^3/m^3
end   
fprintf(fid,'\n/\n');
dummy = 0.0;
fprintf(fid,' %13.5E',dummy);
fprintf(fid,'\n/\n');

% Compute FBHP values:
data = zeros(n_q_o_sc,n_R_go,n_f_w_sc,n_p_tf);
p_wf = zeros(n_p_tf);
for i=1:n_q_o_sc
    for j=1:n_R_go
        for k=1:n_f_w_sc
            for m=1:n_p_tf
                [i j k m]
                q_g_sc = R_go(j) * q_o_sc(i);
                q_w_sc = (f_w_sc(k)/(1-f_w_sc(k))) * q_o_sc(i);
                q_sc = [q_g_sc,q_o_sc(i),q_w_sc];
                p_wf(m) = pipe(survey,d,e,fluid,oil,p_tf(m),q_sc,rho_sc,0,...
                    s_tot,T_tf,T_wf);
                data(i,j,k,m)= p_wf(m);
            end
        end
    end
end

% Regularize:
if regularize == 1
    for i=n_q_o_sc-1:-1:1
        for j=1:n_R_go
            for k=1:n_f_w_sc
                for m=1:n_p_tf
                    if data(i,j,k,m) > data(i+1,j,k,m)
                        data(i,j,k,m) = data(i+1,j,k,m);
                    end
                end
            end
        end
    end
end

% Print to file:
for m=1:n_p_tf
    for k=1:n_f_w_sc
        for j=1:n_R_go
            fprintf(fid,'  %1.0f  ', m);
            fprintf(fid,  '%1.0f  ', k);
            fprintf(fid,  '%1.0f  ', j);
            dummy = 1;  
            fprintf(fid,  '%1.0f  ', dummy);
            for i=1:n_q_o_sc
                p_wf(i) = data(i,j,k,m);
%                fprintf(fid,'%13.5E ', p_wf(i)/1000); % old version
                fprintf(fid,'%13.5E ', p_wf(i)/100000); % new version
            end
            fprintf(fid,'\n/\n');
        end    
    end
end
status = fclose(fid);

% Plot to screen:
subplot(1,2,1); % lowest FTHP
for k=1:n_f_w_sc
    for j=1:n_R_go
%        plot(-q_o_sc(:)*24*3600,data(:,j,k,1)/1e6,'Color','k'); hold on
        plot(-q_o_sc(:)*24*3600,data(:,j,k,1)/1e6); hold on
    end
end
xlabel('Oil Flow Rate,\it -q_{o,sc}\rm (m^3/d)')
ylabel('FBHP,\it p_{wf}\rm (MPa)')
grid on

subplot(1,2,2); % highest FTHP
for k=1:n_f_w_sc
    for j=1:n_R_go
%        plot(-q_o_sc(:)*24*3600,data(:,j,k,n_p_tf)/1e6,'Color','k'); hold on
        plot(-q_o_sc(:)*24*3600,data(:,j,k,n_p_tf)/1e6); hold on
    end
end
xlabel('Oil Flow Rate,\it -q_{o,sc}\rm (m^3/d)')
ylabel('FBHP,\it p_{wf}\rm (MPa)')
grid on
