% Script file Imex_lift_table
%
% Computes a lift table in Eclipse format for a deviated multi-phase gas-oil-water well, using
% the Hagedorn and Brown (1965), Mukherjee_and_Brill (1985) or Beggs and Brill (1973) pressure
% drop correlation.   
%
% Has the possibility to "regularize" the curves by replacing the part to the left of the
% minimum FBHP (the "severe slugging regime") by a horizontal line. Regularization may avoid
% convergence problems in the reservoir simulator.    
%
% Note: All flow rates have negative values, because the flow is from bottom to surface,
% whereas the positive s co-ordinate runs from surface to bottom.   
%
% JDJ, 20-10-03, last revised 26-05-17

clear variables
close all

% ---------------------------------------------------------------------------------------------
% Input data:
% ---------------------------------------------------------------------------------------------

% Well and fluid property input in SI units:
alpha_deg = 0; % wellbore inclination with respect to vertical, degrees
d = 0.1397; % tubing inside diameter, m 
e = 3e-5; % tubing roughness, m
fluid = 4; % fluid = 1: single-phase oil flow
%           fluid = 2: single-phase gas flow
%           fluid = 3: multi-phase gas-oil-water flow, Hagedorn and Brown correlation
%           fluid = 4: multi-phase gas-oil-water flow, Mukherjee and Brill correlation
%           fluid = 5: multi-phase gas-oil-water flow, Beggs and Brill correlation     
%           fluid = 6: multi-phase gas-oil-water flow, Shi et al. drift flux model (exercise)
oil = 1; % parameter to select black oil model or volatile oil table, -. Not relevant for
%          fluids 1 and 2
%          oil = 1: black oil; parameters computed with the aid of Standing correlations
%          oil = 2: black oil; parameters computed with the aid of Glaso correlations
%          oil = 3: volatile oil; parameters read from file 'vol_oil_table_01'
%          Note: When using a volatile oil table, the input parameters R_sb and rho_sc should
%          be consistent with those used to generate the tabulated values. For
%          vol_oil_table_01: rho_g_sc = 0.80 kg/m^3, rho_o_sc = 800 kg/m^3, R_sb = 450 m^3/m^3. 
rho_g_sc = 0.98; % gas density at standard conditions, kg/m^3
rho_o_sc = 876; % oil density at standard conditions, kg/m^3
rho_w_sc = 1030; % water density at standard conditions, kg/m^3
T_wf = 118; % bottomhole temperature, deg. C
T_tf = 43; % tubing head temperature, deg. C

% Flag to indicate if regularization is to be used:
regularize = 1; % 0 = no, 1 = yes

% Lift table input in 'allowable' SI units:
q_o_sc_ASI = [-100 -500 -900 -1300 -1700 -2100 -2500]; % oil rate, m^3/d
% Note: negative value, because oil flows from bottom to surface, whereas
% positive s co-ordinate runs from surface to bottom.
R_go = [20 60 100]; % producing gas-oil ratio, m^3/m^3
f_w_sc = [0.0 0.4 0.8]; % water cut (note: expressed as fraction), -
% p_tf_ASI = [500 1000]; % tubing head pressure, kPa. Note: always put in TWO
% % pressures. No more, no less!
p_tf_ASI = [500 5000 10000]; % tubing head pressure, kPa. 
z_tvd = 2500; % true-vertical depth, m

% -------------------------------------------------------------------------
% End of input data
% -------------------------------------------------------------------------

% Compute auxiliary variables and convert to SI units:
alpha = from_deg_to_rad(alpha_deg); % wellbore inclination , rad
p_tf = 1000 * p_tf_ASI; % flowing tubing head pressure, Pa
q_o_sc = q_o_sc_ASI/(24*3600); % oil rate, m^3/s
s_tot = z_tvd/cos(alpha); % total along-hole well depth, m 

rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc];
n_f_w_sc = length(f_w_sc);
n_q_o_sc = length(q_o_sc);
n_R_go = length(R_go);

fid = fopen('Imex_lift_table.txt','w');
fprintf(fid,'*PTUBE');
fprintf(fid,' *LIQ 1\n');
fprintf(fid,'*DEPTH');
fprintf(fid,' %6.2f\n',s_tot);
fprintf(fid,'*GOR');
for k=1:n_R_go
    fprintf(fid,' %6.2f',R_go(k));
end   
fprintf(fid,'\n*WCUT');
for j=1:n_f_w_sc
    fprintf(fid,' %6.4f',f_w_sc(j));
end   
fprintf(fid,'\n*QO');
for i=1:n_q_o_sc
    fprintf(fid,' %6.2f',-q_o_sc(i)*24*3600);
end  
fprintf(fid,'\n*WHP');
for m=1:2
    fprintf(fid,' %6.1f',p_tf(m)/1000);
end
fprintf(fid,'\n*BHPTO\n');

% Compute FBHP values:
data = zeros(n_q_o_sc,n_f_w_sc,n_R_go,3);
for i=1:n_q_o_sc
    for j=1:n_f_w_sc
        for k=1:n_R_go
            [i j k]
            q_g_sc = R_go(k) * q_o_sc(i);
            q_w_sc = (f_w_sc(j)/(1-f_w_sc(j))) * q_o_sc(i);
            q_sc = [q_g_sc,q_o_sc(i),q_w_sc];
            p_wf(1) = pipe(alpha,d,e,fluid,oil,p_tf(1),q_sc,rho_sc,0,s_tot,T_tf,T_wf);
            p_wf(2) = pipe(alpha,d,e,fluid,oil,p_tf(2),q_sc,rho_sc,0,s_tot,T_tf,T_wf);
            data(i,j,k,1:3)= [-q_o_sc(i) p_wf(1) p_wf(2)];
        end
    end
end

% Regularize:
if regularize == 1
    for i=n_q_o_sc-1:-1:1
        for j=1:n_f_w_sc
            for k=1:n_R_go
                if data(i,j,k,2) > data(i+1,j,k,2)
                    data(i,j,k,2) = data(i+1,j,k,2);
                end
                if data(i,j,k,3) > data(i+1,j,k,3)
                    data(i,j,k,3) = data(i+1,j,k,3);
                end
            end
        end
    end
end

% Print to file:
for i=1:n_q_o_sc
    for j=1:n_f_w_sc
        for k=1:n_R_go
            p_wf(1) = data(i,j,k,2);
            p_wf(2) = data(i,j,k,2);
            fprintf(fid,'%1.0f ', k);
            fprintf(fid,'%1.0f ', j);
            fprintf(fid,'%1.0f ', i);
            fprintf(fid,'%6.1f ', p_wf(1)/1000);
            fprintf(fid,'%6.1f \n', p_wf(2)/1000);
        end
    end
end
status = fclose(fid);

% Plot to screen:
subplot(1,2,1);
for j=1:n_f_w_sc
    for k=1:n_R_go
        plot(data(:,j,k,1)*24*3600,data(:,j,k,2)/1e6); hold on
    end
end
xlabel('Oil Flow Rate,\it -q_{o,sc}\rm (m^3/d)')
ylabel('FBHP,\it p_{wf}\rm (MPa)')
grid on

subplot(1,2,2);
for j=1:n_f_w_sc
    for k=1:n_R_go
        plot(data(:,j,k,1)*24*3600,data(:,j,k,3)/1e6); hold on
    end
end
xlabel('Oil Flow Rate,\it -q_{o,sc}\rm (m^3/d)')
ylabel('FBHP,\it p_{wf}\rm (MPa)')
grid on
