function [H_g,H_l,lambda_l,lambda_g] = Hag_Brown_hold_up(N_d,N_gv,N_lv,N_mu,p,q_g,q_l)
% function [H_g,H_l,lambda_l,lambda_g] = Hag_Brown_hold_up(N_d,N_gv,N_lv,N_mu,p,q_g,q_l)   
%
% Computes the gas and liquid hold-ups and in-situ volume fractions for a given pressure p and
% along-hole distance s, in an element of a flowline-wellbore system.
%
% Uses the Hagedorn and Brown correlation for multiphase flow in (near-)vertical wells; see
% references [1] and [2]. A reality check has been added to ensure that the computed liquid
% hold-up (for flow with slip) is never smaller than the in-situ liquid volume fraction (the
% 'no-slip hold-up'). 
%
% H_g = gas hold-up, -
% H_l = liquid hold-up, -
% lambda_g = in-situ gas volume fraction (no-slip), -
% lambda_l = in-situ liquid volume fraction (no-slip), -
% N_d = pipe diameter number, -
% N_gv = gas velocity number, -
% N_lv = liquid velocity number, -
% N_mu = liquid viscosity number, -
% p = pressure, Pa
% q_g = local gas flow rate, m^3/s
% q_l = local liquid flow rate, m^3/s
%
% JDJ, 30-10-11, last revised 10-05-13
%
% References:
% [1] Hagedorn, A.R. and Brown, K.E., 1965: Experimental study of pressure gradients occurring
% during continuous two-phase flow in small-diameter vertical conduits. Journal of Petroleum
% Technology 17 (4) 475-484.   
% [2] Brill, J.P. and Mukherjee, H., 1999: Multiphase flow in wells, SPE Monograph Series, vol
% 17., SPE, Richardson. 

% Compute internal variables:
p_sc = 1e5; % pressure at standard conditons, Pa

% Check for free gas:
if abs(q_g) < 1.e-12 % no free gas - liquid flow only
    H_l = 1;
    H_g = 0;
    lambda_l = 1;
    lambda_g = 0;
else % gas-liquid flow
    % Compute auxilary parameters (curve fitted from original charts), and liquid hold-up:
    % Chart 1:
    x1_min = 0.002;
    x1_max = 0.5;
    x1 = N_mu;
    if x1 < x1_min | x1 > x1_max
        x1_min
        x1
        x1_max
        error('Parameter x1 out of bounds. Hagedorn and Brown correlation not valid.')
    end
    a0 =  1.9122E-03;
    a1 =  5.6348E-02;
    a2 = -1.2183E-01;
    a3 =  9.0950E-02;
    y1 = a3*x1^3 + a2*x1^2 + a1*x1 + a0;
    N_mu_c = y1; % corrected liquid viscosity number, -

    % Chart 2:
    x2_min = 0.0;
    x2_max = 0.088;
    x2 = N_gv*(N_mu^0.380)/(N_d^2.14);
    if x2 < x2_min | x2 > x2_max
        x2_min
        x2
        x2_max
        error('Parameter x2 out of bounds. Hagedorn and Brown correlation not valid.')
    end
    b0 =  1.8089E-05;
    b1 = -9.9787E-04;
    b2 =  1.0029E-02;
    b3 =  3.6917E-01;
    c0 =  1.9585E-05;
    c1 = -1.2769E-03;
    c2 =  2.7058E-02;
    c3 = -4.8940E-02;
    c4 =  1.0000E+00;
    if x2 > 0.01
        y2 = (b3*x2^3 + b2*x2^2 + b1*x2 + b0) / (c4*x2^4 + c3*x2^3 + c2*x2^2 + c1*x2 + c0);
    else
        y2 = 1;
    end
    psi = y2; % correction factor, -

    % Chart 3:
    x3_min = 0.0000021;
    x3_max = 0.005;
    x3 = (N_lv/(N_gv^0.575))*(p/p_sc)^0.10*(N_mu_c/N_d);
    if x3 < x3_min
        x3_min
        x3
        error('Parameter x3 out of bounds. Hagedorn and Brown correlation not valid.')
    end
    d0 =  4.3260E-11;
    d1 = -3.7902E-05;
    e0 = -3.7520E-10;
    e1 = -1.9323E-04;
    e2 =  1.0000E+00;
    f0 =  6.4898E-05;
    f1 =  1.1470E+00;
    f2 = -1.8530E+01;
    g0 = 4.5319E-04;
    g1 = 1.0000E+00;
    if x3 <= 4.64229E-05
        y3 = (d1*x3 + d0) / (e2*x3^2 + e1*x3 + e0);
    else if x3 < x3_max
            y3 = (f2*x3^2 + f1*x3 + f0) / (g1*x3 + g0);
        else
            y3 = 0.98; % highest hold-up in graph
        end
    end
    H_l = y3 * psi; % liquid hold-up, -

    % Compute liquid and gas volume fractions (no-slip):
    lambda_l = q_l/(q_l+q_g); % liquid volume fraction, -
    lambda_g = 1-lambda_l; % gas volume fraction, -

    % Perform reality check:
    if H_l < lambda_l
        H_l = lambda_l;
    end
    H_g = 1-H_l; % gas hold-up, -
end
