function dpds = Muk_Brill_dpds(s,p,~,alpha,d,e,oil,q_sc,rho_sc,s_in,s_out,T_in,T_out)
% function dpds = Muk_Brill_dpds(s,p,~,alpha,d,e,oil,q_sc,rho_sc,s_in,s_out,T_in,T_out)   
%
% Computes the derivative dp/ds for a given pressure p and along-hole distance s, in an element
% of a flowline-wellbore system. The distance s is measured from the separator towards the
% reservoir. Therefore, flowrates are negative for production wells.
%
% Uses the Mukherjee and Brill correlations for liquid hold-up and pressure drop in inclined
% wells; see references [1] to [3]. The flow regime is computed in an auxiliary function
% Muk_Brill_flow_reg. A reality check has been added to ensure that the computed liquid 
% hold-up (for flow with slip) is never smaller than the in-situ liquid volume fraction (the
% 'no-slip hold-up'). 
% 
% The vector p contains the total pressure, and the pressures taking into account the
% individual effects of gravity, friction and acceleration losses respectively. Accordingly,
% the vector dpds contains the total pressure loss per unit length, as well as the individual
% gravity losses, friction losses and acceleration losses.  
%
% This function can be used to compute the pressure drop through numerical integration. It has
% the correct format to be used in conjunction with one of the standard numerical integration
% routines in MATLAB.   
%
% alpha = inclination wrt. vertical, rad; alternatively alpha can be a survey file (matrix)
%         with AHD values in the first column (in m) and inclination values in the second
%         column (in rad).   
% d = inside diameter, m
% dpds = [dpds_tot;dpds_grav;dpds_fric;dpds_acc]
%   dpds_acc  =  pressure gradient due to acceleration losses, Pa/m
%   dpds_fric =  pressure gradient due to friction losses, Pa/m 
%   dpds_grav = pressure gradient due to head losses, Pa/m 
%   dpds_tot  = dpds_grav + dpds_fric + dpds_acc = total pressure gradient, Pa/m  
% e = roughness, m
% oil = parameter to select black oil model or volatile oil table, -  
%   oil = 1: black oil; parameters computed with the aid of Standing correlations
%   oil = 2: black oil; parameters computed with the aid of Glaso correlations
%   oil = 3: volatile oil; parameters read from file 'vol_oil_table_01'
% p = [p_tot,p_grav,p_fric,p_acc], Pa
%   p_acc  = p_in + pressure increase (decrease for production wells) due to accel. losses, Pa
%   p_fric = p_in + pressure increase (decrease for production wells) due to frict. losses, Pa
%   p_grav = p_in + pressure increase (decrease for production wells) due to head loss, Pa
%   p_tot  = p_in + pressure increase (decrease for production wells) due to gravity, friction
%            and acceleration losses, Pa 
% p_in = pressure at s_in, Pa 
% p_out = pressure at s_out, Pa
% q_sc = [q_g_sc,q_o_sc,q_w_sc], m^3/s
%   q_g_sc = gas flow rate at standard conditions, m^3/s
%   q_o_sc = oil flow rate at standard conditions, m^3/s
%   q_w_sc = water flow rate at standard conditions, m^3/s
% rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc], kg/m^3 
%   rho_g_sc = gas density at standard conditions, kg/m^3
%   rho_o_sc = oil density at standard conditions, kg/m^3
%   rho_w_sc = water density at standard conditions, kg/m^3
% s = along-hole distance, measured from the separator to the reservoir, m   
% s_in = starting point for the integration
% s_out = end point for the integration
% T_in = temperature at s_in, deg. C 
% T_out = temperature at s_out, deg. C 
%
% JDJ, 04-03-02, last revised 10-05-13
%
% References:
% [1] Mukherjee,H. and Brill, J.P., 1985: Pressure drop correlations for inclined two-phase
%     flow, J. of Energy Resources Technology, 107 (4) 549-554.
% [2] Mukherjee H. and Brill, J.P., 1983: Liquid holdup correlations for inclined two-phase
%     flow. J. of Petroleum Technology, 35 (5) 1003-1008. 
% [3] Brill, J.P. and Mukherjee, H., 1999: Multiphase flow in wells, SPE Monograph Series,
%     vol. 17, SPE, Richardson. 

% Check sign of pressure:
p_tot = p(1); % first element of vector p is the total wellbore pressure, Pa 
if min(p_tot) < 1e5
    warning('Pressure below atmospheric.')
    dpds_tot  = 0; 
    dpds_grav = 0; 
    dpds_fric = 0; 
    dpds_acc  = 0; 
    dpds = [dpds_tot;dpds_grav;dpds_fric;dpds_acc];
    return
end

% Determine inclination in case of survey file input:
if length(alpha) > 1
    n_sur = length(alpha(:,1)); % number of survey points
    if s < alpha(1,1)
        help = alpha(1,2);
    else if s > alpha(n_sur,1)
        help = alpha(n_sur,2);
        else
            help = interp1(alpha(:,1),alpha(:,2),s);
        end
    end
    clear alpha;
    alpha = help; % replace survey file by single inclination value, rad
end

% Compute internal variables:
epsilon = e/d; % dimensionless pipe roughness, -
g = 9.81; % acceleration of gravity, m/s^2

% Compute local gas and liquid properties:
[mu_g,mu_l,q_g,q_l,rho_g,rho_l,sigma_gl,v_sg,v_sl] = local_gas_liq_props(d,oil,p_tot,...
    q_sc,rho_sc,s,s_in,s_out,T_in,T_out);

% Compute Duns and Ros dimensionless numbers:
[~,N_gv,N_lv,N_mu] = Duns_Ros_dim_less_nrs(d,mu_l,rho_l,sigma_gl,v_sg,v_sl);

% Determine flow regime and auxiliary parameters:
[flow_dir,flow_reg,theta_MB] = Muk_Brill_flow_reg(alpha,N_gv,N_lv,N_mu,v_sg,v_sl);

% Compute hold-ups (slip) and in-situ volume fractions (no-slip):
[H_g,H_l,lambda_l,lambda_g] = Muk_Brill_hold_up(flow_dir,flow_reg,N_gv,N_lv,...
    N_mu,q_g,q_l,theta_MB);

if flow_reg == 0 % no free gas - liquid flow only
    
    % Compute pressure gradient for liquid-only flow:
    v_l = v_sl; % local liquid velocity, m/s
    N_Re = rho_l*d*abs(v_l)/mu_l; % Reynolds number, -
    f = Moody_friction_factor(epsilon,N_Re); % friction factor, -
    dpds_grav = rho_l*g*cos(alpha); % gravity losses, Pa/m
    dpds_fric = -rho_l*f*v_l*abs(v_l)/(2*d); % friction losses, Pa/m
    dpds_acc = 0; % acceleration losses are neglegible, Pa/m
    dpds_tot = dpds_grav + dpds_fric + dpds_acc; % total pressure gradient, Pa/m
    dpds = [dpds_tot;dpds_grav;dpds_fric;dpds_acc];
    
else % gas-liquid flow
    
    % Compute 'slip' and 'no-slip' gas-liquid mixture properties:
    mu_mn = mu_l*lambda_l + mu_g*lambda_g; % 'no-slip' gas-liquid mixture viscosity, Pa s 
    rho_mn = rho_l*lambda_l + rho_g*lambda_g; % 'no-slip' gas-liquid mixture  density, kg/m^3
    rho_ms = rho_l*H_l + rho_g*H_g; % 'slip' gas-liquid mixture density, kg/m^3
    v_ms = v_sg + v_sl; % 'slip' mixture velocity, m/s  
    
    % Compute pressure gradient for bubble and slug flow:
    if flow_reg == 1 || flow_reg == 2
        N_Re = rho_mn*abs(v_ms)*d/mu_mn; % gas-liquid mixture Reynolds number, -
        f = Moody_friction_factor(epsilon,N_Re); % gas-liquid mixture friction factor, -
        help21 = rho_ms*v_ms*v_sg/p_tot; % acceleration loss factor E_k, -
        if help21 >= 1 
           error('Choked flow. Reduce rate, increase diameter, or increase back pressure.')
        end
        help22 = rho_ms*g*cos(alpha);
        help23 = -f*rho_ms*v_ms*abs(v_ms)/(2*d); 
        dpds_grav = help22; % gravity losses, Pa/m
        dpds_fric = help23; % friction losses, Pa/m
        dpds_acc = (help22+help23)*(help21/(1-help21)); % acceleration losses, Pa/m
        dpds_tot = dpds_grav + dpds_fric + dpds_acc; % total pressure gradient, Pa/m
        dpds = [dpds_tot;dpds_grav;dpds_fric;dpds_acc];
    end
    
    % Compute pressure gradient for annular flow:
    if flow_reg == 3
        H_r = lambda_l/H_l; % volume fraction - holdup ratio, -
        H_r_table_values = [1.e-9 0.01 0.20 0.30 0.40 0.50 0.70 1.00 10.00 1.e9]; 
        f_r_table_values = [1.00  1.00 0.98 1.20 1.25 1.30 1.25 1.00  1.00 1.00];
        f_r = interp1(H_r_table_values,f_r_table_values,H_r); % linear interpolation of
                                                              % friction factor ratio, -
        N_Re = rho_mn*abs(v_ms)*d/mu_mn; % gas-liquid mixture Reynolds number, -
        f_mn = Moody_friction_factor(epsilon,N_Re); % 'no-slip' gas-liquid mixture friction
        %                                             factor, -  
        f = f_mn * f_r; % 'slip' gas-liquid mixture friction factor, -
        help11 = rho_ms*v_ms*v_sg/p_tot; % acceleration loss factor E_k, -
        if help11 >= 1 
           error('Choked flow. Reduce rate, increase diameter, or increase back pressure.')
        end
        help12 = rho_ms*g*cos(alpha);
        help13 = -f*rho_mn*v_ms*abs(v_ms)/(2*d);
        dpds_grav = help12; % gravity losses, Pa/m
        dpds_fric = help13; % friction losses, Pa/m
        dpds_acc = (help12+help13)*(help11/(1-help11)); % acceleration losses, Pa/m
        dpds_tot = dpds_grav + dpds_fric + dpds_acc; % total pressure gradient, Pa/m
        dpds = [dpds_tot;dpds_grav;dpds_fric;dpds_acc];
    end
    
    % Compute pressure gradient for stratified flow:
    if flow_reg == 4
        % Compute delta iteratively through successive substitution:
        iter = 0; %iteration counter, -
        max_iter = 100; % maximum allowed number of iterations, -
        error_abs = 2*pi; % initial error, rad 
        tol_abs = 1.e-6; % absolute convergence criterion, rad
        delta = 0.001; % initial guess, opening angle liquid layer in stratified flow, rad
        while error_abs > tol_abs
            delta_old = delta;
            delta = 2*pi*H_l + sin(delta_old);
            error_abs = abs(delta-delta_old);
            iter = iter+1;
            if iter > max_iter
                error_abs = 0; %%% temporary fix!!!
%                delta
%                H_l
%                error('Error: Maximum allowed number of iterations exceeded.')
            end
        end   
        
        % Compute geometrical parameters:
        A = pi*d^2/4; % cross-sectional area, m^2
        A_g = A * H_g; % gas cross-sectional area, m^2
        A_l = A * H_l; % liquid cross-sectional area, m^2
        help31 = sin(delta);
        help32 = sin(delta/2);
        help33 = delta-help31;
        help34 = delta-2*help32;
        help35 = delta+2*help32;
        d_hg = d*(2*pi-help33)/(2*pi-help34); % gas hydraulic diameter, m
        d_hl = d*help33/help35; % liquid hydraulic diameter, m    
        P = pi*d; % pipe perimeter, m
        P_g = (1-delta/(2*pi))*P; % gas wetted perimeter, m
        P_l = P - P_g; % liquid wetted perimeter, m
        
        % Compute shear stresses:
        v_g = v_sg/H_g; % gas velocity, m/s
        v_l = v_sl/H_l; % liquid velocity, m/s
        N_Re_g = rho_g*abs(v_g)*d_hg/mu_g; % gas Reynolds number, -
        N_Re_l = rho_l*abs(v_l)*d_hl/mu_l; % liquid Reynolds number, -
        f_g = Moody_friction_factor(epsilon,N_Re_g); % gas friction factor, -
        f_l = Moody_friction_factor(epsilon,N_Re_l); % liquid friction factor, -
        tau_wg = f_g*rho_g*v_g*abs(v_g)/2; % gas shear stress, N/m^2
        tau_wl = f_l*rho_l*v_l*abs(v_l)/2; % liquid shear stress, N/m^2
        
        % Compute pressure gradient dpds:
        help42 = (rho_g*A_g + rho_l*A_l)*g*cos(alpha);
        help43 = -(tau_wg*P_g + tau_wl*P_l);
        dpds_grav = (help42); % gravity losses, Pa/m
        dpds_fric = (help43); % friction losses, Pa/m 
        dpds_acc = 0; % acceleration losses neglected, Pa/m
        dpds_tot = dpds_grav + dpds_fric; % total pressure gradient, Pa/m
        dpds = [dpds_tot;dpds_grav;dpds_fric;dpds_acc];
    end
end
