function [flow_dir,flow_reg,theta_MB] = Muk_Brill_flow_reg(alpha,N_gv,N_lv,N_mu,v_sg,v_sl)
% function [flow_dir,flow_reg,theta_MB] = Muk_Brill_flow_reg(alpha,N_gv,N_lv,N_mu,v_sg,v_sl)
%
% Computes the flow regime and some auxiliary parameters for a given pressure p and along-hole
% distance s, in an element of a flowline-wellbore system.  
%
% Uses the Mukherjee and Brill correlation for multiphase flow in inclined wells; see
% references [1] and [2]. 
% 
% alpha = inclination wrt. vertical, rad;
% flow_dir = flow direction, -
%   -1 = downhill flow
%    0 = horizontal flow
%    1 = uphill flow
% flow_reg = flow regime indicator, -
%    0 = single-phase liquid
%    1 = bubble flow
%    2 = slug flow
%    3 = annular mist flow
%    4 = stratified flow
% N_gv = gas velocity number, -
% N_lv = liquid velocity number, -
% N_mu = liquid viscosity number, -
% theta_MB = angle as defined in Mukherjee and Brill (1985), rad
% v_sg = superficial gas velocity, m/s
% v_sl = superficial liquid velocity, m/s
%
% JDJ, 29-10-11, last revised 10-05-13
%
% References:
% [1] Mukherjee H. and Brill, J.P., 1985b: Empirical equations to predict flow patterns in
%     two-phase inclined flow. International Journal of Multiphase Flow, 11 (3) 299-315.  
% [2] Brill, J.P. and Mukherjee, H., 1999: Multiphase flow in wells, SPE Monograph Series, 
%     vol. 17, SPE, Richardson. 

% Determine flow direction (uphill, downhill or horizontal)
v_ms = v_sg + v_sl; % local mixture velocity, m/s
if v_ms > 0 % flow from wellhead to bottomhole (injection well)
    if  alpha < pi/2 % 'downhill' drilled well section (usual situation)
        flow_dir = -1; % downhill flow
    else
        if  alpha > pi/2 % 'uphill' drilled well section (may occur in 'horizontal' wells)
            flow_dir = 1; % uphill flow
        else % alpha = pi/2, horizontal well section
            flow_dir = 0; % horizontal flow
        end
    end
else % flow from bottomhole to wellhead (production well)
    if  alpha < pi/2 % 'downhill' drilled well section
        flow_dir = 1; % uphill flow
    else
        if alpha > pi/2 % 'uphill' drilled well section
            flow_dir = -1; % downhill flow
        else % alpha = pi/2, horizontal well section
            flow_dir = 0; % horizontal flow
        end
    end
end

% Determine the value of theta_MB. This is the angle as defined in the
% original publication of Mukherjee and Brill.
theta_MB = flow_dir*abs(alpha-pi/2); % negative for downward and positive for upward flow

% Check for free gas:
if abs(v_sg) < 1.e-12 % no free gas - liquid flow only
    flow_reg = 0; % liquid-only flow

else % gas-liquid flow
   
    % Determine flow pattern boundaries:
    help01 = sin(theta_MB);
    
    help02 = (log10(N_gv) + 0.940 + 0.074*help01 - 0.855*help01^2 + 3.695*N_mu);
    N_lv_bs = 10^help02; % upflow bubble-slug transition boundary, -
    
    help03 = 1.401 - 2.694*N_mu + 0.521*N_lv^0.329;
    N_gv_sm = 10^help03; % universal slug-mist transition boundary, -
    
    help04 = log10(N_lv);
    help05 = 0.431 - 3.003*N_mu - (1.138*help04 + 0.429*help04^2 - 1.132) * help01;
    N_gv_bs = 10^help05; % downflow and horizontal bubble-slug transition boundary, -
    
    help06 = log10(N_gv);
    help07 = 0.321 - 0.017*N_gv - 4.267*help01 - 2.972*N_mu - 0.033*help06^2 - 3.925* help01^2;
    N_lv_st = 10^help07; % downflow and horizontal stratified flow boundary, -
    
    % Determine flow pattern:
    if N_gv >= N_gv_sm
        flow_reg = 3; % annular mist flow
    else
        if theta_MB > 0 % uphill flow
            if N_lv > N_lv_bs
                flow_reg = 1; % bubble flow
            else
                flow_reg = 2; % slug flow
            end
        else % downhill or horizontal flow
            if abs(theta_MB) > pi/6 % i.e. alpha < 60 deg.
                if N_gv > N_gv_bs
                    if N_lv > N_lv_st
                        flow_reg = 2; % slug flow
                    else
                        flow_reg = 4; % stratified flow
                    end
                else
                    flow_reg = 1; % bubble flow
                end
            else % abs(theta_MB) <= pi/6, i.e. alpha >= 60 deg.
                if N_lv > N_lv_st
                    if N_gv > N_gv_bs
                        flow_reg = 2; % slug flow
                    else
                        flow_reg = 1; % bubble flow
                    end
                else
                    flow_reg = 4; % stratified flow
                end
            end
        end
    end
end       
