function [H_g,H_l,lambda_l,lambda_g] = Muk_Brill_hold_up(flow_dir,flow_reg,N_gv,N_lv,...
    N_mu,q_g,q_l,theta_MB)
% function [H_g,H_l,lambda_l,lambda_g] = Muk_Brill_hold_up(flow_dir,flow_reg,N_gv,N_lv,...
%     N_mu,q_g,q_l,theta_MB)   
%
% Computes the gas and liquid hold-ups and in-situ volume fractions for a given pressure p and
% along-hole distance s, in an element of a flowline-wellbore system.
%
% Uses the Mukherjee and Brill correlation for liquid hold-up in inclined wells; see
% references [1] and [2]. A reality check has been added to ensure that the computed liquid 
% hold-up (for flow with slip) is never smaller than the in-situ liquid volume fraction (the
% 'no-slip hold-up'). 
% 
% flow_dir = flow direction, -
%   -1 = downhill flow
%    0 = horizontal flow
%    1 = uphill flow
% flow_reg = flow regime indicator, -
%    0 = single-phase liquid
%    1 = bubble flow
%    2 = slug flow
%    3 = annular mist flow
%    4 = stratified flow
% H_g = gas hold-up, -
% H_l = liquid hold-up, -
% lambda_g = in-situ gas volume fraction (no-slip), -
% lambda_l = in-situ liquid volume fraction (no-slip), -
% N_gv = gas velocity number, -
% N_lv = liquid velocity number, -
% N_mu = liquid viscosity number, -
% q_g = local gas flow rate, m^3/s
% q_l = local liquid flow rate, m^3/s
% theta_MB = angle as defined in Mukherjee and Brill (1985), rad
%
% JDJ, 30-10-11, last revised 10-05-13
%
% References:
% [1] Mukherjee H. and Brill, J.P., 1983: Liquid holdup correlations for inclined two-phase
%     flow. J. of Petroleum Technology, 35 (5) 1003-1008. 
% [2] Brill, J.P. and Mukherjee, H., 1999: Multiphase flow in wells, SPE Monograph Series,
%     vol. 17, SPE, Richardson. 

if flow_reg == 0 % no free gas - liquid flow only
    H_l = 1;
    H_g = 0;
    lambda_l = 1;
    lambda_g = 0;
else % gas-liquid flow        
    % Compute holdup correlation parameters:
    if flow_dir == 0 || flow_dir == 1 % horizontal or uphill flow
        C1 = -0.380113;
        C2 =  0.129875;
        C3 = -0.119788;
        C4 =  2.343227;
        C5 =  0.475686;
        C6 =  0.288657;
    else % downhill flow
        if flow_reg == 4 % downhill stratified flow
            C1 = -1.330282;
            C2 = 4.808139; 
            C3 =  4.171584;
            C4 = 56.262268;
            C5 =  0.079951;
            C6 =  0.504887;
        else % downhill other flow
            C1 = -0.516644;
            C2 =  0.789805;  
            C3 =  0.551627;
            C4 = 15.519214;
            C5 =  0.371771;
            C6 =  0.393952;
        end
    end
    % Compute liquid and gas volume fractions (no-slip):
    lambda_l = q_l/(q_l+q_g); % liquid volume fraction, -
    lambda_g = 1-lambda_l; % gas volume fraction, -
    % Compute liquid and gas hold-ups (with slip):
    help08 = C1 + C2*sin(theta_MB) + C3*(sin(theta_MB))^2 + C4*N_mu^2;      
    help09 = N_gv^C5 / N_lv^C6;
    H_l = exp(help08*help09); % liquid hold-up, -
    if H_l < 1e-9
        H_l = 1e-9; % to avoid numerical problems and stay within look-up table below
    end
    if H_l < lambda_l % Reality check (not included in [1])
        H_l = lambda_l;
    end
    H_g = 1-H_l; % gas hold-up, -
end
