function f = Zig_and_Syl_fric_fact(epsilon,N_Re)
% function f = Zig_and_Syl_fric_fact(epsilon,N_Re)
%
% Computes the friction factor for pipe flow according to the Moody diagram [1]. In the
% turbulent region, the explicit approximation by Zigrang and Sylvester of the implicit
% Colebrook expression is used to compute the friction factor [2], [3].       
%
% References:
% [1] Moody, L.F.: "Friction factors for pipe flow," Trans. Am. Soc. Mech.
% Eng. (1944) Vol.66, 671-684.      
%
% [2] Zigrang, D.J. and Sylvester, N.D.: "A review of explicit friction
% factor equations," J. Energy Res. Tech. (June 1985), Vol.107, 280-283. 
%
% [3] Colebrook, C.F., "Turbulent flow in pipes with particular reference
% to the transition region between the smooth and rough pipe laws,"
% J. Inst. Civil Eng. (1939) Vol.11, 133-156.  
%
% epsilon = dimensionless roughness, -
% f = friction factor, -
% N_Re = Reynolds number, -
%
% JDJ, 27-01-03, last revised 10-05-13

if N_Re < 2000 % Laminar regime
   f = 64/N_Re;
else % Turbulent or transitional regime
   if N_Re < 3000 % Transitional regime: prepare for interpolation
      f_lam_max = 64/2000; % Highest laminar value 
      alpha = (N_Re-2000)/(3000-2000); % Interpolation parameter
      help01 = 2*epsilon/3.7 + 13/3000;
      help02 = (5.02/3000)*log10(help01);
      f_turb_min = 1/(-2*log10(2*epsilon/3.7 - help02))^2;  
      f = f_lam_max + alpha * (f_turb_min - f_lam_max);             
  else % Turbulent regime
      help01 = 2*epsilon/3.7 + 13/N_Re;
      help02 = (5.02/N_Re)*log10(help01);
      f = 1/(-2*log10(2*epsilon/3.7 - help02))^2;  
  end
end
   
