% Script file example_flowline
%
% Example to compute the pressure drop over an inclined flowline. First the flow line pressure
% is computed for a given manifold pressure. Thereafter the flowline pressure is recomputed by
% integration in the opposite direction.
%
% JDJ, 13-03-03, last revised 26-05-17

clear variables
close all

% ---------------------------------------------------------------------------------------------
% Input data:
% ---------------------------------------------------------------------------------------------
d = 0.062; % flowline inside diameter, m 
e = 30e-6; % flowline roughness, m
fluid = 4; % fluid = 1: single-phase oil flow
%           fluid = 2: single-phase gas flow
%           fluid = 3: multi-phase gas-oil-water flow, Hagedorn and Brown correlation
%           fluid = 4: multi-phase gas-oil-water flow, Mukherjee and Brill correlation
%           fluid = 5: multi-phase gas-oil-water flow, Beggs and Brill correlation     
%           fluid = 6: multi-phase gas-oil-water flow, Shi et al. drift flux model (exercise)
oil = 1; % parameter to select black oil model or volatile oil table, -. Not relevant for
%          fluids 1 and 2
%          oil = 1: black oil; parameters computed with the aid of Standing correlations
%          oil = 2: black oil; parameters computed with the aid of Glaso correlations
%          oil = 3: volatile oil; parameters read from file 'vol_oil_table_01'
%          Note: When using a volatile oil table, the input parameters R_sb and rho_sc should 
%          be consistent with those used to generate the tabulated values. For
%          vol_oil_table_01: rho_g_sc = 0.80 kg/m^3, rho_o_sc = 800 kg/m^3, R_sb = 450 m^3/m^3 
p_mf = 7.0e6 % manifold pressure, Pa.
q_g_sc = -5.e-1; % gas rate at standard conditions, m^3/s
q_o_sc = -0.8e-2; % oil rate at standard conditions, m^3/s. Should be 0 for fluid 2
q_w_sc = -0.2e-2; % water rate at standard conditions, m^3/s. Should be 0 for fluids 1 and 2
% Note: flow rates should have negative values for a production well!
rho_g_sc = 0.95; % gas density at standard conditions, kg/m^3 
rho_o_sc = 850;  % oil density at standard conditions, kg/m^3. Not relevant for fluid 2
rho_w_sc = 1050; % water density at standard conditions, kg/m^3. Not relev. for fluids 1 and 2
s_tot = 1200; % total flow line length, m
theta = 0; % flowline inclination wrt. horizontal, rad; a positive value indicates an
           % increasing flow line elevation, seen from the manifold 
T_fl = 30; % flowline temperature at the well head, deg. C
T_mf = 20; % manifold temperature, deg. C
% ---------------------------------------------------------------------------------------------
% End of input data
% ---------------------------------------------------------------------------------------------

% Create data vectors:
q_sc = [q_g_sc,q_o_sc,q_w_sc];
rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc];

% Convert from pipeline inclination to well bore inclination:
alpha = from_theta_to_alpha(theta);

% Compute the flowline pressure by integrating from 0 to s_tot:
p_fl = pipe(alpha,d,e,fluid,oil,p_mf,q_sc,rho_sc,0,s_tot,T_mf,T_fl)

% Recompute the manifold pressure by integrating from s_tot to 0:
p_mf = pipe(alpha,d,e,fluid,oil,p_fl,q_sc,rho_sc,s_tot,0,T_fl,T_mf)
