% Script file example_hold_up
%
% Example to create a plot of liquid hold-up H_l and liquid volume fraction lambda_l
% versus depth using the Mukherjee and Brill correlation. This requires a two-step approach:  
% 1) Compute the pressure as function of depth (i.e. a traverse) with the aid of pipe.m (which
%    uses Muk_Brill_dpds.m in combination with the standard Matlab routine ode45.m for
%    numerical integration).  
% 2) Recompute the hold-up with the aid of four auxiliary files (local_gas_liq_props.m,
%    Duns_Ros_dim_less_nrs.m, Muk_Brill_flow_reg.m and Muk_Brill_hold_up.m)
% The four auxiliary files are subroutines of Muk_Brill_dpds.m and are therefore also used in
% step 1. However because they are used in combination with ode45.m it is not simple to store
% their results during execution of step 1, and it is easier to simply re-run them in step 2.     
%
% JDJ, 30-10-11, last revised 26-05-17

clear variables
close all

% ---------------------------------------------------------------------------------------------
% Input data:
% ---------------------------------------------------------------------------------------------
alpha = 0; % wellbore inclination , rad
d = 0.062; % tubing inside diameter, m 
e = 30e-6; % tubing rougness, m
fluid = 4; % fluid = 1: single-phase oil flow
%            fluid = 2: single-phase gas flow
%            fluid = 3: multi-phase gas-oil-water flow, Hagedorn and Brown correlation
%            fluid = 4: multi-phase gas-oil-water flow, Mukherjee and Brill correlation
%            fluid = 5: multi-phase gas-oil-water flow, Beggs and Brill correlation     
oil = 3; % parameter to select black oil model or volatile oil table, -. Not relevant for
%          fluids 1 and 2
%          oil = 1: black oil; parameters computed with the aid of Standing correlations
%          oil = 2: black oil; parameters computed with the aid of Glaso correlations
%          oil = 3: volatile oil; parameters read from file 'vol_oil_table_01'
%          Note: When using a volatile oil table, the input parameters R_sb and rho_sc should 
%          be consistent with those used to generate the tabulated values. For
%          vol_oil_table_01: rho_g_sc = 0.80 kg/m^3, rho_o_sc = 800 kg/m^3, R_sb = 450 m^3/m^3 
p_tf = 1.0e6; % FBHP, Pa
f_w_sc = 0.2; % water cut, -
R_go = 450; % gas-oil ratio, m^3/m^3
q_o_sc = -5e-3; % oil rate at standard conditions, m^3/s
q_g_sc = R_go * q_o_sc; % gas rate at standard conditions, m^3/s 
q_w_sc = (f_w_sc/(1-f_w_sc)) * q_o_sc; % water rate at standard conditions, m^3/s
% Note: flow rates should have negative values for a production well!
rho_g_sc = 0.80; % gas density at standard conditions, kg/m^3 
rho_o_sc = 800;  % oil density at standard conditions, kg/m^3
rho_w_sc = 1050; % water density at standard conditions, kg/m^3
s_tot = 3500; % total along-hole well depth, m 
T_tf = 30; % tubing head temperature, deg. C
T_wf = 120; % bottomhole temperature, deg. C
% ---------------------------------------------------------------------------------------------
% End of input data
% ---------------------------------------------------------------------------------------------

% Create data vectors:
q_sc = [q_g_sc,q_o_sc,q_w_sc];
rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc];

% Compute traverse:
[p_tf,s,p] = pipe(alpha,d,e,fluid,oil,p_tf,q_sc,rho_sc,0,s_tot,T_tf,T_wf);

% Recompute the flow regimes and the hold-up as a post-processing step:
n = length(p);
H_l_vec = zeros(1,n);
lambda_l_vec = zeros(1,n);
for i=1:n
    % Compute local gas and liquid properties:
    [mu_g,mu_l,q_g,q_l,rho_g,rho_l,sigma_gl,v_sg,v_sl] = local_gas_liq_props(d,oil,...
        p(i,1),q_sc,rho_sc,s(i),0,s_tot,T_tf,T_wf);

    % Compute Duns and Ros dimensionles numbers: 
    [N_d,N_gv,N_lv,N_mu] = Duns_Ros_dim_less_nrs(d,mu_l,rho_l,sigma_gl,v_sg,v_sl);

    % Compute flow regimes according to the Mukherjee and Brill correlation:
    [flow_dir,flow_reg,theta_MB] = Muk_Brill_flow_reg(alpha,N_gv,N_lv,N_mu,v_sg,v_sl);
    
    % Compute hold-ups according the Mukherjee and Brill correlation:
    [H_g,H_l,lambda_l,lambda_g] = Muk_Brill_hold_up(flow_dir,flow_reg,N_gv,N_lv,...
    N_mu,q_g,q_l,theta_MB);
    H_l_vec(i) = H_l; % collect liquid hold-up values in a vector 
    lambda_l_vec(i) = lambda_l; % collect liquid hold-up values in a vector 
end

% Plot results:
plot(p/1e6,s,'LineWidth',1)
axis ij
xlabel('Wellbore Pressure,\it p\rm (MPa)')
ylabel('AHD,\it s\rm (m)')
grid on
legend('\itp_{\rmtot}','\itp_{\rmgrav}','\itp_{\rmfric}','\itp_{\rmacc}','location','NorthEast')

figure
plot([H_l_vec;lambda_l_vec],s,'LineWidth',1)
axis ij
xlabel('Liquid Hold Up,\it H_l\rm (-)')
ylabel('AHD,\it s\rm (m)')
grid on
legend('\itH_l','\it\lambda_l','location','NorthEast')
