function dpds = gas_dpds(s,p,~,alpha,d,e,q_sc,rho_sc,s_in,s_out,T_in,T_out)
% function dpds = gas_dpds(s,p,~,alpha,d,e,q_sc,rho_sc,s_in,s_out,T_in,T_out)
%
% Computes the derivative dp/ds in single-phase gas flow for a given pressure p and along-hole
% distance s, in an element of a flowline-wellbore system. The distance s is measured from the
% separator towards the reservoir. Therefore, flowrates are negative for production wells.  
%
% The vector p contains the total pressure, and the pressures taking into account the
% individual effects of gravity, friction and acceleration losses respectively. Accordingly,
% the vector dpds contains the total pressure loss per unit length, as well as the individual
% gravity losses, friction losses and acceleration losses.
%
% This function can be used to compute the pressure drop through numerical integration. It has
% the correct format to be used in conjunction with one of the standard numerical integration
% routines in MATLAB.   
%
% alpha = inclination wrt. vertical, rad; alternatively alpha can be a survey file (matrix)
% with AHD values in the first column (in m) and inclination values in the second column (in
% rad).    
% d = inside diameter, m
% dpds = [dpds_tot;dpds_grav;dpds_fric;dpds_acc]
%   dpds_acc  = pressure gradient due to acceleration losses, Pa/m
%   dpds_fric = pressure gradient due to friction losses, Pa/m 
%   dpds_grav = pressure gradient due to head losses, Pa/m 
%   dpds_tot  = dpds_grav + dpds_fric + dpds_acc = total pressure gradient, Pa/m 
% e = roughness, m
% p = [p_tot,p_grav,p_fric,p_acc], Pa
%   p_acc  = p_in + pressure increase (decrease for production wells) due to accel. losses, Pa
%   p_fric = p_in + pressure increase (decrease for production wells) due to frict. losses, Pa
%   p_grav = p_in + pressure increase (decrease for production wells) due to head loss, Pa
%   p_tot  = p_in + pressure increase (decrease for production wells) due to gravity, friction
%            and acceleration losses, Pa 
% p_in = pressure at s_in, Pa 
% p_out = pressure at s_out, Pa
% q_sc = [q_g_sc,q_o_sc,q_w_sc], m^3/s
%   q_g_sc = gas flow rate at standard conditions, m^3/s. 
%   q_o_sc = oil flow rate at standard conditions, m^3/s. Not relevant
%   q_w_sc = water flow rate at standard conditions, m^3/s. Not relevant
% rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc], kg/m^3 
%   rho_g_sc = gas density at standard conditions, kg/m^3.
%   rho_o_sc = oil density at standard conditions, kg/m^3. Not relevant
%   rho_w_sc = water density at standard conditions, kg/m^3. Not relevant
% s = along-hole distance, measured from the separator to the reservoir, m   
% s_in = starting point for the integration
% s_out = end point for the integration
% T_in = temperature at s_in, deg. C 
% T_out = temperature at s_out, deg. C 
%
% JDJ, 21-02-02, revised 18-01-14

% Check sign of pressure:
p_tot = p(1); % first element of vector p is the total wellbore pressure, Pa 
if min(p_tot) < 1e5
    warning('Pressure below atmospheric.')
    dpds_tot  = 0; 
    dpds_grav = 0; 
    dpds_fric = 0; 
    dpds_acc  = 0; 
    dpds = [dpds_tot;dpds_grav;dpds_fric;dpds_acc];
    return
end

% Determine inclination in case of survey file input:
if length(alpha) > 1
    n_sur = length(alpha(:,1)); % number of survey points
    if s < alpha(1,1)
        help = alpha(1,2);
    else if s > alpha(n_sur,1)
        help = alpha(n_sur,2);
        else
            help = interp1(alpha(:,1),alpha(:,2),s);
        end
    end
    clear alpha;
    alpha = help; % replace survey file by single inclination value, rad
end

% Densities and flow rates at standard conditions:
rho_g_sc = rho_sc(1); % gas density, kg/m^3
q_g_sc = q_sc(1); % gas flow rate, m^3/s
q_o_sc = q_sc(2); % oil flow rate, m^3/s
q_w_sc = q_sc(3); % water flow rate, m^3/s

% Check input:
if (q_o_sc ~= 0)
    error('Nonzero oil rate. Single-phase gas solution not valid.')
end
if (q_w_sc ~= 0)
    error('Nonzero water rate. Single-phase gas solution not valid.')
end

% Determine inclination in case of survey file input:
if length(alpha) > 1
    n_sur = length(alpha(:,1)); % number of survey points
    if s < alpha(1,1)
        help = alpha(1,2);
    else if s > alpha(n_sur,1)
        help = alpha(n_sur,2);
        else
            help = interp1(alpha(:,1),alpha(:,2),s);
        end
    end
    clear alpha;
    alpha = help; % replace survey file by single inclination value, rad
end

% Compute internal variables:
A = (pi*d^2)/4;% cross-sectional area, m^2
g = 9.81; % acceleration of gravity, m/s^2
M = from_kg_per_m3_to_molar_mass(rho_g_sc); % molar mass, kg/mol
p_tot = p(1); % first element of vector p is the total wellbore pressure, Pa 
R = 8314; % universal gas constant, J/(K*kmol) 

% Compute temperature through linear interpolation between T_in and T_out:
T = T_in+(T_out-T_in)*(s-s_in)/(s_out-s_in); % temperature, deg. C
T_abs = T + 273.15; % absolute temperature, K

% Compute Z:
p_pc = pres_pseu_crit_Sutton(rho_g_sc); % pseudo-critical pressure, Pa
T_pc = temp_pseu_crit_Sutton(rho_g_sc); % pseudo-critical temperaure, K
p_pr = p_tot/p_pc; % pseudo-reduced pressure, -
T_pr = T_abs/T_pc; % pseudo-reduced temperature, -
Z = Z_factor_DAK(p_pr,T_pr); % Z-factor, -

% Compute dZdp:
h = 1e-6; % peturbation to compute numerical derivative, Pa
p_h = p_tot+h; % perturbed pressure, Pa
p_pr_h = p_h/p_pc; 
Z_h = Z_factor_DAK(p_pr_h,T_pr); 
dZdp = (Z_h-Z)/(p_h-p_tot); 

% Compute mu, rho and v:
mu = gas_viscosity(p_tot,rho_g_sc,T); % local gas viscosity, Pa s
rho = 1000*M*p_tot/(Z*R*T_abs); % local gas density, kg/m^3
v = rho_g_sc*q_g_sc/(A*rho); % local gas velocity, m/s

% Compute f:
N_Re = rho*d*abs(v)/mu; % Reynolds number, -
epsilon = e/d; % dimensionless roughness, -
f = Moody_friction_factor(epsilon,N_Re); % friction factor, -

% Compute dpds:
help01 = rho*v^2*(1/p_tot-dZdp/Z);
help02 = rho*g*cos(alpha);
help03 = -rho*f*v*abs(v)/(2*d);
dpds_grav = help02; % gravity losses, Pa/m
dpds_fric = help03; % friction losses, Pa/m
dpds_acc = (help02+help03)*(help01/(1-help01)); % acceleration losses, Pa/m
dpds_tot = dpds_grav + dpds_fric + dpds_acc; % total pressure gradient, Pa/m
dpds = [dpds_tot;dpds_grav;dpds_fric;dpds_acc];

