function dpds = oil_dpds(s,p,~,alpha,d,e,oil,q_sc,rho_sc,s_in,s_out,T_in,T_out)
% function dpds = oil_dpds(s,p,~,alpha,d,e,oil,q_sc,rho_sc,s_in,s_out,T_in,T_out)
%
% Computes the derivative dp/ds in single-phase oil flow for a given pressure p and along-hole
% distance s, in an element of a flowline-wellbore system. The distance s is measured from the
% separator towards the reservoir.      
%
% The vector p contains the total pressure, and the pressures taking into account the
% individual effects of gravity, friction and acceleration losses respectively. Accordingly,
% the vector dpds contains the total pressure loss per unit length, as well as the individual
% gravity losses, friction losses and acceleration losses. 
%
% This function can be used to compute the pressure drop through numerical integration. It has
% the correct format to be used in conjunction with one of the standard numerical integration
% routines in MATLAB.    
%
% alpha = inclination wrt. vertical, rad; alternatively alpha can be a survey file (matrix)
% with AHD values in the first column (in m) and inclination values in the second column (in
% rad).    
% d = inside diameter, m
% dpds = [dpds_tot;dpds_grav;dpds_fric;dpds_acc]
%   dpds_acc  =  pressure gradient due to acceleration losses, Pa/m
%   dpds_fric =  pressure gradient due to friction losses, Pa/m 
%   dpds_grav = pressure gradient due to head losses, Pa/m 
%   dpds_tot  = dpds_grav + dpds_fric + dpds_acc = total pressure gradient, Pa/m 
% e = roughness, m
% oil = parameter to select black oil model or volatile oil table, -  
%   oil = 1: black oil; parameters computed with the aid of Standing correlations
%   oil = 2: black oil; parameters computed with the aid of Glaso correlations
%   oil = 3: volatile oil; parameters read from file 'vol_oil_table_01'
% p = [p_tot,p_grav,p_fric,p_acc], Pa
%   p_acc  = p_in + pressure increase (decrease for production wells) due to accel. losses, Pa
%   p_fric = p_in + pressure increase (decrease for production wells) due to frict. losses, Pa
%   p_grav = p_in + pressure increase (decrease for production wells) due to head loss, Pa
%   p_tot  = p_in + pressure increase (decrease for production wells) due to gravity, friction
%            and acceleration losses, Pa 
% p_in = pressure at s_in, Pa 
% p_out = pressure at s_out, Pa
% q_sc = [q_g_sc,q_o_sc,q_w_sc], m^3/s
%   q_g_sc = gas flow rate at standard conditions, m^3/s  
%   q_o_sc = oil flow rate at standard conditions, m^3/s
%   q_w_sc = water flow rate at standard conditions, m^3/s. Not relevant
% rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc], kg/m^3 
%   rho_g_sc = gas density at standard conditions, kg/m^3 
%   rho_o_sc = oil density at standard conditions, kg/m^3
%   rho_w_sc = water density at standard conditions, kg/m^3. Not relevant
% s = along-hole distance, measured from the separator to the reservoir, m   
% s_in = starting point for the integration
% s_out = end point for the integration
% T_in = temperature at s_in, deg. C 
% T_out = temperature at s_out, deg. C 
%
% JDJ, 17-03-03, last revised 17-10-13

% Check sign of pressure:
p_tot = p(1); % first element of vector p is the total wellbore pressure, Pa 
if min(p_tot) < 1e5
    warning('Pressure below atmospheric.')
    dpds_tot  = 0; 
    dpds_grav = 0; 
    dpds_fric = 0; 
    dpds_acc  = 0; 
    dpds = [dpds_tot;dpds_grav;dpds_fric;dpds_acc];
    return
end

% Determine inclination in case of survey file input:
if length(alpha) > 1
    n_sur = length(alpha(:,1)); % number of survey points
    if s < alpha(1,1)
        help = alpha(1,2);
    else if s > alpha(n_sur,1)
        help = alpha(n_sur,2);
        else
            help = interp1(alpha(:,1),alpha(:,2),s);
        end
    end
    clear alpha;
    alpha = help; % replace survey file by single inclination value, rad
end

% Densities and flow rates at standard conditions:
rho_g_sc = rho_sc(1); % gas density, kg/m^3
rho_o_sc = rho_sc(2); % oil density, kg/m^3
q_g_sc = q_sc(1); % gas flow rate, m^3/s
q_o_sc = q_sc(2); % oil flow rate, m^3/s
q_w_sc = q_sc(3); % water flow rate, m^3/s

% Check input:
if (q_w_sc ~= 0)
    error('Nonzero water rate. Use of oil_dpds not valid.')
end

% Compute temperature through linear interpolation between T_in and T_out:
T = T_in+(T_out-T_in)*(s-s_in)/(s_out-s_in); % temperature, deg. C

% Check if pressure is above bubble point pressure: 
R_go = q_g_sc/q_o_sc; % producing GOR as would be observed at surface, m^3/m^3
R_sb = R_go; % bubble point GOR for the oil in the wellbore
switch oil
    case 1
        p_b = pres_bub_Standing(R_go,rho_g_sc,rho_o_sc,T);
    case 2
        p_b = pres_bub_Glaso(R_go,rho_g_sc,rho_o_sc,T);
    case 3
        p_b = pres_bub_volatile_oil(T);
end
if p_tot < p_b
    error('Pressure below bubble point pressure. Use of oil_dpds not valid.')
end

% Compute internal variables:
A = (pi*d^2)/4;% cross-sectional area, m^2
g = 9.81; % acceleration of gravity, m/s^2

% Compute local oil properties:
[q,rho] = local_q_and_rho(oil,p_tot,q_sc,R_sb,rho_sc,T);
% q = [q_g, q_o, q_w], m^3/s, rho = [rho_g, rho_o, rho_w], kg/m^3 
q_o = q(2); % local oil flow rate, m^3/s
rho_o = rho(2); % local oil density, kg/m^3
mu_o = oil_viscosity(p_tot,R_sb,rho_g_sc,rho_o_sc,T); % local oil viscosity, Pa s
v_o = q_o/A; % local oil velocity, m/s

% Compute f:
N_Re = rho_o*d*abs(v_o)/mu_o; % Reynolds number, -
epsilon = e/d; % dimensionless roughness, -
f = Moody_friction_factor(epsilon,N_Re); % friction factor, -

% Compute dpds:
dpds_grav = rho_o*g*cos(alpha); % gravity losses, Pa/m
dpds_fric = -rho_o*f*v_o*abs(v_o)/(2*d); % friction losses, Pa/m
dpds_acc = 0; % accelleration losses neglected, Pa/m
dpds_tot = dpds_grav + dpds_fric; % total pressure gradient, Pa/m
dpds = [dpds_tot;dpds_grav;dpds_fric;dpds_acc];
