% Script answer_question_3_11
%
% Computes the pressure drop over a gas export line from an offshore platform (120 m above sea
% bottom) to an onshore gas plant (132 m above sea bottom). 
%
% Note: the coordinate s runs from the gas plant to the platform. The integration is performed
% backwards from s = l_tot to s = 0, where l_tot is the total pipe line length.
%
% JDJ, 25-07-13, last revised 07-01-16

clear variables
close all

% Input data:
l_4 = 120; % pipe line length, from platform to sea bottom (riser), m 
l_3 = 74000; % pipe line length, on sea bottom, m 
l_2 = 15000; % pipe line length, from sea bottom to shore (slope), m 
l_1 = 3000; % pipe line length, onshore, m 
alpha_4 = from_deg_to_rad(180); % pipe line inclination, from platform to sea bottom, rad
alpha_3 = from_deg_to_rad(90); % pipe line inclination, on sea bottom, rad
alpha_2 = atan(l_3/l_1); % pipe line inclination, from sea bottom to shore, rad
alpha_1 = from_deg_to_rad(90); % pipe line inclination, onshore, rad
d = 0.30; % pipe line inside diameter, m
e = 30e-6; % pipe line line roughness, m
fluid = 2; % single-phase gas, -
p_in = 10e6; % inlet pressure, Pa 
rho_g_sc = 0.95; % gas density at standard conditions, kg/m^3
T_in = 50; % inlet temperature, deg. C
T_out = 10; % outlet temperature, deg. C
q_g_sc_d = -1.0e6; % gas rate, m^3/s  

% Compute auxiliary variables:
q_g_sc = q_g_sc_d / (24*3600); % flow rate, m^3/s
rho_sc = [rho_g_sc,0,0]; % vector of densities at standard conditions, kg/m^3 
q_sc = [q_g_sc,0,0]; % vector of flow rates at standard conditions, m^3/s
l_tot = l_1 + l_2 + l_3 + l_4; % total pipe line length, m
Delta_T = T_in - T_out; % total temperature drop over the pipeline
oil = 0; % dummy variable; not relevant for single-phase gas flow, -

% Compute pressure drop:
[p_out_4,s_4,p_4] = pipe(alpha_4,d,e,fluid,oil,p_in,q_sc,rho_sc,...
    l_tot,l_tot-l_4,...
    T_in,T_in-Delta_T*l_4/l_tot); % from platform to sea bottom (riser)
[p_out_3,s_3,p_3] = pipe(alpha_3,d,e,fluid,oil,p_out_4,q_sc,rho_sc,...
    l_tot-l_4,l_tot-(l_4+l_3),...
    T_in-Delta_T*l_4/l_tot,T_in-Delta_T*(l_4+l_3)/l_tot); % on sea bottom
[p_out_2,s_2,p_2] = pipe(alpha_2,d,e,fluid,oil,p_out_3,q_sc,rho_sc,...
    l_tot-(l_4+l_3),l_tot-(l_4+l_3+l_2),...
    T_in-Delta_T*(l_4+l_3)/l_tot,T_in-Delta_T*(l_4+l_3+l_2)/l_tot); % slope
[p_out_1,s_1,p_1] = pipe(alpha_1,d,e,fluid,oil,p_out_2,q_sc,rho_sc,...
    l_tot-(l_4+l_3+l_2),0,...
    T_in-Delta_T*(l_4+l_3+l_2)/l_tot,T_out); % onshore
p_tot = [p_4;p_3;p_2;p_1]; % column vector of pressures in the pipe line, Pa
s_tot = [s_4;s_3;s_2;s_1]; % column vector with distances from the gas plant, Pa

% Note: because the integration has been performed backwards, values in s_tot descend from high
% to low, and the values of p_tot are ranked correspondingly. This is not a problem because it
% does not influence the plotting of the results. However, if it would be necessary to obtain
% vectors in ascending order this could be simply achieved with the command "flipud" (i.e. flip
% upside down): 
% p_tot = flipud(p_tot);
% s_tot = flipud(s_tot);

p_out = p_out_1 % pressure at the gas plant, Pa
Delta_p = p_in - p_out % pressure drop over the pipe line, Pa

% Create simple plot:
plot(s_tot/1e3,p_tot(:,1)/1e6,'LineWidth',1);
grid on  
xlabel('Distance From Gas Plant,\it s\rm (km)')
ylabel('Pressure,\it p\rm (MPa)')

