% Script file answer_question_3_12
%
% Iteratively computes the single-phase gas flow rate through a pipeline for given inlet and
% outlet pressures. 
%
% JDJ, 28-12-13, last revised 07-01-16

clear variables
close all

% ---------------------------------------------------------------------------------------------
% Input data:
% ---------------------------------------------------------------------------------------------
alpha = pi/2; % inclination, rad (horizontal pipeline)
d = 0.40; % inside diameter, m 
e = 50e-6; % roughness, m
fluid = 2; % single-phase gas flow
oil = 1; % arbitrary value; not relevant, but required by pipe.m
p_in = 5.0e6; % inlet pressure, Pa.
p_out = 4.0e6; % outlet pressure, Pa.
rho_g_sc = 0.95; % gas density at standard conditions, kg/m^3 
rho_o_sc = 0; % arbitrary value; not relevant, but required by pipe.m
rho_w_sc = 0; % arbitrary value; not relevant, but required by pipe.m
s_in = 0; % position of inlet, m
s_out = 8000; % position of outlet = pipe line length, m
T_in = 25; % inlet temperature, deg. C
T_out = 25; % outlet temperature, deg. C
% ---------------------------------------------------------------------------------------------
% End of input data
% ---------------------------------------------------------------------------------------------

% Create data vector:
rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc];

% Specify the interval:
lo = 20; % m^3/s
hi = 100; % m^3/s

% Compute the flow rate:
q_g_sc = fzero(@(q_g_sc)...
    p_out - pipe(alpha,d,e,fluid,oil,p_in,[q_g_sc,0,0],rho_sc,s_in,s_out,T_in,T_out), [lo hi])

% See Section E 1.5 in Appendix E for further info on the use of fzero.

% ---------------------------------------------------------------------------------------------
% Alternative implementation with a user-written Newton-Raphson routine: 
% ---------------------------------------------------------------------------------------------

q = (20+100)/2; % initial guess gas flow rate in abbreviated notation, m^3/s
d_q = 1.e-3; % flow rate perturbation size to obtain numerical derivative, m^3/s
tol = 1; % absolute convergence criterion, Pa
max_iter = 100; % maximum allowed number of iterations, -
iter = 0; % iteration counter, -
repeat = 1; % flag to continue iteration, -
while repeat > 0
    if iter > max_iter
        error('Error: Maximum allowed number of iterations exceeded.')
    end
    iter = iter + 1;
    % Compute residual:
    f = p_out - pipe(alpha,d,e,fluid,oil,p_in,[q,0,0],rho_sc,s_in,s_out,T_in,T_out);
    % Check convergence:
    if abs(f) < tol
        repeat = 0; % converged
    end    
    % Compute derivative with numerical perturbation:
    q_pert = q + d_q; % perturbed flow rate, m^3/s
    f_pert = p_out - pipe(alpha,d,e,fluid,oil,p_in,[q_pert,0,0],rho_sc,s_in,s_out,T_in,T_out);
    df = f_pert - f;
    dq = q_pert - q;
    dfdq = df/dq;
    % Compute new iterate:
    q = q - f / dfdq;   
end
% Write results to screen:
q_g_sc = q
iter
