% Script file answer_question_7_12
%
% Modified from file "example_point_sources".
%
% JDJ 06-01-16, last revised 29-05-17

clear variables
close all

% ---------------------------------------------------------------------------------------------
% Input data:
% ---------------------------------------------------------------------------------------------
h = 20; % reservoir height, m
r_e = 1000; % external radius, m
r_w = 0.20; % well radius, m
k = 1.0e-13; % permeability, m^2
mu = 5.8e-3; % viscosity, Pa s
p_R = 8e6; % reservoir pressure, Pa
p_wf = 7.0e6; % well bore pressure, Pa
B_o = 1.03; % oil formation volume factor, -
L_bb = 500; % well length back bone, m
L_br = 150; % well length branches, m
n_br = 0; % number of branches, -
N_bb = 80; % number of sources to represent back bone between branches (incl. kick-off pts.), -
N_br = 25; % number of sources to represent branches, -
theta_deg = 40; % angle of branches with respect to back bone, deg 
x_lo = -250; % lower boundary of plot domain in x-direction, m
x_hi = 750; % upper boundary of plot domain in x-direction, m
y_lo = -400; % lower boundary of plot domain in y-direction, m
y_hi = 400; % upper boundary of plot domain in y-direction, m
N_x = 400; % number of pressure points in x-direction (for contour plot), - 
N_y = 320; % number of pressure points in y-direction (for contour plot), - 
S_v = 5; % skin factor (as measured in a vertical well in the same reservoir), -
% ---------------------------------------------------------------------------------------------
% End of input data
% ---------------------------------------------------------------------------------------------

% Compute auxiliary variables:
L_tot = L_bb + n_br * L_br; % total well length, m
S_conv = log(h/(2*pi*r_w)); % geometric skin due to near-well streamline convergence, -
theta_rad = theta_deg*pi/180; % angle of branches with respect to backbone, rad
theta = ones(n_br,1) * theta_rad; % vector of angles for all branches, rad
for i = 2:2:n_br
    theta(i) = -theta(i); % switches the angle for all even branches
end

% Compute kick-off points:
Delta_x_ko = L_bb/(n_br+1); % distance between kick-off points, m
x_ko = zeros(n_br,1);
for i = 1:n_br
    x_ko(i) = i*Delta_x_ko; % x-coordinates kick-off points, m
end
x_ko_plus = [0;x_ko;L_bb]; % extended kick-off point vector, incl. begin and end of backbone, m

% Compute source coordinates:
N_bb_tot = (n_br+1) * (N_bb-1) + 1; % total number of sources in back bone, - 
Delta_L_bb = L_bb/(N_bb_tot-1); % distance between sources in back bone, m
Delta_L_br = L_br/(N_br-1); % distance between sources in branches, m
N_tot = N_bb_tot + n_br * (N_br-1) % total number of sources, -
x = zeros(N_tot,1);
y = zeros(N_tot,1);
for i = 1:N_bb_tot 
    x(i) = (i-1)*Delta_L_bb; % x coordinates backbone sources, m 
    y(i) = 0;                % y coordinates backbone sources, m
end
for i = 1:n_br
    for j = 1:N_br-1
        n = N_bb_tot + (i-1)*(N_br-1) + j;
        x(n) = x_ko(i) + cos(theta(i))*j*Delta_L_br; % x coordinates branch sources, m
        y(n) =           sin(theta(i))*j*Delta_L_br; % y coordinates branch sources, m
    end 
end

% Compute drawdown vector:
Delta_p = ones(N_tot,1) * (p_R - p_wf) * (-4)*pi*k*h / (mu*B_o); % drawdown in each source, Pa

% Define coordinates of points with fixed pressures (at wellbore radius):
x_p = zeros(N_tot,1);
y_p = zeros(N_tot,1);
x_p(1:N_bb_tot) = x(1:N_bb_tot); % x coord. for fixed pr. in backbone sources, m
y_p(1:N_bb_tot) = y(1:N_bb_tot) + r_w; % y coord. for fixed pr. in backbone sources, m
for i = 1:n_br
    for j = 1:N_br-1
        n = N_bb_tot + (i-1)*(N_br-1) + j;
        x_p(n) = x(n) + abs(sin(theta(i)))*r_w; % x coord. for fixed pr. in branch s., m
        y_p(n) = y(n) - sign(theta(i))*cos(theta(i))*r_w; % y coord. for f. pr. in br. s., m
    end 
end

% Assemble elements A_ij of matrix A:
A = zeros(N_tot,N_tot);
for i = 1:N_tot
    for j = 1:N_tot     
        A(i,j) = log(r_e^2) - log((x_p(i)-x(j))^2+(y_p(i)-y(j))^2) ...
            + 2*((h/L_tot)*S_conv + S_v);
    end
end

% Compute flow rate and PI:
q = A \ Delta_p; % source flow rates, m^3/s 
q_o_sc_tot = sum(q) % total flow rate, m^3/s
q_o_sc_tot_FU = from_m3_per_s_to_bpd(q_o_sc_tot) % total flow rate, bpd
p_R_FU = from_Pa_to_psi(p_R); % reservoir pressure, psi
p_wf_FU = from_Pa_to_psi(p_wf); % well bore pressure, psi
J = -q_o_sc_tot / (p_R - p_wf) % PI, m^3/(s*Pa)
J_FU = -q_o_sc_tot_FU / (p_R_FU - p_wf_FU) % PI, bpd/psi

% Generate matrix P_plot (densely sampled pressure values to allow for countour plotting):
x_plot = linspace(x_lo,x_hi,N_x);
y_plot = linspace(y_lo,y_hi,N_y);
p_plot = p_R * ones(N_y,N_x); % intialization at reservoir pressure
for i = 1:N_x
    for j = 1:N_y
        for n = 1:N_tot
            p_plot(j,i) = p_plot(j,i) + q(n) * (log(r_e^2)...
                - log((x_plot(i)-x(n))^2 + (y_plot(j)-y(n))^2))...
                * (mu*B_o)/(4*pi*k*h); 
        end
    end
end

% Plot pressure field:
p_plot = p_plot / 1e6; % scaled to plot contour lines in MPa instead of Pa
level = 7.5 + 0.05*[-10 -9 -8 -7 -6 -5 -4 -3 -2 -1 0 1 2 3 4 5 6 7 8 9 10]; % cont. levels, MPa
[C,h] = contour(x_plot,y_plot,p_plot,level,'LineWidth',1);
v = 7.5 + 0.05*[-8 -6 -4 -2 0 2 4 6 8 10]; % contour levels with labels, MPa
clabel(C,h,v) % plots labels at contour lines specified in v
hold on
plot([0,L_bb],[0,0],'Color','k','LineWidth',2.5); % horizontal line (backbone)
for i = 1:n_br
    plot([x_ko(i),x_ko(i)+cos(theta(i))*L_br],[0,sin(theta(i))*L_br],...
        'Color','k','LineWidth',2.5); % line (branch)
end
hold off
xlabel('Coordinate,\it x\rm (m)')
ylabel('Coordinate,\it y\rm (m)')
