% Script file answer_question_8_10_part_2
%
% Computes the traverse for a system consisting of a near-well bore area, a deviated
% well, a well head choke and a flow line, for different choices of the analysis node. 
%
% JDJ, 02-01-14, last revised, 29-05-17

clear variables
close all

% ---------------------------------------------------------------------------------------------
% Input data:
% ---------------------------------------------------------------------------------------------
% Production data:
f_w = 0; % water cut, - 
R_go = 450; % producing gas-oil ratio, m^3/m^3
p_mf = 3e6; % manifold pressure, Pa
p_R = 45e6; % reservoir pressure, Pa
q_o_sc = -2.22e-3; % oil rate, m^3/s

% Fluid data:
oil = 3; % volatile oil; parameters read from file 'vol_oil_table_01'
R_sb = R_go; % solution GOR at bubble point, m^3/m^3 
rho_g_sc = 0.80; % gas density at standard conditions, kg/m^3
rho_o_sc = 800;  % oil density at standard conditions, kg/m^3
rho_w_sc = 1080; % water density at standard conditions, kg/m^3
fluid = 4; % multi-phase gas-oil-water flow, Mukherjee and Brill correlation, -

% Reservoir data:
beta = 0; % Forchheimer coefficient, m^-1 
h = 30; % reservoir hight, m
k = 1e-14; % permeability, m^2
r_e = 400; % reservoir drainage radius, m
r_w = 0.1778; % wellbore radius, m
        
% Rel perm data:
k_rg_0 = 0.7; % end-point relative permeability to gas, -
k_ro_0 = 0.9; % end-point relative permeability to oil, -
k_rw_0 = 0.5; % end-point relative permeability to water, -
n_g    = 4; % Corey exponent for gas, -
n_og   = 4; % Corey exponent for oil in gas-oil flow, -
n_ow   = 4; % Corey exponent for oil in oil-water flow, -
n_w    = 4; % Corey exponent for water, -
S_gc   = 0.15; % critical gas saturation, -
S_or   = 0.10; % residual oil saturation, -
S_wi   = 0.20; % immobile water saturation, -

% Well data:
L_int = 3000; % AHD until kick-off point, m
L_tot = 4000; % total AHD (from tubing head), m 
alpha_top = 0; % wellbore inclination, top part, rad
alpha_bot = from_deg_to_rad(50); % wellbore inclination, bottom part, rad
d_w = 0.07600; % inside diameter of a 3.5 inch, 9.30 lbs/ft tubing, m
e_w = 30e-6; % tubing roughness, m

% Choke data:
choke_model = 1; % (extended) Gilbert (1954)
d_ch = from_in_to_m(28/64); % inside diameter, m

% Flow line data:
L_fl = 2000; % flow line length, m
alpha_fl = from_deg_to_rad(90); % flow line inclination, rad
d_fl = 0.10053; % inside diameter of a 4.5 inch flow line, m
e_fl = 30e-6; % flow line roughness, m

% Temperature data:
T_R = 95; % reservoir temperature, deg. C
T_wf = 95; % bottom hole temperature, deg. C
T_int = 90; % intermediate temperature, deg. C
T_tf = 85; % tubing head temperature, deg. C
T_fl = 80; % flow line entry temperature, deg. C
T_mf = 55; % manifold temperature, deg. C

% Plot data: 
n_pt = 50; % number of points in plot, -
q_o_sc_max = -3.0e-3; % maximum oil rate, m^3/s. 
% ---------------------------------------------------------------------------------------------
% End of input data
% ---------------------------------------------------------------------------------------------

% Compute auxiliary variables:
rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc];
delta_q_o_sc = q_o_sc_max/n_pt; % oil rate increment, m^3/s
rel = [k_rg_0,k_ro_0,k_rw_0,n_g,n_og,n_ow,n_w,S_gc,S_or,S_wi]; % rel perms, -
L_top = L_int; % lenght of well bore top section, m
L_bot = L_tot - L_int; % lenght of well bore bottom section, m

% Compute the traverse:
q_g_sc = R_go * q_o_sc; % gas rate, m^3/s
q_w_sc = (f_w/(1-f_w)) * q_o_sc; % water rate, m^3/s
q_sc = [q_g_sc,q_o_sc,q_w_sc];
% Compute pressure drop over the flow line:
p_mf_ds = p_mf; % manifold pressure for downstream calculations
[p_fl_ds,s_fl,p_fl] = pipe(alpha_fl,d_fl,e_fl,fluid,oil,p_mf_ds,q_sc,rho_sc,0,L_fl,T_mf,T_fl); % FLP
% Compute pressure drop over the choke:
p_tf_ds = choke_multi_phase_simp(choke_model,d_ch,p_fl_ds,q_sc); % THP
% Compute pressure drop over the wellbore (top part):
[p_int_ds,s_top,p_top] = pipe(alpha_top,d_w,e_w,fluid,oil,p_tf_ds,q_sc,rho_sc,0,L_top,...
    T_tf,T_int); % intermediate pressure for downstream calculations
% Compute pressure drop over the wellbore (bottom part):
[p_wf_ds,s_bot,p_bot] = pipe(alpha_bot,d_w,e_w,fluid,oil,p_int_ds,q_sc,rho_sc,0,L_bot,...
    T_int,T_wf); % BHP for downstream calculations
% Compute pressure drop over the reservoir:
[p_R_ds,s_res,p_res] = res(beta,fluid,h,k,oil,p_wf_ds,q_sc,r_w,r_e,rel,rho_sc,...
    T_wf); % Reservoir pressure for downstream calculations

% Assemble along-hole distances:
s_fl  = s_fl'; % row vector
s_top = s_top' + L_fl;
s_bot = s_bot' + L_fl + L_top;
s_res = s_res' + L_fl + L_top + L_bot + r_w;
s = [s_fl s_top s_bot s_res]; % distance from manifold, m

% Assemble pressures:
p_fl_tot  = p_fl(:,1)';
p_top_tot = p_top(:,1)';
p_bot_tot = p_bot(:,1)';
p_res_tot = p_res';
p = [p_fl_tot p_top_tot p_bot_tot p_res_tot]; % total pressure drop, Pa

% Compute pressure due to gravity:
n_fl = length(s_fl); % number of points in flow line pressure output, - 
n_res = length(s_res); % number of points in reservoir pressure output, - 
p_fl_gr  = zeros(1,n_fl); 
p_top_gr = (p_top(:,2)' - p_top(1,2)) + p_fl_gr(end); 
p_bot_gr = (p_bot(:,2)' - p_bot(1,2)) + p_top_gr(end); 
p_res_gr = zeros(1,n_res)             + p_bot_gr(end); 
p_gr = [p_fl_gr p_top_gr p_bot_gr p_res_gr]; % pressure due to gravity, Pa

% Compute pressure without gravity contribution:
p_fl_wog  = p_fl_tot - p_fl_gr; 
p_top_wog = p_top_tot - p_top_gr; 
p_bot_wog = p_bot_tot - p_bot_gr; 
p_res_wog = p_res_tot - p_res_gr;
p_wog = [p_fl_wog p_top_wog p_bot_wog p_res_wog]; % pressure w/o gravity, Pa

% Create simple plots:
% Plot total pressures: 
plot(s,p/1e6,'LineWidth',1)
grid on
xlabel('Distance From Manifold,\it s\rm (m)')
ylabel('Pressure,\it p\rm (MPa)')

% Plot pressures due to gravity: 
figure
plot(s,p_gr/1e6,'LineWidth',1)
grid on
xlabel('Distance From Manifold,\it s\rm (m)')
ylabel('Pressure Due To Gravity,\it p\rm (MPa)')

% Plot pressures w/o gravity: 
figure
plot(s,p_wog/1e6,'LineWidth',1)
grid on
xlabel('Distance From Manifold,\it s\rm (m)')
ylabel('Pressure W/O Gravity,\it p\rm (MPa)')
