% Script file answer_question_8_11
%
% Computes performance curves and a lift requirement curve for a system consisting of a
% near-well bore area, a deviated well, a well head choke and a flow line. The analysis node is
% at the bottom of the well.   
%
% JDJ, 07-01-14, last revised 29-05-17

clear variables
close all

% ---------------------------------------------------------------------------------------------
% Input data:
% ---------------------------------------------------------------------------------------------
% Production data:
f_w = 0.60; % water cut, - 
R_go =  20; % producing gas-oil ratio, m^3/m^3
p_mf = 0.5e6; % manifold pressure, Pa
p_R = 37e6; % reservoir pressure, Pa

% Fluid data:
oil = 1; % volatile oil; parameters read from file 'vol_oil_table_01'
R_sb = R_go; % solution GOR at bubble point, m^3/m^3 
rho_g_sc = 0.98; % gas density at standard conditions, kg/m^3
rho_o_sc = 865;  % oil density at standard conditions, kg/m^3
rho_w_sc = 1080; % water density at standard conditions, kg/m^3
fluid = 4; % multi-phase gas-oil-water flow, Mukherjee and Brill correlation, -

% Reservoir data:
beta = 0; % Forchheimer coefficient, m^-1 
h = 30; % reservoir hight, m
k = 1.8e-13; % permeability, m^2
r_e = 400; % reservoir drainage radius, m
r_w = 0.1778; % wellbore radius, m
        
% Rel perm data:
k_rg_0 = 0.7; % end-point relative permeability to gas, -
k_ro_0 = 0.9; % end-point relative permeability to oil, -
k_rw_0 = 0.5; % end-point relative permeability to water, -
n_g    = 4; % Corey exponent for gas, -
n_og   = 4; % Corey exponent for oil in gas-oil flow, -
n_ow   = 4; % Corey exponent for oil in oil-water flow, -
n_w    = 4; % Corey exponent for water, -
S_gc   = 0.15; % critical gas saturation, -
S_or   = 0.10; % residual oil saturation, -
S_wi   = 0.20; % immobile water saturation, -

% Well data:
L_int = 3000; % AHD until kick-off point, m
L_tot = 4000; % total AHD (from tubing head), m 
alpha_top = 0; % wellbore inclination, top part, rad
alpha_bot = from_deg_to_rad(50); % wellbore inclination, bottom part, rad
d_w = 0.07600; % inside diameter of a 3.5 inch, 9.30 lbs/ft tubing, m
e_w = 30e-6; % tubing roughness, m

% Flow line data:
L_fl = 2000; % flow line length, m
alpha_fl = from_deg_to_rad(90); % flow line inclination, rad
d_fl = 0.10053; % inside diameter of a 4.5 inch flow line, m
e_fl = 30e-6; % flow line roughness, m

% Temperature data:
T_R = 95; % reservoir temperature, deg. C
T_wf = 95; % bottom hole temperature, deg. C
T_int = 90; % intermediate temperature, deg. C
T_tf = 85; % tubing head temperature, deg. C
T_fl = 80; % flow line entry temperature, deg. C
T_mf = 55; % manifold temperature, deg. C

% Plot data: 
n_pt = 50; % number of points in plot, -
q_o_sc_max = -3.0e-3; % maximum oil rate, m^3/s. 
% ---------------------------------------------------------------------------------------------
% End of input data
% ---------------------------------------------------------------------------------------------

% Compute auxiliary variables:
rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc];
delta_q_o_sc = q_o_sc_max/n_pt; % oil rate increment, m^3/s
rel = [k_rg_0,k_ro_0,k_rw_0,n_g,n_og,n_ow,n_w,S_gc,S_or,S_wi]; % rel perms, -
L_top = L_int; % lenght of well bore top section, m
L_bot = L_tot - L_int; % lenght of well bore bottom section, m

% Initialize output:
results = zeros(n_pt,5);

tic % intializes timer 
for i=1:n_pt
    i
    q_o_sc = i * delta_q_o_sc; % oil rate, m^3/s 
    q_g_sc = R_go * q_o_sc; % gas rate, m^3/s
    q_w_sc = (f_w/(1-f_w)) * q_o_sc; % water rate, m^3/s
    q_sc = [q_g_sc,q_o_sc,q_w_sc];
    
    % 1) Compute upstream performance curve:
    % 1.1) Compute pressure drop over the reservoir:
    p_wf_us = res(beta,fluid,h,k,oil,p_R,q_sc,r_e,r_w,rel,rho_sc,T_R); % BHP
    results(i,1)= -q_o_sc; % flow rate
    results(i,2)= p_wf_us; % upstream FBHP

    % 2) Compute downstream performance curve:
    % 2.1) Compute pressure drop over the flow line:
    p_fl = pipe(alpha_fl,d_fl,e_fl,fluid,oil,p_mf,q_sc,rho_sc,0,L_fl,T_mf,T_fl); % FLP  
    % 2.2) Compute pressure drop over the wellbore (top part): 
    p_int = pipe(alpha_top,d_w,e_w,fluid,oil,p_fl,q_sc,rho_sc,0,L_top,T_tf,T_int);
    % 2.3) Compute pressure drop over the wellbore (bottom part): 
    p_wf_ds = pipe(alpha_bot,d_w,e_w,fluid,oil,p_int,q_sc,rho_sc,0,L_bot,...
        T_int,T_wf); % THP
    results(i,3)= p_wf_ds; % downstream FBHP 
    
    % 3) Compute lift requirement curve:
    p_wf_av = (p_wf_ds + p_wf_us) / 2; % average FBHP, Pa
    [q,rho] = local_q_and_rho(oil,p_wf_av,q_sc,R_go,rho_sc,T_wf);
    q_o = q(2);
    q_w = q(3);
    q_l = q_o + q_w; % local liquid rate, m^3/s
    rho_o = rho(2);
    rho_w = rho(3);
    rho_l = (rho_o*q_o + rho_w*q_w) / (q_o + q_w); % local liquid density, kg/m^3
    Delta_p_lift = p_wf_ds - p_wf_us; % lift requirement curve, Pa
    g = 9.81; % accelleration of gravity, m/s^2 
    H_lift = Delta_p_lift / (rho_l*g); % head requirement curve, m
    results(i,4)= -q_l; 
    results(i,5)= H_lift;   
end
toc % stops timer

% Create simple plots:
% Performance plots at bottom hole:
plot(results(:,1),results(:,2)/1e6,'LineWidth',1);
hold on
plot(results(:,1)*1e3,results(:,3)/1e6,'-r','LineWidth',1);
legend('Upstream','Downstream','Location','southwest')
hold off
%axis([0 3 15 45])
grid on  
xlabel('Oil Flowrate,\it -q_{o,sc}\rm (10^{-3} m^3/s)')
ylabel('FBHP,\it p_{wf}\rm MPa')

% Lift requirement curve:
figure
plot(results(:,4)*1e3,results(:,5),'LineWidth',1);
%axis([0 5 0 1000])
grid on  
xlabel('Local Liquid Flowrate,\it -q_l\rm (10^{-3} m^3/s)')
ylabel('Required Head Capacity,\it H\rm (m)')
