% Script file answer_question_8_12
%
% Computes performance curves for a gas-lifted offshore well. The analysis node is in the well
% bore at the depth of the gas lift valve. Creates a plot for different lift gas rates.     
%
% JDJ, 11-01-14, last revised 27-06-17

clear variables
close all

% ---------------------------------------------------------------------------------------------
% Input data:
% ---------------------------------------------------------------------------------------------
% Production data:
f_w = 0.00; % water cut, - 
R_sb = 20; % solution GOR at bubble point, m^3/m^3
p_1 = 0.5e6; % manifold pressure, Pa
p_6 = 26e6; % reservoir pressure, Pa
% Note: negative value because of sign convention for injection and production

% Fluid data:
fluid = 4; % multi-phase gas-oil-water flow, Mukherjee and Brill correlation, -
oil = 2; % black oil; Glaso correlations, -
rho_g_sc = 0.93; % gas density at standard conditions, kg/m^3
rho_o_sc = 845;  % oil density at standard conditions, kg/m^3
rho_w_sc = 1070; % water density at standard conditions, kg/m^3
c_l = 0.80; % specific heat capacity of liquid at constant volume, J/(kg K)
c_p = 0.25; % specific heat capacity of gas at constant pressure, J/(kg K)
c_v = 0.20; % specific heat capacity of gas at constant volume, J/(kg K)

% Reservoir data:
h = 40; % reservoir hight, m
k_o = 3e-14; % oil permeability, m^2
beta = 4.0e8; % Forchheimer coefficient, 1/m 
S = -1; % skin, -
r_e = 800; % reservoir drainage radius, m
r_w = 0.1778; % wellbore radius, m
semi = 1; % use semi-steady-state conditions
av = 0; % parameter to select reservoir pressure, -
        % av = 0: use reservoir pressure at external boundary
        % av = 1: use average reservoir pressure
        
% Well data:
s_tot = 3000; % total AHD (from tubing head), m
s_gl = 1600; % gas lift setting depth, m
alpha_t = 0; % tubing inclination, rad
d_t_i = 0.088; % inside diameter of a 4.0 inch, 11 lbs/ft tubing, m
e_t = 30e-6; % tubing roughness, m

% Oil flow line and riser data:
L_fl = 1000; % flow line length, m
alpha_fl = from_deg_to_rad(90); % flow line inclination, rad
d_fl = 0.10053; % inside diameter of a 4.5 inch flow line, m
e_fl = 30e-6; % flow line roughness, m
L_or = 120; % oil riser length, m 
alpha_or = 0; % riser inclination, rad
d_or = d_fl; % riser inside diameter, m
e_or = e_fl; % riser roughness, m

% Temperature data:
T_1 = 45; % oil manifold temperature, deg. C
T_2 = 50; % oil riser entry temperature, deg. C
T_3 = 90; % oil tubing head temperature, deg. C
T_4 = 92; % oil @ gas lift valve temperature, deg. C
T_5 = 95; % bottom hole temperature, deg. C
T_6 = 95; % reservoir temperature, deg. C

% Plot data: 
n_pt = 50; % number of points in plot, -
q_o_sc_max = -5.0e-3; % maximum oil rate, m^3/s. 
% Note: negative value because of sign convention for injection and production
% ---------------------------------------------------------------------------------------------
% End of input data
% ---------------------------------------------------------------------------------------------

% Compute auxiliary variables:
rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc];
delta_q_o_sc = q_o_sc_max/n_pt; % oil rate increment, m^3/s
L_top = s_gl; % distance between well head and gas lift valve, m
L_bot = s_tot - s_gl; % distance between gas lift valve and well bottom, m

% Initialize output:
results = zeros(n_pt,13);

tic % intializes timer
j_3 = 0; % counter for number of runs with R_go values within Glaso correlation limits 
j_4 = 0;
j_5 = 0;
j_6 = 0;
j_7 = 0;
j_8 = 0;
j_9 = 0;
j_10 = 0;
j_11 = 0;
j_12 = 0;
j_13 = 0;
for i=1:n_pt
    % Compute rates below and above gas lift valve:
    q_o_sc = i * delta_q_o_sc; % oil rate, m^3/s 
    q_w_sc = (f_w/(1-f_w)) * q_o_sc; % water rate, m^3/s
    q_g_sc_bot = R_sb * q_o_sc; % gas rate below the gas lift valve, m^3/s
    q_sc_bot = [q_g_sc_bot,q_o_sc,q_w_sc];
    
    % 1) Compute upstream performance curve:
    % 1.1) Compute pressure drop over the reservoir:
    p_5 = res_oil_simp(av,h,k_o,oil,p_6,q_sc_bot,r_e,r_w,rho_sc,semi,S,T_6); % FBHP
    p_4_us = pipe(alpha_t,d_t_i,e_t,fluid,oil,p_5,q_sc_bot,rho_sc,L_bot,0,T_5,T_4);
    results(i,1)= -q_o_sc; % flow rate
    results(i,2)= p_4_us; % upstream pressure at gas lift valve

    % 2) Compute downstream performance curves:
    q_g_sc_lift = 0; % lift gas rate, m^3/s
    q_g_sc_top = q_g_sc_bot + q_g_sc_lift; % gas rate above the gas lift valve, m^3/s
    q_sc_top = [q_g_sc_top,q_o_sc,q_w_sc];
    R_go = q_g_sc_top / q_o_sc; % producing GOR, m^3/m^3
    if R_go < 363 % Restrict to values that are within the limits of the Glaso correlation
        j_3 = j_3+1;
        % 2.1) Compute pressure drop over the oil riser:
        p_2 = pipe(alpha_or,d_or,e_or,fluid,oil,p_1,q_sc_top,rho_sc,0,L_or,T_1,T_2);
        % 2.2) Compute pressure drop over the oil flow line:
        p_3 = pipe(alpha_fl,d_fl,e_fl,fluid,oil,p_2,q_sc_top,rho_sc,0,L_fl,T_2,T_3);
        % 2.3) Compute pressure drop over the wellbore (from well head to gas lift valve):
        p_4_ds = pipe(alpha_t,d_t_i,e_t,fluid,oil,p_3,q_sc_top,rho_sc,0,L_top,T_3,T_4);
        results(i,3)= p_4_ds; % downstream pressure at gas lift valve
    end
    q_g_sc_lift = -5e-3; % lift gas rate, m^3/s
    q_g_sc_top = q_g_sc_bot + q_g_sc_lift; % gas rate above the gas lift valve, m^3/s
    q_sc_top = [q_g_sc_top,q_o_sc,q_w_sc];
    R_go = q_g_sc_top / q_o_sc; % producing GOR, m^3/m^3
    if R_go < 363 % Restrict to values that are within the limits of the Glaso correlation
        j_4 = j_4+1;
        % 2.1) Compute pressure drop over the oil riser:
        p_2 = pipe(alpha_or,d_or,e_or,fluid,oil,p_1,q_sc_top,rho_sc,0,L_or,T_1,T_2);
        % 2.2) Compute pressure drop over the oil flow line:
        p_3 = pipe(alpha_fl,d_fl,e_fl,fluid,oil,p_2,q_sc_top,rho_sc,0,L_fl,T_2,T_3);
        % 2.3) Compute pressure drop over the wellbore (from well head to gas lift valve):
        p_4_ds = pipe(alpha_t,d_t_i,e_t,fluid,oil,p_3,q_sc_top,rho_sc,0,L_top,T_3,T_4);
        results(i,4)= p_4_ds; % downstream pressure at gas lift valve
    end
    q_g_sc_lift = -10e-3; % lift gas rate, m^3/s
    q_g_sc_top = q_g_sc_bot + q_g_sc_lift; % gas rate above the gas lift valve, m^3/s
    q_sc_top = [q_g_sc_top,q_o_sc,q_w_sc];
    R_go = q_g_sc_top / q_o_sc; % producing GOR, m^3/m^3
    if R_go < 363 % Restrict to values that are within the limits of the Glaso correlation
        j_5 = j_5+1;
        % 2.1) Compute pressure drop over the oil riser:
        p_2 = pipe(alpha_or,d_or,e_or,fluid,oil,p_1,q_sc_top,rho_sc,0,L_or,T_1,T_2);
        % 2.2) Compute pressure drop over the oil flow line:
        p_3 = pipe(alpha_fl,d_fl,e_fl,fluid,oil,p_2,q_sc_top,rho_sc,0,L_fl,T_2,T_3);
        % 2.3) Compute pressure drop over the wellbore (from well head to gas lift valve):
        p_4_ds = pipe(alpha_t,d_t_i,e_t,fluid,oil,p_3,q_sc_top,rho_sc,0,L_top,T_3,T_4);
        results(i,5)= p_4_ds; % downstream pressure at gas lift valve
    end
    q_g_sc_lift = -20e-3; % lift gas rate, m^3/s
    q_g_sc_top = q_g_sc_bot + q_g_sc_lift; % gas rate above the gas lift valve, m^3/s
    q_sc_top = [q_g_sc_top,q_o_sc,q_w_sc];
    R_go = q_g_sc_top / q_o_sc; % producing GOR, m^3/m^3
    if R_go < 363 % Restrict to values that are within the limits of the Glaso correlation
        j_6 = j_6+1;
        % 2.1) Compute pressure drop over the oil riser:
        p_2 = pipe(alpha_or,d_or,e_or,fluid,oil,p_1,q_sc_top,rho_sc,0,L_or,T_1,T_2);
        % 2.2) Compute pressure drop over the oil flow line:
        p_3 = pipe(alpha_fl,d_fl,e_fl,fluid,oil,p_2,q_sc_top,rho_sc,0,L_fl,T_2,T_3);
        % 2.3) Compute pressure drop over the wellbore (from well head to gas lift valve):
        p_4_ds = pipe(alpha_t,d_t_i,e_t,fluid,oil,p_3,q_sc_top,rho_sc,0,L_top,T_3,T_4);
        results(i,6)= p_4_ds; % downstream pressure at gas lift valve
    end
    q_g_sc_lift = -50e-3; % lift gas rate, m^3/s
    q_g_sc_top = q_g_sc_bot + q_g_sc_lift; % gas rate above the gas lift valve, m^3/s
    q_sc_top = [q_g_sc_top,q_o_sc,q_w_sc];
    R_go = q_g_sc_top / q_o_sc; % producing GOR, m^3/m^3
    if R_go < 363 % Restrict to values that are within the limits of the Glaso correlation
        j_7 = j_7+1;
        % 2.1) Compute pressure drop over the oil riser:
        p_2 = pipe(alpha_or,d_or,e_or,fluid,oil,p_1,q_sc_top,rho_sc,0,L_or,T_1,T_2);
        % 2.2) Compute pressure drop over the oil flow line:
        p_3 = pipe(alpha_fl,d_fl,e_fl,fluid,oil,p_2,q_sc_top,rho_sc,0,L_fl,T_2,T_3);
        % 2.3) Compute pressure drop over the wellbore (from well head to gas lift valve):
        p_4_ds = pipe(alpha_t,d_t_i,e_t,fluid,oil,p_3,q_sc_top,rho_sc,0,L_top,T_3,T_4);
        results(i,7)= p_4_ds; % downstream pressure at gas lift valve
    end
    q_g_sc_lift = -100e-3; % lift gas rate, m^3/s
    q_g_sc_top = q_g_sc_bot + q_g_sc_lift; % gas rate above the gas lift valve, m^3/s
    q_sc_top = [q_g_sc_top,q_o_sc,q_w_sc];
    R_go = q_g_sc_top / q_o_sc; % producing GOR, m^3/m^3
    if R_go < 363 % Restrict to values that are within the limits of the Glaso correlation
        j_8 = j_8+1;
        % 2.1) Compute pressure drop over the oil riser:
        p_2 = pipe(alpha_or,d_or,e_or,fluid,oil,p_1,q_sc_top,rho_sc,0,L_or,T_1,T_2);
        % 2.2) Compute pressure drop over the oil flow line:
        p_3 = pipe(alpha_fl,d_fl,e_fl,fluid,oil,p_2,q_sc_top,rho_sc,0,L_fl,T_2,T_3);
        % 2.3) Compute pressure drop over the wellbore (from well head to gas lift valve):
        p_4_ds = pipe(alpha_t,d_t_i,e_t,fluid,oil,p_3,q_sc_top,rho_sc,0,L_top,T_3,T_4);
        results(i,8)= p_4_ds; % downstream pressure at gas lift valve
    end
    q_g_sc_lift = -200e-3; % lift gas rate, m^3/s
    q_g_sc_top = q_g_sc_bot + q_g_sc_lift; % gas rate above the gas lift valve, m^3/s
    q_sc_top = [q_g_sc_top,q_o_sc,q_w_sc];
    R_go = q_g_sc_top / q_o_sc; % producing GOR, m^3/m^3
    if R_go < 363 % Restrict to values that are within the limits of the Glaso correlation
        j_9 = j_9+1;
        % 2.1) Compute pressure drop over the oil riser:
        p_2 = pipe(alpha_or,d_or,e_or,fluid,oil,p_1,q_sc_top,rho_sc,0,L_or,T_1,T_2);
        % 2.2) Compute pressure drop over the oil flow line:
        p_3 = pipe(alpha_fl,d_fl,e_fl,fluid,oil,p_2,q_sc_top,rho_sc,0,L_fl,T_2,T_3);
        % 2.3) Compute pressure drop over the wellbore (from well head to gas lift valve):
        p_4_ds = pipe(alpha_t,d_t_i,e_t,fluid,oil,p_3,q_sc_top,rho_sc,0,L_top,T_3,T_4);
        results(i,9)= p_4_ds; % downstream pressure at gas lift valve
    end
    q_g_sc_lift = -500e-3; % lift gas rate, m^3/s
    q_g_sc_top = q_g_sc_bot + q_g_sc_lift; % gas rate above the gas lift valve, m^3/s
    q_sc_top = [q_g_sc_top,q_o_sc,q_w_sc];
    R_go = q_g_sc_top / q_o_sc; % producing GOR, m^3/m^3
    if R_go < 363 % Restrict to values that are within the limits of the Glaso correlation
        j_10 = j_10+1;
        % 2.1) Compute pressure drop over the oil riser:
        p_2 = pipe(alpha_or,d_or,e_or,fluid,oil,p_1,q_sc_top,rho_sc,0,L_or,T_1,T_2);
        % 2.2) Compute pressure drop over the oil flow line:
        p_3 = pipe(alpha_fl,d_fl,e_fl,fluid,oil,p_2,q_sc_top,rho_sc,0,L_fl,T_2,T_3);
        % 2.3) Compute pressure drop over the wellbore (from well head to gas lift valve):
        p_4_ds = pipe(alpha_t,d_t_i,e_t,fluid,oil,p_3,q_sc_top,rho_sc,0,L_top,T_3,T_4);
        results(i,10)= p_4_ds; % downstream pressure at gas lift valve
    end
    q_g_sc_lift = -900e-3; % lift gas rate, m^3/s
    q_g_sc_top = q_g_sc_bot + q_g_sc_lift; % gas rate above the gas lift valve, m^3/s
    q_sc_top = [q_g_sc_top,q_o_sc,q_w_sc];
    R_go = q_g_sc_top / q_o_sc; % producing GOR, m^3/m^3
    if R_go < 363 % Restrict to values that are within the limits of the Glaso correlation
        j_11 = j_11+1;
        % 2.1) Compute pressure drop over the oil riser:
        p_2 = pipe(alpha_or,d_or,e_or,fluid,oil,p_1,q_sc_top,rho_sc,0,L_or,T_1,T_2);
        % 2.2) Compute pressure drop over the oil flow line:
        p_3 = pipe(alpha_fl,d_fl,e_fl,fluid,oil,p_2,q_sc_top,rho_sc,0,L_fl,T_2,T_3);
        % 2.3) Compute pressure drop over the wellbore (from well head to gas lift valve):
        p_4_ds = pipe(alpha_t,d_t_i,e_t,fluid,oil,p_3,q_sc_top,rho_sc,0,L_top,T_3,T_4);
        results(i,11)= p_4_ds; % downstream pressure at gas lift valve
    end
    q_g_sc_lift = -1000e-3; % lift gas rate, m^3/s
    q_g_sc_top = q_g_sc_bot + q_g_sc_lift; % gas rate above the gas lift valve, m^3/s
    q_sc_top = [q_g_sc_top,q_o_sc,q_w_sc];
    R_go = q_g_sc_top / q_o_sc; % producing GOR, m^3/m^3
    if R_go < 363 % Restrict to values that are within the limits of the Glaso correlation
        j_12 = j_12+1;
        % 2.1) Compute pressure drop over the oil riser:
        p_2 = pipe(alpha_or,d_or,e_or,fluid,oil,p_1,q_sc_top,rho_sc,0,L_or,T_1,T_2);
        % 2.2) Compute pressure drop over the oil flow line:
        p_3 = pipe(alpha_fl,d_fl,e_fl,fluid,oil,p_2,q_sc_top,rho_sc,0,L_fl,T_2,T_3);
        % 2.3) Compute pressure drop over the wellbore (from well head to gas lift valve):
        p_4_ds = pipe(alpha_t,d_t_i,e_t,fluid,oil,p_3,q_sc_top,rho_sc,0,L_top,T_3,T_4);
        results(i,12)= p_4_ds; % downstream pressure at gas lift valve
    end  
    q_g_sc_lift = -1100e-3; % lift gas rate, m^3/s
    q_g_sc_top = q_g_sc_bot + q_g_sc_lift; % gas rate above the gas lift valve, m^3/s
    q_sc_top = [q_g_sc_top,q_o_sc,q_w_sc];
    R_go = q_g_sc_top / q_o_sc; % producing GOR, m^3/m^3
    if R_go < 363 % Restrict to values that are within the limits of the Glaso correlation
        j_13 = j_13+1;
        % 2.1) Compute pressure drop over the oil riser:
        p_2 = pipe(alpha_or,d_or,e_or,fluid,oil,p_1,q_sc_top,rho_sc,0,L_or,T_1,T_2);
        % 2.2) Compute pressure drop over the oil flow line:
        p_3 = pipe(alpha_fl,d_fl,e_fl,fluid,oil,p_2,q_sc_top,rho_sc,0,L_fl,T_2,T_3);
        % 2.3) Compute pressure drop over the wellbore (from well head to gas lift valve):
        p_4_ds = pipe(alpha_t,d_t_i,e_t,fluid,oil,p_3,q_sc_top,rho_sc,0,L_top,T_3,T_4);
        results(i,13)= p_4_ds; % downstream pressure at gas lift valve
    end  
end
toc % stops timer

% Performance plots at bottom hole:
plot(results(:,1)*1e3,results(:,2)/1e6);
hold on
plot(results(n_pt-(j_3-1):n_pt,1)*1e3,results(n_pt-(j_3-1):n_pt,3)/1e6,'-r','LineWidth',1);
plot(results(n_pt-(j_4-1):n_pt,1)*1e3,results(n_pt-(j_4-1):n_pt,4)/1e6,'-r','LineWidth',1);
plot(results(n_pt-(j_5-1):n_pt,1)*1e3,results(n_pt-(j_5-1):n_pt,5)/1e6,'-r','LineWidth',1);
plot(results(n_pt-(j_6-1):n_pt,1)*1e3,results(n_pt-(j_6-1):n_pt,6)/1e6,'-r','LineWidth',1);
plot(results(n_pt-(j_7-1):n_pt,1)*1e3,results(n_pt-(j_7-1):n_pt,7)/1e6,'-r','LineWidth',1);
plot(results(n_pt-(j_8-1):n_pt,1)*1e3,results(n_pt-(j_8-1):n_pt,8)/1e6,'-r','LineWidth',1);
plot(results(n_pt-(j_9-1):n_pt,1)*1e3,results(n_pt-(j_9-1):n_pt,9)/1e6,'-r','LineWidth',1);
plot(results(n_pt-(j_10-1):n_pt,1)*1e3,results(n_pt-(j_10-1):n_pt,10)/1e6,'-r','LineWidth',1);
plot(results(n_pt-(j_12-1):n_pt,1)*1e3,results(n_pt-(j_12-1):n_pt,12)/1e6,'-r','LineWidth',1);
legend('Upstream','Downstream','Location','southwest')
hold off
axis([0 5 6 14])
grid on  
xlabel('Oil Flowrate,\it -q_{o,sc}\rm (10^{-3} m^3/s)')
ylabel('Wellbore Pressure at Gas Lift Valve,\it p_4\rm (MPa)')

figure
plot(results(:,1)*1e3,results(:,2)/1e6);
hold on
plot(results(n_pt-(j_11-1):n_pt,1)*1e3,results(n_pt-(j_11-1):n_pt,11)/1e6,':r','LineWidth',1);
plot(results(n_pt-(j_12-1):n_pt,1)*1e3,results(n_pt-(j_12-1):n_pt,12)/1e6,'-r','LineWidth',1);
plot(results(n_pt-(j_13-1):n_pt,1)*1e3,results(n_pt-(j_13-1):n_pt,13)/1e6,'-.r','LineWidth',1);
legend('Upstream','\itq_{g,sc,\rmlift} = \rm0.9 m^3/s','1.0 m^3/s','1.1 m^3/s','Location','northeast')
hold off
axis([4.34 4.37 6.98 7.04])
grid on  
xlabel('Oil Flowrate,\it -q_{o,sc}\rm (10^{-3} m^3/s)')
ylabel('Wellbore Pressure at Gas Lift Valve,\it p_4\rm (MPa)')
