function E = error_drdp(beta,k,k_r,mu,multi,rel,rho_g,v)
% function E = error_drdp(beta,k,k_r,mu,multi,rel,rho_g,v)
%
% Computes the error in the inverse pressure gradients as a function of saturations. Auxiliary
% function for function saturations.m.      

% beta = Forchheimer coefficient, m^-1 
% E = [E_go,E_ow]
%    E_go = error between dp/dr for gas and dp/dr for oil, m/Pa
%    E_ow = error between dp/dr for oil and dp/dr for water, m/Pa
% k = absolute permeability, m^2
% k_r = [k_rg,k_rog,k_row,k_rw]
%   k_rg = relative permeability to gas, -
%   k_rog = relative permeability to oil in gas-oil flow, -
%   k_row = relative permeability to oil in oil-water flow, -
%   k_rw = relative permeability to water, -
% mu = [mu_g,mu_o,mu_w]
%   mu_g = gas viscosity, Pa s 
%   mu_o = oil viscosity, Pa s 
%   mu_w = water viscosity, Pa s
% multi = parameter to indicate flow
%   multi = 1: two-phase oil/water flow
%   multi = 2: two-phase gas/oil flow
%   multi = 3: multi-phase gas/oil/water flow
% rel = [k_rg_0,k_ro_0,k_rw_0,n_g,n_og,n_ow,n_w,S_gc,S_or,S_wc]
%   k_rg_0 = end-point relative permeability to gas, -
%   k_ro_0 = end-point relative permeability to oil, -
%   k_rw_0 = end-point relative permeability to water, -
%   n_g = Corey exponent for gas, -
%   n_og = Corey exponent for oil in gas-oil flow, -
%   n_ow = Corey exponent for oil in oil-water flow, -
%   n_w = Corey exponent for water, -
%   S_gc = critical gas saturation, -
%   S_or = residual oil saturation, -
%   S_wi = immobile water saturation, -
% rho_g = local gas density, kg/m^3
% v = [v_g,v_o,v_w]
%   v_g = gas velocity, m/s 
%   v_o = oil velocity, m/s 
%   v_w = water velocity, m/s 
%
% JDJ, 28-01-05, last revised 22-01-14

k_rg = k_r(1);
k_rog = k_r(2);
k_row = k_r(3);
k_rw = k_r(4);
k_ro_0 = rel(2);
k_ro = k_ro_0*((k_rog/k_ro_0+k_rg)*(k_row/k_ro_0+k_rw)-(k_rg+k_rw));
if k_ro < 0
    k_ro = eps;
end
k_g = k*k_rg;
k_o = k*k_ro;
k_w = k*k_rw;
switch multi
    case 1
        drdp_o = -k_o/(v(2)*mu(2));
        drdp_w = -k_w/(v(3)*mu(3));
        E_go = 0;
        E_ow = drdp_o - drdp_w;
    case 2
        drdp_g = -1/( v(1)*mu(1)/k_g + beta*rho_g*v(1)*abs(v(1)) );
        drdp_o = -k_o/(v(2)*mu(2));
        E_go = drdp_g - drdp_o;
        E_ow = 0;
    case 3
        drdp_g = -1/( v(1)*mu(1)/k_g + beta*rho_g*v(1)*abs(v(1)) );
        drdp_o = -k_o/(v(2)*mu(2));
        drdp_w = -k_w/(v(3)*mu(3));
        E_go = drdp_g - drdp_o;
        E_ow = drdp_o - drdp_w;
end
E = [E_go E_ow];
