% Script file example_IPR
%
% Example to create an inflow performance relationship for a vertical well producting from a
% circular drainage area. 
%
% JDJ, 13-01-05, last revised 29-05-17

clear variables
close all

% ---------------------------------------------------------------------------------------------
% Input data:
% ---------------------------------------------------------------------------------------------
av         = 0; % use reservoir pressure at external boundary
beta       = 0; % Forcheimer coefficient, m^-1. Not relevant for fluid 1.
f_w_sc     = 0.0; % = q_w_sc/(q_w_sc+q_o_sc) = water cut, -. Not relevant for fluids 1 and 2
fluid      = 3; % fluid  = 1: single-phase oil flow
                % fluid  = 2: single-phase gas flow
                % fluid >= 3: multi-phase gas-oil-water flow
h          = 20.0; % reservoir height, m
k          = 1.11e-13; % effective permeability for fluids 1 and 2, or absolute permeability
                       % for fluid 3, m^2
n_pt       = 100; % number of points in plot, - 
oil = 1; % parameter to select black oil model or volatile oil table, -. Not relevant for
%          fluids 1 and 2
%          oil = 1: black oil; parameters computed with the aid of Standing correlations
%          oil = 2: black oil; parameters computed with the aid of Glaso correlations
%          oil = 3: volatile oil; parameters read from file 'vol_oil_table_01'
p_R        = 30.0e6 % reservoir pressure, Pa
q_o_sc_max = -5e-2; % maximum oil flow rate, m^3/s
r_e        = 500; % external radius, m
r_w        = 0.2; % well bore radius, m
R_go       = 200; % producing GOR as observed at surface, m^3/m^3. Not relevant for fluid 2
rho_g_sc   = 0.95; % gas density at standard conditions, kg/m^3. 
rho_o_sc   = 850;  % oil density at st. conditions, kg/m^3. Not relevant for fluid 2
rho_w_sc   = 1000; % water density at st. conditions, kg/m^3. Not relevant for fluids 1 and 2
semi       = 0; % semi = 0: use steady-state conditions
%                 semi = 1: use semi-steady-state conditions
simp       = 1; % simp = 0: numerical solution using Runge Kutta integration
%                 simp = 1: simplified (semi-)analytical solution
S          = 0; % skin, -
T_R        = 60; % reservoir temperature, deg. C

% Relative permeability data (not relevant for fluids 1 and 2):
k_rg_0  = 0.7; % end-point relative permeability to gas, -
k_ro_0  = 0.9; % end-point relative permeability to oil, -
k_rw_0  = 0.5; % end-point relative permeability to water, -
n_g     = 3; % Corey exponent for gas, -
n_og    = 3; % Corey exponent for oil in gas-oil flow, -
n_ow    = 3; % Corey exponent for oil in oil-water flow, -
n_w     = 3; % Corey exponent for water, -
S_gc    = 0.00; % critical gas saturation, -
S_or    = 0.10; % residual oil saturation, -
S_wi    = 0.15; % immobile water saturation, -
% --------------------------------------------------------------------------------------------
% End of input data
% ---------------------------------------------------------------------------------------------

% Check input:
if (simp == 1 && beta ~= 0)
    warning('Forcheimer flow only available numerically.')
    warning('Parameter "simp" reset to 0.')
    simp = 0;
end
if (simp == 0 && semi == 1)
    warning('Semi-steady state solution only available analytically.')
    warning('Parameter "semi" reset to 0.')
    semi = 0;
end
if (simp == 0 && av == 1)
    warning('Average reservoir pressure solution only available analytically.')
    warning('Parameter "av" reset to 0.')
    av = 0;
end

% Create data vectors:
rel = [k_rg_0,k_ro_0,k_rw_0,n_g,n_og,n_ow,n_w,S_gc,S_or,S_wi];
rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc];

% Compute and print bubble point pressure (for info only):
if fluid ~= 2
    switch oil
        case 1
            p_b = pres_bub_Standing(R_go,rho_g_sc,rho_o_sc,T_R)
        case 2
            p_b = pres_bub_Glaso(R_go,rho_g_sc,rho_o_sc,T_R)
        case 3
            p_b = pres_bub_volatile_oil(T_R)
    end
end
    
% Compute and plot IPR:
switch fluid
    case 1 % oil
        delta_q_o_sc = q_o_sc_max/n_pt; % oil rate increment, m^3/s
        p_wf = p_R;
        results = zeros(n_pt,2);
        for i=1:n_pt 
            q_o_sc = i * delta_q_o_sc; % oil rate, m^3/s
            q_g_sc = 0; % gas rate, m^3/s
            q_w_sc = 0; % water rate, m^3/s
            q_sc = [q_g_sc,q_o_sc,q_w_sc];
            if simp == 0 % numerical solution
                p_old = p_wf;
                p_wf = res(beta,fluid,h,k,oil,p_R,q_sc,r_e,r_w,rel,rho_sc,T_R);
                Delta_p = p_old - p_wf;
                results(i,1:2)= [-q_o_sc p_wf];
                if p_wf < 1e5 + Delta_p % to avoid pressures below atmospheric
                    break
                end
            else % simple semi-analytical solution
                p_old = p_wf;
                p_wf = res_oil_simp(av,h,k,oil,p_R,q_sc,r_e,r_w,rho_sc,semi,S,T_R);
                Delta_p = p_old - p_wf;
                results(i,1:2)= [-q_o_sc p_wf];
                if p_wf < 1e5 + Delta_p % to avoid pressures below atmospheric
                    break
                end
            end
        end
        nonzero_rows = results(:,1) ~= 0; % select non-zero rows
        results = results(nonzero_rows,:); % remove zero rows
        plot(results(:,1),results(:,2))
        xlabel('Oil flow rate\it -q_{o,sc} ,\rm m^3/s')
        ylabel('Flowing bottom-hole pressure\it p_{wf} ,\rm Pa')
        grid on
    case 2 % gas
        delta_q_g_sc = q_g_sc_max/n_pt; % gas rate increment, m^3/s
        p_wf = p_R;
        results = zeros(n_pt,2);
        for i=1:n_pt
            q_g_sc = i * delta_q_g_sc; % gas rate, m^3/s
            q_o_sc = 0; % oil rate, m3/s
            q_w_sc = 0; % water rate, m3/s
            q_sc = [q_g_sc,q_o_sc,q_w_sc];
            if simp == 0 % numerical solution
                p_old = p_wf;
                p_wf = res(beta,fluid,h,k,oil,p_R,q_sc,r_e,r_w,rel,rho_sc,T_R);
                Delta_p = p_old - p_wf;
                results(i,1:2)= [-q_g_sc p_wf];
                if p_wf < 1e5 + Delta_p % to avoid pressures below atmospheric
                    break
                end
            else % simple semi-analytical solution
                p_old = p_wf;
                p_wf = res_gas_simp(av,h,k,p_R,q_sc,r_e,r_w,rho_sc,semi,S,T_R);
                Delta_p = p_old - p_wf;
                results(i,1:2)= [-q_g_sc p_wf];
                if p_wf < 1e5 + Delta_p % to avoid pressures below atmospheric
                    break
                end
            end
        end
        nonzero_rows = results(:,1) ~= 0; % select non-zero rows
        results = results(nonzero_rows,:); % remove zero rows
        plot(results(:,1),results(:,2))
        xlabel('Gas flow rate\it -q_{g,sc} ,\rm m^3/s')
        ylabel('Flowing bottom-hole pressure\it p_{wf} ,\rm Pa')
        grid on
    otherwise % multi-phase       
        delta_q_o_sc = q_o_sc_max/n_pt; % oil rate increment, m^3/s
        p_wf = p_R;
        results = zeros(n_pt,2);
        for i=1:n_pt
            q_o_sc = i * delta_q_o_sc % oil rate, m^3/s
            q_g_sc = R_go * q_o_sc; % gas rate, m^3/s
            q_w_sc = (f_w_sc/(1-f_w_sc)) * q_o_sc; % water rate, m^3/s
            q_sc = [q_g_sc,q_o_sc,q_w_sc];
            if simp == 0 % numerical solution
                p_old = p_wf;
                p_wf = res(beta,fluid,h,k,oil,p_R,q_sc,r_e,r_w,rel,rho_sc,T_R);
                Delta_p = p_old - p_wf;
                results(i,1:2)= [-q_o_sc p_wf];
                if p_wf < 1e5 + Delta_p % to avoid pressures below atmospheric
                    break
                end
            else % Vogel solution
                p_old = p_wf;
                p_wf = res_Vogel(av,h,k,oil,p_R,q_sc,r_e,r_w,rel,rho_sc,semi,S,T_R);
                Delta_p = p_old - p_wf;
                results(i,1:2)= [-q_o_sc p_wf];
                if p_wf < 1e5 + Delta_p % to avoid pressures below atmospheric
                    break
                end
            end
        end
        nonzero_rows = results(:,1) ~= 0; % select non-zero rows
        results = results(nonzero_rows,:); % remove zero rows
        plot(results(:,1)*1e3,results(:,2)/1e6,'LineWidth',1)
        xlabel('Oil Flowrate,\it -q_{o,sc}\rm (10^{-3} m^3/s)')
        ylabel('FBHP,\it p_{wf}\rm (MPa)')
        grid on
end
