% Script file example_PI_oil_hor
%
% Example to compute a plot of the PI of horizontal well in a box-shaped or an elliptical
% reservoir.
%
% JDJ, 10-01-16, last revised 29-05-17

clear variables
close all

% ---------------------------------------------------------------------------------------------
% Input data:
% ---------------------------------------------------------------------------------------------
av         = 0; % av = 0: use reservoir pressure at external boundary
%                 av = 1: use average reservoir pressure (for box-shaped reservoir only)
flag_res   = 2; % flag_res = 1: box-shaped reservoir
%                 flag_res = 2: elliptical reservoir
h          = 20.0; % reservoir height, m
k_o_h      = 0.8e-13; % effective horizontal permeability to oil.
k_o_v      = 0.4e-13; % effective vertical permeability to oil.
L          = 2000; % well length, m
n_pt       = 100; % number of points in plot, -
oil        = 1; % black oil; parameters computed with the aid of Standing correlations
p_R        = 30e6; % reservoir pressure, Pa
q_o_sc_max = -4e-2; % maximum oil flow rate (negative for production), m^3/s
r_w        = 0.20; % well bore radius, m
rho_o_sc   = 850;  % oil density at st. conditions, kg/m^3.
rho_g_sc   = 0.95; % gas density at standard conditions, kg/m^3. Not relevant.
rho_w_sc   = 1050; % water density at st. conditions, kg/m^3. Not relevant.
R_sb       = 5; % solution GOR, m^3/m^3
semi       = 0; % semi = 0: use steady-state conditions
%                 semi = 1: use semi-steady-state conditions (for box-shaped reservoir only)
S_v        = 0; % skin factor as measured in a vertical well, -
T_R        = 90; % reservoir temperature, deg. C
w          = 500; % reservoir width, m
% ---------------------------------------------------------------------------------------------
% End of input data
% ---------------------------------------------------------------------------------------------

% Create data vectors:
rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc];

% Compute and plot PI:
delta_q_o_sc = q_o_sc_max/n_pt; % oil rate increment, m^3/s
p_wf = p_R;
results = zeros(n_pt,2);
for i=1:n_pt
    q_o_sc = i * delta_q_o_sc; % oil rate, m^3/s
    q_g_sc = R_sb * delta_q_o_sc; % gas rate, m^3/s
    q_w_sc = 0; % water rate, m^3/s
    q_sc = [q_g_sc,q_o_sc,q_w_sc];
    p_old = p_wf;
    if flag_res == 1
        p_wf = res_oil_box(av,h,k_o_h,k_o_v,L,oil,p_R,q_sc,rho_sc,semi,S_v,T_R,w/2,r_w);
    else
        p_wf = res_oil_ellipse(h,k_o_h,k_o_v,L,oil,p_R,q_sc,rho_sc,S_v,T_R,w/2,r_w);
    end
    Delta_p = p_old - p_wf;
    results(i,1:2)= [-q_o_sc p_wf];
    if p_wf < 1e5 + Delta_p % to avoid pressures below atmospheric
        break
    end
end
nonzero_rows = results(:,1) ~= 0; % select non-zero rows
results = results(nonzero_rows,:); % remove zero rows
plot(results(:,1)*1e3,results(:,2)/1e6,'LineWidth',1)
xlabel('Oil Flowrate,\it -q_{o,sc}\rm (10^{-3} m^3/s)')
ylabel('FBHP,\it p_{wf}\rm (MPa)')
grid on

