% Script file example_res_pres
%
% Example to create a plot of the near-well reservoir pressure versus radial distance in a
% vertical well.  
%
% JDJ, 13-01-05, last revised 29-05-17

clear variables
close all

% ---------------------------------------------------------------------------------------------
% Input data:
% ---------------------------------------------------------------------------------------------

av         = 0; % use reservoir pressure at external boundary
beta       = 3.5e10; % Forcheimer coefficient, m^-1. Not relevant for fluid 1.
f_w_sc     = 0.0; % = q_w_sc/(q_w_sc+q_o_sc) = water cut, -. Not relevant for fluids 1 and 2
fluid      = 4; % multi-phase gas-oil-water flow
h          = 20.0; % reservoir height, m
k          = 1.11e-13; % effective permeability for fluids 1 and 2, or absolute permeability
                       % for fluid 3, m^2
n_pt       = 200; % number of points in plot, - 
oil        = 1; % Standing black oil correlations, -                        
p_R        = 9.3e6; % reservoir pressure, Pa
q_o_sc     = -10e-3; % oil flow rate, m^3/s
r_e        = 500; % external radius, m
r_w        = 0.2; % well bore radius, m
R_sb       = 50; % solution GOR, m^3/m^3
rho_g_sc   = 0.95; % gas density at standard conditions, kg/m^3. 
rho_o_sc   = 850;  % oil density at st. conditions, kg/m^3. Not relevant for fluid 2
rho_w_sc   = 1050; % water density at st. conditions, kg/m^3. Not relevant for fluids 1 and 2
semi       = 0; % semi = 0: use steady-state conditions
%                 semi = 1: use semi-steady-state conditions
simp       = 0; % simp = 0: numerical solution using Runge Kutta integration
%                 simp = 1: simplified (semi-)analytical solution
S          = 0; % skin, -
T_R        = 90; % reservoir temperature, deg. C

% Relative permeability data (not relevant for fluids 1 and 2):
k_rg_0 = 0.7; % end-point relative permeability to gas, -
k_ro_0 = 0.9; % end-point relative permeability to oil, -
k_rw_0 = 0.5; % end-point relative permeability to water, -
n_g = 3; % Corey exponent for gas, -
n_og = 3; % Corey exponent for oil in gas-oil flow, -
n_ow = 3; % Corey exponent for oil in oil-water flow, -
n_w = 3; % Corey exponent for water, -
S_gc = 0.05; % critical gas saturation, -
S_or = 0.10; % residual oil saturation, -
S_wi = 0.15; % immobile water saturation, -

% ---------------------------------------------------------------------------------------------
% End of input data
% ---------------------------------------------------------------------------------------------

% Check input:
if (simp == 1 && beta ~= 0)
    warning('Forcheimer flow only available numerically.')
    warning('Parameter "simp" reset to 0.')
    simp = 0;
end
if (simp == 0 && semi == 1)
    warning('Semi-steady state solution only available analytically.')
    warning('Parameter "semi" reset to 0.')
    semi = 0;
end
if (simp == 0 && av == 1)
    warning('Average reservoir pressure solution only available analytically.')
    warning('Parameter "av" reset to 0.')
    av = 0;
end

% Create data vectors:
rel = [k_rg_0,k_ro_0,k_rw_0,n_g,n_og,n_ow,n_w,S_gc,S_or,S_wi];
rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc];
R_go = R_sb;

% Compute bubble point pressure (printed for info only):
if fluid ~= 2 % not single-phase gas
    switch oil
        case 1
            p_b = pres_bub_Standing(R_go,rho_g_sc,rho_o_sc,T_R);
        case 2
            p_b = pres_bub_Glaso(R_go,rho_g_sc,rho_o_sc,T_R);
        case 3
            p_b = pres_bub_volatile_oil(T_R);
    end
end

% Compute pressure:
switch fluid
    case 1 % oil
        q_g_sc = R_go * q_o_sc; % gas rate, m^3/s
        q_w_sc = 0; % water rate, m^3/s
        q_sc = [q_g_sc,q_o_sc,q_w_sc];
        if simp == 0 % numerical solution
            [p_wf,r,p] = res(beta,fluid,h,k,oil,p_R,q_sc,r_e,r_w,rel,rho_sc,T_R);
        else % simple semi-analytical solution
            [p_wf,r,p] = res_oil_simp(av,h,k,oil,p_R,q_sc,r_e,r_w,rho_sc,semi,S,T_R);
        end
    case 2 % gas
        q_o_sc = 0; % oil rate, m3/s
        q_w_sc = 0; % water rate, m3/s
        q_sc = [q_g_sc,q_o_sc,q_w_sc];
        if simp == 0 % numerical solution
            [p_wf,r,p] = res(beta,fluid,h,k,oil,p_R,q_sc,r_e,r_w,rel,rho_sc,T_R);
        else % simple semi-analytical solution
            [p_wf,r,p] = res_gas_simp(av,h,k,p_R,q_sc,r_e,r_w,rho_sc,semi,S,T_R);
        end
    otherwise % multi-phase
        q_g_sc = R_go * q_o_sc; % gas rate, m^3/s
        q_w_sc = (f_w_sc/(1-f_w_sc)) * q_o_sc; % water rate, m^3/s
        q_sc = [q_g_sc,q_o_sc,q_w_sc];
        if simp == 0 % numerical solution
            [p_wf,r,p] = res(beta,fluid,h,k,oil,p_R,q_sc,r_e,r_w,rel,rho_sc,T_R);
        else % simple semi-analytical solution
            p_wf = res_Vogel(av,h,k,oil,p_R,q_sc,r_e,r_w,rel,rho_sc,semi,S,T_R);
        end
end

if fluid > 2 && simp == 1 % Vogel solution, no plots
    fprintf('No plots for Vogel solution\n'); 
else
    % Create plots:
    if r(end) < r(1) % checks if the elements of r are ascending
        r = r(end:-1:1,1); % reverses the element order in r
        p = p(end:-1:1,1); 
    end
    plot(r,p/1e6,'LineWidth',1)
    xlabel('Radial Distance,\it r\rm (m)')
    ylabel('Pressure,\it p\rm (MPa)')
    grid on
    
    if fluid > 2 % only relevant for multi-phase flow
        n_pt = length(r); % number of plot points
        S_g = zeros(n_pt,1);
        S_o = zeros(n_pt,1);
        S_w = zeros(n_pt,1);
        for i = 1:n_pt
            % Compute analytical two-phase approximations as starting values:
            pp = p(i);
            rr = r(i);
            sat_go = saturations_gas_oil(oil,pp,q_sc,rel,rho_sc,T_R);
            sat_ow = saturations_oil_water(oil,pp,q_sc,rel,rho_sc,T_R);
            start = [sat_go(1),sat_ow(3)]; % starting values for S_g and S_w
            [n_iter,sat] = saturations(beta,h,k,oil,pp,q_sc,rr,rel,rho_sc,start,T_R);
            S_g(i) = sat(1);
            S_o(i) = sat(2);
            S_w(i) = sat(3);
        end
        figure
        plot(r,S_g,'-g','LineWidth',1)
        hold on
        plot(r,S_o,'-r','LineWidth',1)
        plot(r,S_w,'-b','LineWidth',1)
        hold off
        xlabel('Radial Distance,\it r\rm (m)')
        ylabel('Saturation,\it S\rm (-)')
        legend('\itS_g','\itS_o','\itS_w','location','NorthWest')
        axis([0,r_e,0,1])
    end
end

% Print output:
q_o_sc
p_R
p_wf
p_b % for info only

% Print output in field units:
q_o_sc_FU = from_m3_per_s_to_bpd(q_o_sc)
p_R_FU = from_Pa_to_psi(p_R)
p_wf_FU = from_Pa_to_psi(p_wf)
p_b = from_Pa_to_psi(p_b);
