% Script file example_res_pres_oil_hor
%
% Example to compute a plot of the reservoir pressure versus horizontal distance perpendicular
% to the well for single-phase oil flow towards a horizontal well in a box-shaped or an
% elliptical reservoir.
%
% JDJ, 09-01-16, last revised 29-05-17

clear variables
close all

% ---------------------------------------------------------------------------------------------
% Input data:
% ---------------------------------------------------------------------------------------------
av       = 0; % av = 0: use reservoir pressure at external boundary
%               av = 1: use average reservoir pressure (for box-shaped reservoir only)
flag_res = 2; % flag_res = 1: box-shaped reservoir
%               flag_res = 2: elliptical reservoir
h        = 20.0; % reservoir height, m
k_o_h    = 0.8e-13; % effective horizontal permeability to oil.
k_o_v    = 0.4e-13; % effective vertical permeability to oil.
L        = 2000; % well length, m
n_pt     = 100; % number of points in plot, -
oil      = 1; % black oil; parameters computed with the aid of Standing correlations
p_R      = 30e6; % reservoir pressure, Pa
q_o_sc   = -0.01; % oil flow rate at standard conditions (negative for production), m^3/s
r_w      = 0.20; % well bore radius, m
rho_o_sc = 850;  % oil density at st. conditions, kg/m^3.
rho_g_sc = 0.95; % gas density at standard conditions, kg/m^3. Not relevant.
rho_w_sc = 1050; % water density at st. conditions, kg/m^3. Not relevant.
R_sb     = 5; % solution GOR, m^3/m^3
semi     = 0; % semi = 0: use steady-state conditions
%               semi = 1: use semi-steady-state conditions (for box-shaped reservoir only)
S_v      = 0; % skin factor as measured in a vertical well, -
T_R      = 90; % reservoir temperature, deg. C
w        = 500; % reservoir width, m
% ---------------------------------------------------------------------------------------------
% End of input data
% ---------------------------------------------------------------------------------------------

% Create data vectors:
q_g_sc = R_sb * q_o_sc;
q_w_sc = 0;
q_sc = [q_w_sc,q_o_sc,q_w_sc];
rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc];

% Compute pressure:
if flag_res == 1
    [p_wf,y,p,J] = res_oil_box(av,h,k_o_h,k_o_v,L,oil,p_R,q_sc,rho_sc,semi,S_v,T_R,w/2,r_w);
else
    [p_wf,y,p,J] = res_oil_ellipse(h,k_o_h,k_o_v,L,oil,p_R,q_sc,rho_sc,S_v,T_R,w/2,r_w);
end
% Create plots:
figure
plot(y,p/1e6,'LineWidth',1)
hold on
plot(y(1),p(1)/1e6,'bo','MarkerFaceColor','r');
plot(y(2),p(2)/1e6,'bo','MarkerFaceColor','r');
plot(y(end),p(end)/1e6,'bo','MarkerFaceColor','r');
hold off
xlabel('Horizontal Distance From Well,\it y\rm (m)')
ylabel('Pressure,\it p\rm (MPa)')
grid on

% Print output:
q_o_sc
p_R
p_wf
J

% Print output in field units:
q_o_sc_FU = from_m3_per_s_to_bpd(q_o_sc)
p_R_FU = from_Pa_to_psi(p_R)
p_wf_FU = from_Pa_to_psi(p_wf)
J_FU = from_m3_per_s_Pa_to_bpd_psi(J)
