function [lambda_g,lambda_o,lambda_w] = mobilities(k,p,q_sc,rel,rho_sc,sat,T_R)
% function [lambda_g,lambda_o,lambda_w] = mobilities(k,p,q_sc,rel,rho_sc,sat,T_R)
%
% Computes the mobilities for a given saturation
%
% JDJ, 12-03-11, last revised 18-02-15
%
% k = permeability, m^2
% p = pressure, Pa
% q_sc = [q_g_sc,q_o_sc,q_w_sc], m^3/s
%   q_g_sc = gas flow rate at standard conditions, m^3/s.
%   q_o_sc = oil flow rate at standard conditions, m^3/s.
%   q_w_sc = water flow rate at standard conditions, m^3/s. Not relevant.
% rel = [k_rg_0,k_ro_0,k_rw_0,n_g,n_og,n_ow,n_w,S_gc,S_or,S_wc]
%   k_rg_0 = end-point relative permeability to gas, -
%   k_ro_0 = end-point relative permeability to oil, -
%   k_rw_0 = end-point relative permeability to water, -
%   n_g = Corey exponent for gas, -
%   n_og = Corey exponent for oil in gas-oil flow, -
%   n_ow = Corey exponent for oil in oil-water flow, -
%   n_w = Corey exponent for water, -
%   S_gc = critical gas saturation, -
%   S_or = residual oil saturation, -
%   S_wi = immobile water saturation, -
% rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc], kg/m^3
%   rho_g_sc = gas density at standard conditions, kg/m^3.
%   rho_o_sc = oil density at standard conditions, kg/m^3.
%   rho_w_sc = water density at standard conditions, kg/m^3. Not relevant.
% sat = [S_g,S_o,S_w]
%   S_g = gas saturation, -
%   S_o = oil saturation, -
%   S_w = water saturation, -
% T_R = reservoir temperature, deg. C

% Retrieve data:
q_g_sc   = q_sc(1);
q_o_sc   = q_sc(2);
rho_g_sc = rho_sc(1);
rho_o_sc = rho_sc(2);

% Compute auxiliary data:
R_sb = q_g_sc/q_o_sc;
mu_g = gas_viscosity(p,rho_g_sc,T_R); % local gas viscosity, Pa s
mu_o = oil_viscosity(p,R_sb,rho_g_sc,rho_o_sc,T_R); % local oil viscosity, Pa s
mu_w = water_viscosity();

% Check gas saturation:
S_gc = rel(8);
S_g  = sat(1);
if S_g < S_gc
    S_gc = S_g;
end
rel(8) = S_gc;

% Compute rel perms:
k_r = rel_perms(rel,sat); % k_r = [k_rg,k_rog,k_row,k_rw]
k_rg = k_r(1);
k_ro = Stone_II(k_r,rel); % modified Stone II
k_rw = k_r(4);

% Compute total mobilities:
lambda_g = k*k_rg/mu_g;
lambda_o = k*k_ro/mu_o;
lambda_w = k*k_rw/mu_w;
