function k_r = rel_perms(rel,sat);
% function k_r = rel_perms(rel,sat);
%
% Computes relative permeabilities using Corey functions.
%
% k_r = [k_rg,k_rog,k_row,k_rw];
%   k_rg = relative permeability to gas, -
%   k_rog = relative permeability to oil in gas-oil flow, -
%   k_row = relative permeability to oil in oil-water flow, -
%   k_rw = relative permeability to water, -
% rel = [k_rg_0,k_ro_0,k_rw_0,n_g,n_og,n_ow,n_w,S_gc,S_or,S_wc]
%   k_rg_0 = end-point relative permeability to gas, -
%   k_ro_0 = end-point relative permeability to oil, - 
%   k_rw_0 = end-point relative permeability to water, -
%   n_g = Corey exponent for gas, -
%   n_og = Corey exponent for oil in gas-oil flow, -
%   n_ow = Corey exponent for oil in oil-water flow, -
%   n_w = Corey exponent for water, -
%   S_gc = critical gas saturation, -
%   S_or = residual oil saturation, -
%   S_wi = immobile water saturation, -
% sat = [S_g,S_o,S_w]
%   S_g = gas saturation, -
%   S_o = oil saturation, -
%   S_w = water saturation, -
%
% JDJ, 09-12-04, last modified 10-05-13

% Retrieve data:
k_rg_0 = rel(1);
k_ro_0 = rel(2);
k_rw_0 = rel(3);
n_g    = rel(4);
n_og   = rel(5);
n_ow   = rel(6);
n_w    = rel(7);
S_g    = sat(1);
S_w    = sat(3);
S_gc   = rel(8);
S_or   = rel(9);
S_wi   = rel(10);

% Compute relative permeabilities:
% Oil-water: 
S_w_red  = (S_w-S_wi)/(1-S_or-S_wi); % reduced water saturation, - 
k_row = max(0,k_ro_0*(1-S_w_red)^n_ow);
k_rw  = max(0,k_rw_0*S_w_red^n_w); 

% Gas-oil: 
S_g_red  = (S_g-S_gc)/(1-S_gc-S_or-S_wi); % reduced gas saturation, - 
k_rg  = max(0,k_rg_0*S_g_red^n_g);
k_rog = max(0,k_ro_0*(1-S_g_red)^n_og);  

k_r = [k_rg,k_rog,k_row,k_rw];
