function [p_out,r,p] = res(beta,fluid,h,k,oil,p_in,q_sc,r_in,r_out,rel,rho_sc,T_R)
% function [p_out,r,p,sat] = res(beta,fluid,h,k,oil,p_in,q_sc,r_in,r_out,rel,rho_sc,T_R)
%
% Computes the pressure drop for flow in a reservoir drained by a vertical well producing from
% a cylindrical drainage area under steady-state conditions. The solution is obtained through
% numerical integration from r_in to r_out. If the input pressure p_in is taken as the
% reservoir pressure p_R, the output pressure p_out is equal to the flowing bottom hole
% pressure p_wf. Conversely, if p_in is taken as p_wf, p_out is equal to p_R.
%
% The function can be used in the following fluid modes:
%
% fluid  = 1: single-phase oil flow
% fluid  = 2: single-phase gas flow 
% fluid >= 3: multi-phase gas-oil-water flow
% 
% See script files 'example_res_pres' and 'example_IPR' for examples of how to use this
% function.  
%
% beta = Forchheimer coefficient, m^-1 
% h = reservoir height, m
% k = absolute permeability, m^2
% oil = parameter to select black oil model or volatile oil table, -  
%   oil = 1: black oil; parameters computed with the aid of Standing correlations
%   oil = 2: black oil; parameters computed with the aid of Glaso correlations
%   oil = 3: volatile oil; parameters read from file 'vol_oil_table_01'
% p = pressure as a function of r, Pa
% p_in = input pressure, Pa
% p_out = output pressure, Pa
% q_sc = [q_g_sc,q_o_sc,q_w_sc], m^3/s
%   q_g_sc = gas flow rate at st. cond., m^3/s. Not relevant for fluid 1 
%   q_o_sc = oil flow rate at st. cond., m^3/s. Not relevant for fluid 2 
%   q_w_sc = water flow rate at st. cond., m^3/s. Not relevant for fluids 1 and 2  
%   Note: Flowrates in a production well need to have a negative value  
% rel = [k_rg_0,k_ro_0,k_rw_0,n_g,n_og,n_ow,n_w,S_gc,S_or,S_wc];
%   Note: relative permeability data are not relevant for fluids 1 and 2 
%   k_rg_0 = end-point relative permeability to gas, -
%   k_ro_0 = end-point relative permeability to oil, - 
%   k_rw_0 = end-point relative permeability to water, -
%   n_g = Corey exponent for gas, -
%   n_og = Corey exponent for oil in gas-oil flow, -
%   n_ow = Corey exponent for oil in oil-water flow, -
%   n_w = Corey exponent for water, -
%   S_gc = critical gas saturation, -
%   S_or = residual oil saturation, -
%   S_wi = immobile water saturation, -
% rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc], kg/m^3 
%   rho_g_sc = gas density at standard conditions, kg/m^3. Not relevant for fluid 1
%   rho_o_sc = oil density at standard conditions, kg/m^3. Not relevant for fluid 2
%   rho_w_sc = water density at standard conditions, kg/m^3. Not relevant for fluids 1 and 2 
% r = radial coordinate, m
% r_in = either the reservoir radius r_e or the well bore radius r_w, m
% r_out = r_w (if r_in = r_e) or r_out = r_e (if r_in = r_w), m
% T_R = reservoir temperature, deg. C
%
% JDJ, 14-01-05, last revised 21-12-16

interval = [r_in,r_out]; % integration interval, m
boundcon = p_in; % boundary condition, Pa
options = []; % dummy variable, -
%options = odeset('MaxStep',10,'RelTol',1e-3); % tight tolerances; time consuming!
%options = odeset('MaxStep',1,'RelTol',1e-4); % more tight tolerances; more time consuming!

switch fluid
    case 1 % oil
        k_o = k; % effective permeability to oil, [m^2]
        [r,p] = ode45('res_oil_dpdr',interval,boundcon,options,h,k_o,oil,q_sc,rho_sc,T_R);
    case 2 % gas
        k_g = k; % effective permeability to gas, [m^2]
        [r,p] = ode23('res_gas_dpdr',interval,boundcon,options,beta,h,k_g,q_sc,rho_sc,T_R);
    case {3,4,5,6} % multi-phase
        [r,p] = ode23('res_dpdr',interval,boundcon,options,beta,h,k,oil,q_sc,rel,rho_sc,T_R);
end
n = length(p);
p_out = p(n,1);
