function [p_wf,J_star] = res_Vogel(av,h,k,oil,p_R_ref,q_sc,r_e,r_w,rel,rho_sc,semi,S,T_R)
% [p_wf,J_star] = res_Vogel(av,h,k,oil,p_R_ref,q_sc,r_e,r_w,rel,rho_sc,semi,S,T_R)
%
% Computes the flowing bottom hole pressure p_wf for two-phase (gas-oil) flow in a reservoir
% produced with solution gas drive through a vertical well producing from a cylindrical
% drainage area under (semi-)steady-state conditions. The input pressure p_R_ref is the
% reservoir pressure p_R at the external boundary or the average reservoir pressure p_R_av. The
% parameter "av" indicates the choice between the use of the reservoir pressure or the average
% reservoir pressure. The parameter "semi" indicates the choice between steady-state or
% semi-steady-state conditions.    
%
% Uses an analytical approximation based on the Vogel (1986) empirical relationship.    
%
% av = parameter to select reservoir pressure, -
%   av = 0: use reservoir pressure p_R at external boundary
%   av = 1: use average reservoir pressure p_R_av
% h = reservoir height, m
% J_star = initial productivity index (slope of the IPR for p_wf = p_in), m^3/(s Pa)
% k = absolute permeability, m^2
% oil = parameter to select black oil model or volatile oil table, -  
%   oil = 1: black oil; parameters computed with the aid of Standing correlations
%   oil = 2: black oil; parameters computed with the aid of Glaso correlations
%   oil = 3: volatile oil; parameters read from file 'vol_oil_table_01'
% p_R_ref = input pressure (reservoir pressure p_R or average reservoir presure p_R_av), Pa
% p_wf = flowing bottomhole pressure (output pressure), Pa
% q_sc = [q_g_sc,q_o_sc,q_w_sc], m^3/s
%   q_g_sc = gas flow rate at standard conditions, m^3/s. 
%   q_o_sc = oil flow rate at standard conditions, m^3/s. 
%   q_w_sc = water flow rate at standard conditions, m^3/s. Not relevant
% r_e = reservoir radius, m
% r_w = well bore radius, m
% rel = [k_rg_0,k_ro_0,k_rw_0,n_g,n_og,n_ow,n_w,S_gc,S_or,S_wi]
%   k_rg_0 = end-point relative permeability to gas, -
%   k_ro_0 = end-point relative permeability to oil, -
%   k_rw_0 = end-point relative permeability to water, -
%   n_g = Corey exponent for gas, -
%   n_og = Corey exponent for oil in gas-oil flow, -
%   n_ow = Corey exponent for oil in oil-water flow, -
%   n_w = Corey exponent for water, -
%   S_gc = critical gas saturation, -
%   S_or = residual oil saturation, -
%   S_wi = immobile water saturation, -
% rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc], kg/m^3 
%   rho_g_sc = gas density at standard conditions, kg/m^3.
%   rho_o_sc = oil density at standard conditions, kg/m^3. 
%   rho_w_sc = water density at standard conditions, kg/m^3. Not relevant
% semi = parameter to select flow conditions, -
%   semi = 0: use steady-state conditions
%   semi = 1: use semi-steady-state conditions
% S = skin factor, -
% T_R = reservoir temperature, deg. C
%
% JDJ, 31-01-15, last revised 03-04-15

% Retrieve data:
k_ro_0 = rel(2);
S_wi = rel(10);

% Set parameters:
alpha = 0.2; % coefficient in Vogel's equation, -

% Densities and flow rates at standard conditions:
rho_g_sc = rho_sc(1); % gas density, kg/m^3
rho_o_sc = rho_sc(2); % oil density, kg/m^3
q_g_sc = q_sc(1); % gas flow rate, m^3/s
q_o_sc = q_sc(2); % oil flow rate, m^3/s
q_w_sc = q_sc(3); % water flow rate, m^3/s
R_go = q_g_sc/q_o_sc; % producing GOR as observed at surface, m^3/m^3
R_sb = R_go; % assumption!, -

% Check input:
if (q_w_sc ~= 0)
    error('Nonzero water rate. Use of res_Vogel not valid.')
end
switch oil
    case 1
        p_b = pres_bub_Standing(R_sb,rho_g_sc,rho_o_sc,T_R);
    case 2
        p_b = pres_bub_Glaso(R_sb,rho_g_sc,rho_o_sc,T_R);
    case 3
        p_b = pres_bub_volatile_oil(T_R);
end

% Compute f_R:
if semi == 0 && av == 0 % steady-state, reservoir pressure at external boundary
    f_R = 0;
else if semi == 0 && av == 1 % steady-state, average reservoir pressure
        f_R = 0.5;
    else if semi == 1 && av == 0 % semi-steady-state, reservoir pressure at external boundary
            f_R = 0.5;
        else % semi-steady-state, average reservoir pressure
            f_R = 0.75;
        end
    end
end

% Compute p_wf:
if p_R_ref > p_b
    mu_o = oil_viscosity(p_R_ref,R_sb,rho_g_sc,rho_o_sc,T_R);
    B_o = oil_form_vol_fact_Standing(R_sb,rho_g_sc,rho_o_sc,T_R);
    k_o = k * k_ro_0;
    J = (2*pi*k_o*h) / (mu_o*B_o*(log(r_e/r_w)-f_R+S));
    q_ob_sc = - J * (p_R_ref - p_b); % flow rate at bubble point pressure, Pa
    if abs(q_o_sc) <= abs(q_ob_sc)
        p_wf = p_R_ref + q_o_sc / J;
    else
        q_o_sc_max = q_ob_sc + J * p_b / (alpha-2);
        help01 = ((q_o_sc-q_ob_sc) / (q_o_sc_max-q_ob_sc)) - 1;
        p_wf = p_b * ((-alpha + sqrt(alpha^2-4*help01*(1-alpha))) / (2 * (1-alpha)) );
    end
    J_star = J;
else
    mu_o_ref = oil_viscosity(p_R_ref,R_sb,rho_g_sc,rho_o_sc,T_R); % local oil viscosity, Pa s
    R_s = gas_oil_rat_Standing(p_R_ref,rho_g_sc,rho_o_sc,T_R);
    B_o_ref = oil_form_vol_fact_Standing(R_s,rho_g_sc,rho_o_sc,T_R);
    S_o_ref = (p_R_ref / p_b) * (1-S_wi);
    S_g_ref = 1 - S_o_ref - S_wi;
    sat_ref = [S_g_ref,S_o_ref,S_wi];
    k_r = rel_perms(rel,sat_ref); % k_r = [k_rg,k_rog,k_row,k_rw]
    k_ro = Stone_II(k_r,rel); % modified Stone II
    k_o_ref = k * k_ro;
    J_star = (2*pi*k_o_ref*h) / (mu_o_ref*B_o_ref*(log(r_e/r_w)-f_R+S)); % initial PI, m^3/(Pa s)
    q_o_sc_max = J_star * p_R_ref / (alpha-2);
    help01 = (q_o_sc / q_o_sc_max) - 1;
    p_wf = p_R_ref * ((-alpha + sqrt(alpha^2-4*help01*(1-alpha))) / (2 * (1-alpha)) );
end

if ~isreal(p_wf) % p_wf is complex-valued
    error('Pressure in res_Vogel has dropped below zero.')
end
