function dpdr = res_dpdr(r,p,~,beta,h,k,oil,q_sc,rel,rho_sc,T_R)
% function dpdr = res_dpdr(r,p,~,beta,h,k,oil,q_sc,rel,rho_sc,T_R)
%
% Computes the derivative dp/dr in three-phase gas/oil/water flow for a given pressure p and
% radial distance r, in a reservoir penetrated by a vertical well producing from a cylindrical
% drainage area under steady-state conditions.  
%
% This function can be used to compute the pressure drop through numerical integration. It has
% the correct format to be used in conjunction with one of the standard numerical integration
% routines in MATLAB. 
%
% dpdr = pressure gradient, Pa/m
% beta = Forchheimer coefficient, m^-1 
% h = reservoir height, m
% k = absolute permeability, m^2
%   oil = 1: black oil; parameters computed with the aid of Standing correlations
%   oil = 2: black oil; parameters computed with the aid of Glaso correlations
%   oil = 3: volatile oil; parameters read from file 'vol_oil_table_01'
% p = pressure, Pa
% q_sc = [q_g_sc,q_o_sc,q_w_sc], m^3/s
%   q_g_sc = gas flow rate at standard conditions, m^3/s.
%   q_o_sc = oil flow rate at standard conditions, m^3/s.
%   q_w_sc = water flow rate at standard conditions, m^3/s.
% r = radial distance, measured from the well center to the reservoir boundary, m
% rel = [k_rg_0,k_ro_0,k_rw_0,n_g,n_og,n_ow,n_w,S_gc,S_or,S_wi]
%   k_rg_0 = end-point relative permeability to gas, -
%   k_ro_0 = end-point relative permeability to oil, -
%   k_rw_0 = end-point relative permeability to water, -
%   n_g = Corey exponent for gas, -
%   n_og = Corey exponent for oil in gas-oil flow, -
%   n_ow = Corey exponent for oil in oil-water flow, -
%   n_w = Corey exponent for water, -
%   S_gc = critical gas saturation, -
%   S_or = residual oil saturation, -
%   S_wi = immobile water saturation, -
% rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc], kg/m^3
%   rho_g_sc = gas density at standard conditions, kg/m^3.
%   rho_o_sc = oil density at standard conditions, kg/m^3.
%   rho_w_sc = water density at standard conditions, kg/m^3. 
% T_R = reservoir temperature, deg. C
%
% JDJ, 11-03-11, last revised 18-02-15

persistent sat; % kept in memory to speed up next call of saturations.m

% Check sign of pressure:
if min(p) < 1e5
    warning('Pressure below atmospheric.')
    dpdr   = 0;
    return
end

% Retrieve data:
q_g_sc   = q_sc(1); 
q_o_sc   = q_sc(2); 
rho_g_sc = rho_sc(1); 
rho_o_sc = rho_sc(2); 

% Check for presence of oil:
if abs(q_o_sc) < eps
    error('No oil flow. Two-phase gas-water flow cannot be handled.')
end

% Compute auxiliary variables:
R_go = q_g_sc/q_o_sc; % producing GOR as would be observed at surface, m^3/m^3
R_sb = R_go; % assumes R_sb = R_go 
switch oil
    case 1
        p_b = pres_bub_Standing(R_sb,rho_g_sc,rho_o_sc,T_R);
    case 2
        p_b = pres_bub_Glaso(R_sb,rho_g_sc,rho_o_sc,T_R);
    case 3
        p_b = pres_bub_volatile_oil(T_R);
end
[q,rho] = local_q_and_rho(oil,p,q_sc,R_sb,rho_sc,T_R);
q_g = q(1); % local gas rate, m^3/s
q_o = q(2); % local oil rate, m^3/s
q_w = q(3); % local water rate, m^3/s
rho_g = rho(1); % local gas density, kg/m^3
v_g = q_g/(2*pi*h*r); % local superficial gas velocity, m/s
v_o = q_o/(2*pi*h*r); % local superficial oil velocity, m/s
v_w = q_w/(2*pi*h*r); % local superficial water velocity, m/s
R_go_is = max(0,v_g/v_o); % in-situ GOR, m^3/m^3
R_wo_is = max(0,v_w/v_o); % in-situ WOR, m^3/m^3

% Compute analytical two-phase approximations as starting values:
sat_go = saturations_gas_oil(oil,p,q_sc,rel,rho_sc,T_R);
sat_ow = saturations_oil_water(oil,p,q_sc,rel,rho_sc,T_R);

% Compute saturations:
if numel(sat) == 0
    start = [sat_go(1),sat_ow(3)]; % starting values for S_g and S_w
else
    start = [sat(1),sat(3)];
end
[~,sat] = saturations(beta,h,k,oil,p,q_sc,r,rel,rho_sc,start,T_R); % sat = [S_g,S_o,S_w]

% Compute mobilities:
[lambda_g,lambda_o,lambda_w] = mobilities(k,p,q_sc,rel,rho_sc,sat,T_R);
if p > p_b % no gas
    lambda_t = lambda_o + lambda_w;
else
    lambda_t = lambda_g + lambda_o + lambda_w; % total mobility, m^2/(Pa s) 
end

% Compute Forcheimer term:
F = beta * rho_g * lambda_g * R_go_is * abs(v_g);

% Compute pressure drop:
dpdr = -(1/lambda_t) * (1 + R_go_is + F + R_wo_is) * v_o;
