function dpdr = res_gas_dpdr(r,p,~,beta,h,k_g,q_sc,rho_sc,T_R)
% function dpdr = res_gas_dpdr(r,p,~,beta,h,k_g,q_sc,rho_sc,T_R)
%
% Computes the derivative dp/dr in single-phase gas flow for a given pressure p and radial
% distance r, in a reservoir penetrated by a vertical well producing from a cylindrical
% drainage area under steady-state conditions. 
%
% This function can be used to compute the pressure drop through numerical integration. It has
% the correct format to be used in conjunction with one of the standard numerical integration
% routines in MATLAB. 
%
% beta = Forchheimer coefficient, m^-1 
% dpdr = pressure gradient, Pa/m
% h = reservoir height, m
% k_g = effective permeability to gas, m^2
% p = pressure, Pa
% q_sc = [q_g_sc,q_o_sc,q_w_sc], m^3/s
%   q_g_sc = gas flow rate at standard conditions, m^3/s. 
%   q_o_sc = oil flow rate at standard conditions, m^3/s. Not relevant.
%   q_w_sc = water flow rate at standard conditions, m^3/s. Not relevant.
% r = radial distance, measured from the well to the reservoir boundary, m
% rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc], kg/m^3 
%   rho_g_sc = gas density at standard conditions, kg/m^3.
%   rho_o_sc = oil density at standard conditions, kg/m^3. Not relevant.
%   rho_w_sc = water density at standard conditions, kg/m^3. Not relevant.
% T_R = reservoir temperature, deg. C
%
% JDJ, 14-01-05, last revised 18-01-14

% Check sign of pressure:
if min(p) < 1e5
    warning('Pressure below atmospheric.')
    dpdr   = 0;
    return
end

% Densities and flow rates at standard conditions:
rho_g_sc = rho_sc(1); % gas density, kg/m^3
q_g_sc = q_sc(1); % gas flow rate, m^3/s
q_o_sc = q_sc(2); % oil flow rate, m^3/s
q_w_sc = q_sc(3); % water flow rate, m^3/s

% Check input:
if (q_o_sc ~= 0)
    error('Nonzero oil rate. Single-phase gas solution not valid.')
end
if (q_w_sc ~= 0)
    error('Nonzero water rate. Single-phase gas solution not valid.')
end

% Compute internal variables:
M = from_kg_per_m3_to_molar_mass(rho_g_sc); % molar mass, kg/mol
R = 8314; % universal gas constant, J/(K*kmol)
T_R_abs = T_R + 273.15; % absolute reservoir temperature, K

% Compute Z:
p_pc = pres_pseu_crit_Sutton(rho_g_sc); % pseudo-critical pressure, Pa
T_pc = temp_pseu_crit_Sutton(rho_g_sc); % pseudo-critical temperaure, K
p_pr = p/p_pc; % pseudo-reduced pressure, -
T_pr = T_R_abs/T_pc; % pseudo-reduced temperature, -
Z = Z_factor_DAK(p_pr,T_pr); % Z-factor, -

% Compute mu, rho and v:
mu = gas_viscosity(p,rho_g_sc,T_R); % local gas viscosity, Pa s
rho = 1000*M*p/(Z*R*T_R_abs); % local gas density, kg/m^3
v = rho_g_sc*q_g_sc/(2*pi*h*r*rho); % local gas velocity, m/s

% Compute dpdr:
dpdr = -(mu/k_g)*v - beta*rho*v^2*sign(v);
