function [p_out,r,p,J_pseu] = res_gas_simp(av,h,k_g,p_in,q_sc,r_in,r_out,rho_sc,semi,S,T_R)
% function [p_out,r,p,J_pseu] = res_gas_simp(av,h,k_g,p_in,q_sc,r_in,r_out,rho_sc,semi,S,T_R)
%
% Computes the pressure drop for single-phase gas flow in a reservoir penetrated by a vertical
% well producing from a cylindrical drainage area under (semi-)steady-state conditions. If the
% input pressure p_in is taken as the reservoir pressure p_R or the average reservoir pressure
% p_R_av, the output pressure p_out is equal to the flowing bottom hole pressure p_wf.
% Conversely, if p_in is taken as p_wf, p_out is equal to p_R or p_R_av (depending on the value
% of parameter "av"). The parameter "semi" indicates the choice between steady-state or
% semi-steady-state conditions.
%
% Uses an analytical approximation which is only valid if the difference between p_R and p_wf
% is not too large.
%
% av = parameter to select reservoir pressure, -
%   av = 0: use reservoir pressure at external boundary
%   av = 1: use average reservoir pressure
% h = reservoir height, m
% J_pseu = pseudo productivity index, m^3/(Pa s)
% k_g = effective permeability to gas, m^2
% p = pressure as a function of r, Pa
% p_in = input pressure, Pa
% p_out = output pressure, Pa
% q_sc = [q_g_sc,q_o_sc,q_w_sc], m^3/s
%   q_g_sc = gas flow rate at standard conditions, m^3/s.
%   q_o_sc = oil flow rate at standard conditions, m^3/s. Not relevant
%   q_w_sc = water flow rate at standard conditions, m^3/s. Not relevant
% r = radial distance, measured from the well to the reservoir boundary, m
% rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc], kg/m^3
%   rho_g_sc = gas density at standard conditions, kg/m^3.
%   rho_o_sc = oil density at standard conditions, kg/m^3. Not relevant
%   rho_w_sc = water density at standard conditions, kg/m^3. Not relevant
% r_in = either the reservoir radius r_e or the well bore radius r_w, m
% r_out = r_w (if r_in = r_e) or r_out = r_e (if r_in = r_w), m
% semi = parameter to select flow conditions, -
%   semi = 0: use steady-state conditions
%   semi = 1: use semi-steady-state conditions
% S = skin factor, -
% T_R = reservoir temperature, deg. C
%
% JDJ, 19-03-03, last revised 11-01-16

% Densities and flow rates at standard conditions:
rho_g_sc = rho_sc(1); % gas density, kg/m^3
q_g_sc = q_sc(1); % gas flow rate, m^3/s
q_o_sc = q_sc(2); % oil flow rate, m^3/s
q_w_sc = q_sc(3); % water flow rate, m^3/s

% Check input:
if (q_o_sc ~= 0)
    error('Nonzero oil rate. Single-phase gas solution not valid.')
end
if (q_w_sc ~= 0)
    error('Nonzero water rate. Single-phase gas solution not valid.')
end

% Compute internal variables:
r_e = max(r_in,r_out); % reservoir radius, m
r_w = min(r_in,r_out); % well bore radius, m
p_pc = pres_pseu_crit_Sutton(rho_g_sc); % pseudo-critical pressure, Pa
T_pc = temp_pseu_crit_Sutton(rho_g_sc); % pseudo-critical temperaure, K
T_R_abs = T_R + 273.15; % absolute reservoir temperature, K

% Make initial guess:
p_R = p_in; % first guess reservoir pressure, Pa
p_R_av = p_in; % first guess average pressure, Pa

% Improve the initial guess with Picard iteration (subsequent substitution):
f_damp = 0.5; % damping factor for Picard iteration to avoid oscillations, -
tol_abs = 1.e2; % absolute convergence criterion
tol_rel = 1.e-3; % relative convergence criterion
max_iter = 1000; % maximum allowed number of iterations
iter = 0; % iteration counter
repeat = 1;
while repeat > 0
    iter = iter+1;
    p_R_old = p_R;
    p_R_av_old = p_R_av;
    
    % Compute average gas properties:
    p_pr_av = p_R_av/p_pc; % average pseudo-reduced pressure, -
    T_pr_av = T_R_abs/T_pc; % pseudo-reduced temperature, -
    Z_av = Z_factor_DAK(p_pr_av,T_pr_av); % average Z-factor, -
    p_sc = 1.e5; % pressure at standard conditions, Pa
    T_sc_abs = 15 + 273.15; % absolute temperature at standard conditons, K
    Z_sc = 1; % Z factor at standard conditions
    B_g_av = (p_sc*T_R_abs*Z_av)/(p_R_av*T_sc_abs*Z_sc); % av. gas formation volume factor, -
    mu_g_av = gas_viscosity(p_R_av,rho_g_sc,T_R); % average gas viscosity, Pa s
    
    % Compute pressure drop:
    if semi == 0 && av == 0 % steady-state, reservoir pressure at external boundary
        f_R = 0;
    else if semi == 0 && av == 1 % steady-state, average reservoir pressure
            f_R = 0.5;
        else if semi == 1 && av == 0 % semi-steady-state, reservoir pressure at ext. boundary
                f_R = 0.5;
            else % semi-steady-state, average reservoir pressure
                f_R = 0.75;
            end
        end
    end
    help01 = mu_g_av*B_g_av*q_g_sc*(log(r_e/r_w)-f_R+S)/(pi*k_g*h);
    if (r_out > r_in) % compute p_R or p_R_av
        if av == 1
            % Compute p_R_av directly, estimate p_R iteratively: 
            p_wf = p_in;
            p_R_av = sqrt(abs(p_wf^2 - help01*p_R_av));
            % Note: abs required to avoid complex values during iteration.
            p_out = p_R_av;
            p_R_new = p_R_av - mu_g_av*B_g_av*q_g_sc/(4*pi*k_g*h);       
            p_R = f_damp * p_R_old + (1-f_damp) * p_R_new;
            diff = p_R-p_R_old;
            rel_diff = diff/p_R_old;
        else
            % Compute p_R directly, estimate p_R_av iteratively:
            p_wf = p_in;
            p_R = sqrt(abs(p_wf^2 - help01*p_R_av));
            % Note: abs required to avoid complex values during iteration.
            p_out = p_R;
            p_R_av_new = p_R + mu_g_av*B_g_av*q_g_sc/(4*pi*k_g*h); 
            p_R_av = f_damp * p_R_av_old + (1-f_damp) * p_R_av_new;
            diff = p_R_av-p_R_av_old;
            rel_diff = diff/p_R_av_old;
        end
    else % compute p_wf
        if av == 1  
            % Compute p_wf directly, estimate p_R iteratively: 
            p_R_av = p_in;
            p_wf = sqrt(abs(p_R_av^2 + help01*p_R_av));
            % Note: abs required to avoid complex values during iteration.
            p_out = p_wf;
            p_R_new = p_wf+(p_R_av-p_wf)*3/2; % simple average
            p_R = f_damp * p_R_old + (1-f_damp) * p_R_new;
            diff = p_R-p_R_old;
            rel_diff = diff/p_R_old;
        else
            % Compute p_wf directly, estimate p_R_av iteratively:
            p_R = p_in;
            p_wf = sqrt(abs(p_R^2 + help01*p_R_av));
            % Note: abs required to avoid complex values during iteration.
            p_out = p_wf;
            p_R_av_new = p_wf+(p_R-p_wf)*2/3; % simple average
            p_R_av = f_damp * p_R_av_old + (1-f_damp) * p_R_av_new;
            diff = p_R_av-p_R_av_old;
            rel_diff = diff/p_R_av_old;
        end
    end
    % Check for convergence:
    if abs(diff) > tol_abs
        repeat = 1;
    else
        if abs(rel_diff) > tol_rel
            repeat = 1;
        else
            repeat = 0;
        end
    end
    if iter > max_iter
        error('Maximum allowed number of iterations exceeded.')
    end
end

% Compute p as function of r:
n_p = 100; % number of output points for p as function of r
r_inc = (r_e-r_w)/(n_p-1); % increment
r = zeros(n_p,1);
p = zeros(n_p,1);
r(1) = r_w;
p(1) = p_wf;
for i = 1:n_p-1
    r(i+1) = r(i) + r_inc;
    p(i+1) = sqrt(p_wf^2 + (p_R^2-p_wf^2) * log(r(i+1)/r_w)/log(r_e/r_w) );
end

% Compute the pseudo productivity index:
J_pseu = mu_g_av*B_g_av*p_R_av*(log(r_w/r_e)-f_R+S)/(pi*k_g*h*p_R);

