function [p_out,y,p,J] = res_oil_box(av,h,k_o_h,k_o_v,L,oil,p_in,q_sc,rho_sc,semi,S_v,T_R,...
    y_in,y_out)
% [p_out,y,p,J] = res_oil_box(av,h,k_o_h,k_o_v,L,oil,p_in,q_sc,rho_sc,semi,S_v,T_R,...
%     y_in,y_out)
%
% Computes the pressure drop for single-phase oil flow in a box-shaped reservoir drained by a
% fully penetrating horizontal well under (semi-)steady-state conditions. If the input pressure
% p_in is taken as the reservoir pressure p_R or the average reservoir pressure p_R_av, the
% output pressure p_out is equal to the flowing bottom hole pressure p_wf. Conversely, if p_in
% is taken as p_wf, p_out is equal to p_R or p_R_av (depending on the value of parameter "av").
% The parameter "semi" indicates the choice between steady-state or semi-steady-state
% conditions.
%
% av = parameter to select reservoir pressure, -
%   av = 0: use reservoir pressure at external boundary
%   av = 1: use average reservoir pressure
% h = reservoir height, m
% J = productivity index, m^3/(s Pa)
% k_o_h = effective horizontal permeability to oil, m^2
% k_v_h = effective vertical permeability to oil, m^2
% L = well length = reservoir length, m
% oil = parameter to select black oil model or volatile oil table, -  
%   oil = 1: black oil; parameters computed with the aid of Standing correlations
%   oil = 2: black oil; parameters computed with the aid of Glaso correlations
%   oil = 3: volatile oil; parameters read from file 'vol_oil_table_01'
% p = pressure as a function of y, Pa
% p_in = input pressure, Pa
% p_out = output pressure, Pa
% q_sc = [q_g_sc,q_o_sc,q_w_sc], m^3/s
%   q_g_sc = gas flow rate at standard conditions, m^3/s. 
%   q_o_sc = oil flow rate at standard conditions, m^3/s. 
%   q_w_sc = water flow rate at standard conditions, m^3/s. Not relevant
% rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc], kg/m^3 
%   rho_g_sc = gas density at standard conditions, kg/m^3.
%   rho_o_sc = oil density at standard conditions, kg/m^3. 
%   rho_w_sc = water density at standard conditions, kg/m^3. Not relevant
% semi = parameter to select flow conditions, -
%   semi = 0: use steady-state conditions
%   semi = 1: use semi-steady-state conditions
% S_v = skin factor as measured in a vertical well, -
% T_R = reservoir temperature, deg. C
% y = lateral distance, measured from the well to the reservoir boundary, m
% y_in = w/2 (if y_in > y_out) or y_in = r_w (if y_in < y_out), m
% y_out = r_w (if y_in > y_out) or y_out = w/2 (if y_in < y_out), m
%
% JDJ, 09-01-16, last revised 15-12-16

% Densities and flow rates at standard conditions:
rho_g_sc = rho_sc(1); % gas density, kg/m^3
rho_o_sc = rho_sc(2); % oil density, kg/m^3
q_g_sc = q_sc(1); % gas flow rate, m^3/s
q_o_sc = q_sc(2); % oil flow rate, m^3/s
q_w_sc = q_sc(3); % water flow rate, m^3/s

% Compute local oil properties:
R_go = q_g_sc/q_o_sc; % producing GOR as observed at surface, m^3/m^3
R_sb = R_go; % bubble point GOR 
[q,~] = local_q_and_rho(oil,p_in,q_sc,R_sb,rho_sc,T_R);
mu_o = oil_viscosity(p_in,R_sb,rho_g_sc,rho_o_sc,T_R); % local oil viscosity, Pa s
q_o = q(2); % local oil rate, m^2/s

% Check input:
if (q_w_sc ~= 0)
    error('Nonzero water rate. Use of res_oil_box not valid.')
end

% Compute internal variables:
w = 2 * max(y_in,y_out); % reservoir width, m
r_w = min(y_in,y_out); % well bore radius, m
I_vh = sqrt(k_o_v/k_o_h); % anisotropy index, -
S_conv_hat = (1/I_vh)*log(h * (1/(I_vh+1)) / (pi*r_w)); % geometric skin due to streamline convergence, -
S_h_hat = L*sqrt(I_vh)*S_v/h; % S_h = skin factor as applicable to a horizontal well, -

% Compute pressure drop:
help01 = -mu_o*q_o/(2*pi*k_o_h*L); % negative flow rate for production
Delta_p_skin = help01 * (S_conv/I_vh + S_h); % pressure drop due to skin, Pa
if (y_out > y_in) % p_wf given; compute p_R and p_R_av
    p_wf = p_in;
    if semi == 1
        p_R = p_wf + help01*(pi*w/(4*h) + S_conv_hat + S_h_hat);
        p_R_av = p_wf + help01*(pi*w/(6*h) + S_conv_hat + S_h_hat);
    else
        p_R = p_wf + help01*(pi*w/(2*h) + S_conv_hat + S_h_hat);
        p_R_av = p_wf + help01*(pi*w/(4*h) + S_conv_hat + S_h_hat);
    end
    if av == 1 
        p_out = p_R_av;
    else 
        p_out = p_R;
    end
else % p_R_av or p_R given; compute p_wf (and p_R or p_R_av)
    if av == 1 % p_R_av given
        p_R_av = p_in;
        if semi == 1
            p_wf = p_R_av - help01*(pi*w/(6*h) + S_conv_hat + S_h_hat) ;
            p_R = p_wf + help01*(pi*w/(4*h) + S_conv_hat + S_h_hat);
        else
            p_wf = p_R_av - help01*(pi*w/(4*h) + S_conv_hat + S_h_hat) ;
            p_R = p_wf + help01*(pi*w/(2*h) + S_conv_hat + S_h_hat);
        end
    else % p_R given
        p_R = p_in;
        if semi == 1
            p_wf = p_R - help01*(pi*w/(4*h) + S_conv_hat + S_h_hat);
            p_R_av = p_wf + help01*(pi*w/(6*h) + S_conv_hat + S_h_hat);
        else
            p_wf = p_R - help01*(pi*w/(2*h) + S_conv_hat + S_h_hat);
            p_R_av = p_wf + help01*(pi*w/(4*h) + S_conv_hat + S_h_hat);
        end
    end
    p_out = p_wf;
end

% Check if pressure is above bubble point pressure: 
switch oil
    case 1
        p_b = pres_bub_Standing(R_go,rho_g_sc,rho_o_sc,T_R);
    case 2
        p_b = pres_bub_Glaso(R_go,rho_g_sc,rho_o_sc,T_R);
    case 3
        p_b = pres_bub_volatile_oil(T_R);
end
if p_wf < p_b
    p_wf
    p_b
    error('Pressure below bubble point pressure. Use of res_oil_box not valid.')
end

% Compute p as function of y: 
n_p = 100; % number of output points
range_y = w/2 - r_w; % plot range y, m 
y_inc = (range_y)/(n_p-1); % increment, m
y = zeros(n_p,1);
p = zeros(n_p,1);
y(1) = r_w;
p(1) = p_wf;
y(2) = r_w;
p(2) = p_wf + Delta_p_skin;
for i = 2:n_p-1
    y(i+1) = y(i) + y_inc;
    p(i+1) = p_wf + Delta_p_skin + help01*(pi*2*y(i+1)/(2*h));
end

% Compute the productivity index:
if av == 1
    J = -q_o_sc/(p_R_av-p_wf); 
else
    J = -q_o_sc/(p_R-p_wf);
end
