function dpdr = res_oil_dpdr(r,p,~,h,k_o,oil,q_sc,rho_sc,T_R)
% function dpdr = res_oil_dpdr(r,p,~,h,k_o,oil,q_sc,rho_sc,T_R)
%
% Computes the derivative dp/dr in single-phase oil flow for a given pressure p and radial
% distance r, in a reservoir penetrated by a vertical well producing from a cylindrical
% drainage area under steady-state conditions. 
%
% This function can be used to compute the pressure drop through numerical integration. It has
% the correct format to be used in conjunction with one of the standard numerical integration
% routines in MATLAB. 
%
% dpdr = pressure gradient, Pa/m
% h = reservoir height, m
% k_o = effective permeability to oil, m^2
% oil = parameter to select black oil model or volatile oil table, -  
%   oil = 1: black oil; parameters computed with the aid of Standing correlations
%   oil = 2: black oil; parameters computed with the aid of Glaso correlations
%   oil = 3: volatile oil; parameters read from file 'vol_oil_table_01'
% p = pressure, Pa
% q_sc = [q_g_sc,q_o_sc,q_w_sc], m^3/s
%   q_g_sc = gas flow rate at standard conditions, m^3/s.  
%   q_o_sc = oil flow rate at standard conditions, m^3/s.
%   q_w_sc = water flow rate at standard conditions, m^3/s. Not relevant
% r = radial distance, measured from the well to the reservoir boundary, m
% rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc], kg/m^3 
%   rho_g_sc = gas density at standard conditions, kg/m^3. 
%   rho_o_sc = oil density at standard conditions, kg/m^3.
%   rho_w_sc = water density at standard conditions, kg/m^3. Not relevant
% T_R = reservoir temperature, deg. C
%
% JDJ, 14-01-05, last revised 10-05-13

% Check sign of pressure:
if min(p) < 1e5
    warning('Pressure below atmospheric.')
    dpdr   = 0;
    return
end

% Densities and flow rates at standard conditions:
rho_g_sc = rho_sc(1); % gas density, kg/m^3
rho_o_sc = rho_sc(2); % oil density, kg/m^3
q_g_sc = q_sc(1); % gas flow rate, m^3/s
q_o_sc = q_sc(2); % oil flow rate, m^3/s
q_w_sc = q_sc(3); % water flow rate, m^3/s

% Check input:
if (q_w_sc ~= 0)
    error('Nonzero water rate. Single-phase oil solution not valid.')
end

% Check if pressure is above bubble point pressure: 
R_go = q_g_sc/q_o_sc; % producing GOR as would be observed at surface, m^3/m^3
R_sb = R_go; % assumes R_sb = R_go 
switch oil
    case 1
        p_b = pres_bub_Standing(R_sb,rho_g_sc,rho_o_sc,T_R);
    case 2
        p_b = pres_bub_Glaso(R_sb,rho_g_sc,rho_o_sc,T_R);
    case 3
        p_b = pres_bub_volatile_oil(T_R);
end
if p < p_b
    error('Pressure below bubble point pressure. Use of res_oil_dpds not valid.')
end

% Compute local oil properties:
[q,~] = local_q_and_rho(oil,p,q_sc,R_sb,rho_sc,T_R);
mu_o = oil_viscosity(p,R_sb,rho_g_sc,rho_o_sc,T_R); % local oil viscosity, Pa s
q_o = q(2); % local oil flow rate
v_o = q_o/(2*pi*h*r); % local oil velocity, m/s

% Compute dpdr:
dpdr = -(mu_o/k_o)*v_o;

