function [p_out,y,p,J] = res_oil_ellipse(h,k_o_h,k_o_v,L,oil,p_in,q_sc,rho_sc,S_v,T_R,...
    y_in,y_out)
% [p_out,y,p,J] = res_oil_ellipse(h,k_o_h,k_o_v,L,oil,p_in,q_sc,rho_sc,S_v,T_R,...
%     y_in,y_out)
%
% Computes the pressure drop for single-phase oil flow in an elliptical reservoir drained by a
% partially penetrating horizontal well under steady-state conditions. If the input pressure
% p_in is taken as the reservoir pressure p_R the output pressure p_out is equal to the flowing
% bottom hole pressure p_wf. Conversely, if p_in is taken as p_wf, p_out is equal to p_R. The
% pressure output is given as a function of y, where y is the coordinate along the short axis
% of the ellipse (i.e. perpendicular to the well). The length of the short axis length is
% defined as b_e = 2 * max(y_in, y_out).    
%
% h = reservoir height, m
% J = productivity index, m^3/(s Pa)
% k_o_h = effective horizontal permeability to oil, m^2
% k_v_h = effective vertical permeability to oil, m^2
% L = well length, m
% oil = parameter to select black oil model or volatile oil table, -  
%   oil = 1: black oil; parameters computed with the aid of Standing correlations
%   oil = 2: black oil; parameters computed with the aid of Glaso correlations
%   oil = 3: volatile oil; parameters read from file 'vol_oil_table_01'
% p = pressure as a function of y, Pa
% p_in = input pressure, Pa
% p_out = output pressure, Pa
% q_sc = [q_g_sc,q_o_sc,q_w_sc], m^3/s
%   q_g_sc = gas flow rate at standard conditions, m^3/s. 
%   q_o_sc = oil flow rate at standard conditions, m^3/s. 
%   q_w_sc = water flow rate at standard conditions, m^3/s. Not relevant
% rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc], kg/m^3 
%   rho_g_sc = gas density at standard conditions, kg/m^3.
%   rho_o_sc = oil density at standard conditions, kg/m^3. 
%   rho_w_sc = water density at standard conditions, kg/m^3. Not relevant
% S_v = skin factor as measured in a vertical well, -
% T_R = reservoir temperature, deg. C
% y = lateral distance, measured from the well to the reservoir boundary along the short axis
%     of the ellipse, m
% y_in = b_e/2 (if y_in > y_out) or y_in = 0 (if y_in < y_out), m
% y_out = 0 (if y_in > y_out) or y_out = b_e/2 (if y_in < y_out), m
%
% JDJ, 10-01-16, last revised 10-01-16

% Densities and flow rates at standard conditions:
rho_g_sc = rho_sc(1); % gas density, kg/m^3
rho_o_sc = rho_sc(2); % oil density, kg/m^3
q_g_sc = q_sc(1); % gas flow rate, m^3/s
q_o_sc = q_sc(2); % oil flow rate, m^3/s
q_w_sc = q_sc(3); % water flow rate, m^3/s

% Compute local oil properties:
R_go = q_g_sc/q_o_sc; % producing GOR as observed at surface, m^3/m^3
R_sb = R_go; % bubble point GOR 
[q,~] = local_q_and_rho(oil,p_in,q_sc,R_sb,rho_sc,T_R);
mu_o = oil_viscosity(p_in,R_sb,rho_g_sc,rho_o_sc,T_R); % local oil viscosity, Pa s
q_o = q(2); % local oil rate, m^2/s
% Check input:
if (q_w_sc ~= 0)
    error('Nonzero water rate. Use of res_oil_ellipse not valid.')
end

% Compute internal variables:
b_e = 2 * max(y_in,y_out); % short axis length = reservoir width, m
r_w = min(y_in,y_out); % well bore radius, m
I_vh = sqrt(k_o_v/k_o_h); % anisotropy index, -
S_conv = log(h * (1/(I_vh+1)) / (pi*r_w)); % geometric skin due to streamline convergence, -

% Compute pressure drop:
help01 = -mu_o*q_o/(2*pi*k_o_h*h); % negative flow rate for production
Delta_p_skin = help01 * (h*S_conv/(L*I_vh) + S_v); % pressure drop due to skin, Pa
if (y_out > y_in) % p_wf given; compute p_R
    p_wf = p_in;
    p_R = p_wf + help01 * (log(b_e/L + sqrt(b_e^2/L^2 + 1)) + h*S_conv/(L*I_vh) + S_v);
    p_out = p_R;
else % p_R given; compute p_wf
    p_R = p_in;
    p_wf = p_R - help01 * (log(b_e/L + sqrt(b_e^2/L^2 + 1)) + h*S_conv/(L*I_vh) + S_v);
    p_out = p_wf;
end

% Check if pressure is above bubble point pressure: 
switch oil
    case 1
        p_b = pres_bub_Standing(R_go,rho_g_sc,rho_o_sc,T_R);
    case 2
        p_b = pres_bub_Glaso(R_go,rho_g_sc,rho_o_sc,T_R);
    case 3
        p_b = pres_bub_volatile_oil(T_R);
end
if p_wf < p_b
    p_wf
    p_b
    error('Pressure below bubble point pressure. Use of res_oil_ellipse not valid.')
end

% Compute p as function of y: 
% Note: the lower limit is taken as 0, unlike for a box-shaped reservoir where the lower limit
% is taken as r_w.
n_p = 100; % number of output points for p as function of x
range_y = b_e/2; % plot range y, m
y_e = b_e/2;
help02 = y_e^2 / L^2;
help03 = help02 + 0.25;
H_ksi_e = 2 * sqrt(help03);
ksi_e = acosh(H_ksi_e);
range_ksi = ksi_e; % plot range ksi (elliptic coordinate), m 
y_inc = (range_y)/(n_p-1); % increment, m
y = zeros(n_p,1);
p = zeros(n_p,1);
ksi = zeros(n_p,1);
y(1) = 0;
p(1) = p_wf;
y(2) = 0;
p(2) = p_wf + Delta_p_skin;
for i = 2:n_p-1
    y(i+1) = y(i) + y_inc;
    help04= y(i+1)^2 / L^2;
    help05 = help04 + 0.25;
    H_ksi = 2* sqrt(help05);
    ksi(i+1) = acosh(H_ksi);
    p(i+1) = p_wf + Delta_p_skin + (p_R - p_wf - Delta_p_skin) * (ksi(i+1)/range_ksi);
end

% Compute the productivity index:
J = -q_o_sc/(p_R-p_wf);
