function [p_out,r,p,J] = res_oil_simp(av,h,k_o,oil,p_in,q_sc,r_in,r_out,rho_sc,semi,S,T_R)
% [p_out,r,p,J] = res_oil_simp(av,h,k_o,oil,p_in,q_sc,r_in,r_out,rho_sc,semi,S,T_R)
%
% Computes the pressure drop for single-phase oil flow in a reservoir penetrated by a vertical
% well producing from a cylindrical drainage area under (semi-)steady-state conditions. If the
% input pressure p_in is taken as the reservoir pressure p_R or the average reservoir pressure
% p_R_av, the output pressure p_out is equal to the flowing bottom hole pressure p_wf.
% Conversely, if p_in is taken as p_wf, p_out is equal to p_R or p_R_av (depending on the value
% of parameter "av"). The parameter "semi" indicates the choice between steady-state or
% semi-steady-state conditions.
%
% Uses an analytical approximation which is valid under most circumstances.    
%
% av = parameter to select reservoir pressure, -
%   av = 0: use reservoir pressure at external boundary
%   av = 1: use average reservoir pressure
% h = reservoir height, m
% J = productivity index, m^3/(s Pa)
% k_o = effective permeability to oil, m^2
% oil = parameter to select black oil model or volatile oil table, -  
%   oil = 1: black oil; parameters computed with the aid of Standing correlations
%   oil = 2: black oil; parameters computed with the aid of Glaso correlations
%   oil = 3: volatile oil; parameters read from file 'vol_oil_table_01'
% p = pressure as a function of r, Pa
% p_in = input pressure, Pa
% p_out = output pressure, Pa
% q_sc = [q_g_sc,q_o_sc,q_w_sc], m^3/s
%   q_g_sc = gas flow rate at standard conditions, m^3/s. 
%   q_o_sc = oil flow rate at standard conditions, m^3/s. 
%   q_w_sc = water flow rate at standard conditions, m^3/s. Not relevant
% r = radial distance, measured from the well to the reservoir boundary, m
% r_in = either the reservoir radius r_e or the well bore radius r_w, m
% r_out = r_w (if r_in = r_e) or r_out = r_e (if r_in = r_w), m
% rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc], kg/m^3 
%   rho_g_sc = gas density at standard conditions, kg/m^3.
%   rho_o_sc = oil density at standard conditions, kg/m^3. 
%   rho_w_sc = water density at standard conditions, kg/m^3. Not relevant
% semi = parameter to select flow conditions, -
%   semi = 0: use steady-state conditions
%   semi = 1: use semi-steady-state conditions
% S = skin factor, -
% T_R = reservoir temperature, deg. C
%
% JDJ, 19-03-03, last revised 11-01-16

% Densities and flow rates at standard conditions:
rho_g_sc = rho_sc(1); % gas density, kg/m^3
rho_o_sc = rho_sc(2); % oil density, kg/m^3
q_g_sc = q_sc(1); % gas flow rate, m^3/s
q_o_sc = q_sc(2); % oil flow rate, m^3/s
q_w_sc = q_sc(3); % water flow rate, m^3/s

% Compute local oil properties:
R_go = q_g_sc/q_o_sc; % producing GOR as observed at surface, m^3/m^3
R_sb = R_go; % bubble point GOR 
[q,~] = local_q_and_rho(oil,p_in,q_sc,R_sb,rho_sc,T_R);
mu_o = oil_viscosity(p_in,R_sb,rho_g_sc,rho_o_sc,T_R); % local oil viscosity, Pa s
q_o = q(2); % local oil rate, m^2/s

% Check input:
if (q_w_sc ~= 0)
    error('Nonzero water rate. Use of res_oil_simp not valid.')
end

% Compute internal variables:
r_e = max(r_in,r_out); % reservoir radius, m
r_w = min(r_in,r_out); % well bore radius, m
if semi == 0 && av == 0 % steady-state, reservoir pressure at external boundary
    f_R = 0;
else if semi == 0 && av == 1 % steady-state, average reservoir pressure
        f_R = 0.5;
    else if semi == 1 && av == 0 % semi-steady-state, reservoir pressure at external boundary
            f_R = 0.5;
        else % semi-steady-state, average reservoir pressure
            f_R = 0.75;
        end
    end
end

% Compute pressure drop:
help01 = -mu_o*q_o*(log(r_e/r_w)-f_R+S)/(2*pi*k_o*h);
Delta_p_skin = -mu_o*q_o*S/(2*pi*k_o*h); % pressure drop due to skin, Pa
if (r_out > r_in) % compute p_R or p_R_av
    p_out = p_in + help01;
    p_wf = p_in;
    if av == 1
        p_R_av = p_out;
        p_R = p_R_av - mu_o*q_o/(4*pi*k_o*h);
    else
        p_R = p_out;
    end
else % compute p_wf
    p_out = p_in - help01;
    p_wf = p_out;
    if av == 1
        p_R_av = p_in;
        p_R = p_R_av - mu_o*q_o/(4*pi*k_o*h);
    else
        p_R = p_in;
    end
end

% Check if pressure is above bubble point pressure: 
switch oil
    case 1
        p_b = pres_bub_Standing(R_go,rho_g_sc,rho_o_sc,T_R);
    case 2
        p_b = pres_bub_Glaso(R_go,rho_g_sc,rho_o_sc,T_R);
    case 3
        p_b = pres_bub_volatile_oil(T_R);
end
if p_wf < p_b
    p_wf
    p_b
    error('Pressure below bubble point pressure. Use of res_oil_simp not valid.')
end

% Compute p as function of r: 
n_p = 100; % number of output points for p as function of r
r_inc = (r_e-r_w)/(n_p-1); % increment
r = zeros(n_p,1);
p = zeros(n_p,1);
r(1) = r_w;
p(1) = p_wf;
y(2) = r_w;
p(2) = p_wf + Delta_p_skin;
for i = 2:n_p-1
    r(i+1) = r(i) + r_inc;
    p(i+1) = p_wf + Delta_p_skin + (p_R-(p_wf-Delta_p_skin)) * log(r(i+1)/r_w) / log(r_e/r_w);
end

% Compute the productivity index:
if av == 1
    J = -q_o_sc/(p_R_av-p_wf); 
else
    J = -q_o_sc/(p_R-p_wf);
end
