function [n_iter,sat] = saturations(beta,h,k,oil,p,q_sc,r,rel,rho_sc,start,T_R)
% function [n_iter,sat] = saturations(beta,h,k,oil,p,q_sc,r,rel,rho_sc,start,T_R)
%
% Computes the saturations S_g, S_o and S_w in three-phase gas/oil/water flow for a given
% pressure p and radial distance r, in a reservoir penetrated by a vertical well producing from
% a cylindrical drainage area under steady-state conditions.
%
% beta = Forchheimer coefficient, m^-1 
% h = reservoir height, m
% n_iter = number of iterations, -
% k = absolute permeability, m^2
% oil = parameter to select black oil model or volatile oil table, -  
%   oil = 1: black oil; parameters computed with the aid of Standing correlations
%   oil = 2: black oil; parameters computed with the aid of Glaso correlations
%   oil = 3: volatile oil; parameters read from file 'vol_oil_table_01'
% p = pressure, Pa
% q_sc = [q_g_sc,q_o_sc,q_w_sc], m^3/s
%   q_g_sc = gas flow rate at standard conditions, m^3/s.
%   q_o_sc = oil flow rate at standard conditions, m^3/s.
%   q_w_sc = water flow rate at standard conditions, m^3/s. Not relevant.
% r = radial distance, measured from the well to the reservoir boundary, m
% rel = [k_rg_0,k_ro_0,k_rw_0,n_g,n_og,n_ow,n_w,S_gc,S_or,S_wi]
%   k_rg_0 = end-point relative permeability to gas, -
%   k_ro_0 = end-point relative permeability to oil, -
%   k_rw_0 = end-point relative permeability to water, -
%   n_g = Corey exponent for gas, -
%   n_og = Corey exponent for oil in gas-oil flow, -
%   n_ow = Corey exponent for oil in oil-water flow, -
%   n_w = Corey exponent for water, -
%   S_gc = critical gas saturation, -
%   S_or = residual oil saturation, -
%   S_wi = immobile water saturation, -
% rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc], kg/m^3
%   rho_g_sc = gas density at standard conditions, kg/m^3.
%   rho_o_sc = oil density at standard conditions, kg/m^3.
%   rho_w_sc = water density at standard conditions, kg/m^3. Not relevant.
% sat = [S_g,S_o,S_w]
%   S_g = gas saturation, -
%   S_o = oil saturation, -
%   S_w = water saturation, -
% start = [S_g_start,S_w_start]
%   S_g_start = starting value for gas saturation
%   S_w_start = starting value for water saturation
% T_R = reservoir temperature, deg. C
%
% JDJ, 27-01-05, last revised 22-01-14

% Densities and flow rates at standard conditions:
rho_g_sc = rho_sc(1); % gas density, kg/m^3
rho_o_sc = rho_sc(2); % oil density, kg/m^3
q_g_sc = q_sc(1); % gas flow rate, m^3/s
q_o_sc = q_sc(2); % oil flow rate, m^3/s

% GOR:
R_sb = q_g_sc/q_o_sc; % R_sb = R_go, m^3/m^3

% Compute local properties:
[q,rho] = local_q_and_rho(oil,p,q_sc,R_sb,rho_sc,T_R);
q_g = q(1); % local gas rate, m^3/s
q_o = q(2); % local oil rate, m^3/s
q_w = q(3); % local water rate, m^3/s
rho_g = rho(1); % local gas density, kg/m^3
mu_g = gas_viscosity(p,rho_g_sc,T_R); % local gas viscosity, Pa s
mu_o = oil_viscosity(p,R_sb,rho_g_sc,rho_o_sc,T_R); % local oil viscosity, Pa s
mu_w = water_viscosity; % local water viscosity, Pa s
mu = [mu_g,mu_o,mu_w];
v_g = q_g/(2*pi*h*r); % local superficial gas velocity, m/s
v_o = q_o/(2*pi*h*r); % local superficial oil velocity, m/s
v_w = q_w/(2*pi*h*r); % local superficial water velocity, m/s
v = [v_g,v_o,v_w];
S_wi = rel(10);

% Compute saturations with Newton Raphson iteration:
n_iter = 0;
small = eps; % cut off value for two/three-phase flow
S_g = start(1);
S_w = start(2);
if abs(q_g) < small % two-phase oil/water flow
    multi = 1;
    S_g = 0;
    if abs(q_w) < small % single-phase oil flow
        multi = 0;
        S_g = 0;
        S_o = 1 - S_wi;
        S_w = S_wi;
        sat = [S_g,S_o,S_w];     
        return
    end
else
    if abs(q_w) < small % two-phase gas/oil flow
        multi = 2;
        S_w = S_wi;
    else % three-phase gas/oil/water flow
        multi = 3;
    end
end
d_S = 1e-6; % saturation change to obtain numerical derivative
tol_abs = 1.e-9; % absolute convergence criterion
tol_rel = 1.e-6; % relative convergence criterion
max_iter_1 = 50; % maximum allowed number of iterations before cutting step sizes
max_iter_2 = 10000; % maximum allowed number of total iterations
max_diff_g = 0.1; % maximum allowed absolute difference in S_g per iteration
max_diff_w = 0.1; % maximum allowed absolute difference in S_w per iteration
iter_1 = 0; % iteration counter
iter_2 = 0; % iteration counter
repeat = 1;
while repeat > 0
    if iter_1 > max_iter_1
        max_diff_g = max_diff_g/5;
        max_diff_w = max_diff_w/5;
        %         q_t = q_g + q_o + q_w;
        %         help_1 = abs(q_g/q_t)*(1-S_gc-S_or-S_wi)+S_gc;
        %         help_2 = abs(q_w/q_t)*(1-S_gc-S_or-S_wi)+S_wi;
        %         start = [help_1,help_2]; % new starting values for S_g and S_w
        %         start = [1/3,1/3]; % new starting values for S_g and S_w
        iter_1 = 0;
    end
    if iter_2 > max_iter_2
        E_go
        E_ow
        S_g
        S_w
        error('Error: Maximum allowed number of iterations exceeded.')
    end
    iter_1 = iter_1+1;
    iter_2 = iter_2+1;
    S_g_old = S_g;
    S_w_old = S_w;
    % Determine error:
    S_o = 1-S_g-S_w;
    sat = [S_g,S_o,S_w];
    k_r = rel_perms(rel,sat); % k_r = [k_rg,k_rog,k_row,k_rw]
    E = error_drdp(beta,k,k_r,mu,multi,rel,rho_g,v); % E = [E_go,E_ow]
    E_go = E(1); % error between dp/dr for gas and dp/dr for oil, m/Pa
    E_ow = E(2); % error between dp/dr for oil and dp/dr for water, m/Pa
    % Perturb S_g:
    if (S_g < 0.5)
        n = 1;
    else
        n = -1;
    end
    S_g = S_g + n*d_S;
    S_o = 1-S_g-S_w;
    sat = [S_g,S_o,S_w];
    k_r = rel_perms(rel,sat);
    E_g = error_drdp(beta,k,k_r,mu,multi,rel,rho_g,v);
    E_go_g = E_g(1);
    E_ow_g = E_g(2);
    dEgodg = (E_go_g-E_go)/d_S;
    dEowdg = (E_ow_g-E_ow)/d_S;
    S_g = S_g_old; % reset
    % Perturb S_w:
    if (S_w < 0.5)
        n = 1;
    else
        n = -1;
    end
    S_w = S_w + n*d_S;
    S_o = 1-S_g-S_w;
    sat = [S_g,S_o,S_w];
    k_r = rel_perms(rel,sat);
    E_w = error_drdp(beta,k,k_r,mu,multi,rel,rho_g,v);
    E_go_w = E_w(1);
    E_ow_w = E_w(2);
    dEgodw = (E_go_w-E_go)/d_S;
    dEowdw = (E_ow_w-E_ow)/d_S;
    S_w = S_w_old; % reset
    % Update saturations:
    switch multi
        case 1 % two-phase oil/water flow
            S_w = S_w - E_ow/dEowdw; % Newton Raphson update
            diff_w = S_w - S_w_old;
            if abs(diff_w) > max_diff_w % step size check
                S_w = S_w_old + max_diff_w * sign(diff_w);
                diff_w = max_diff_w;
            end
            S_o = 1-S_g-S_w;
            rel_diff_w = diff_w/S_w_old;
            if abs(diff_w) > tol_abs || ... % convergence check
                    abs(rel_diff_w) > tol_rel
                repeat = 1;
            else
                repeat = 0;
            end
            sat = [0,S_o,S_w];
        case 2 % two-phase gas/oil flow
            S_g = S_g - E_go/dEgodg; % Newton Raphson update
            diff_g = S_g - S_g_old;
            if abs(diff_g) > max_diff_g
                S_g = S_g_old + max_diff_g * sign(diff_g);
                diff_g = max_diff_g;
            end
            S_o = 1-S_g-S_w;
            if S_g_old > eps
                rel_diff_g = diff_g/S_g_old;
            else % occurs when pressure drops below the bubble point
                rel_diff_g = 1;
            end
            if abs(diff_g) > tol_abs || ...
                    abs(rel_diff_g) > tol_rel
                repeat = 1;
            else
                repeat = 0;
            end
            sat = [S_g,S_o,S_wi];
        case 3 % three-phase flow
            J = [dEgodg dEgodw ; ...
                dEowdg dEowdw]; % Jacobian
            S = [S_g;S_w];
            if abs(det(J)) > eps
                S = S - J\[E_go;E_ow]; % Newton Rapson update
            else % J is singular
                S = S;
            end
            S_g = S(1);
            S_w = S(2);
            diff_g = S_g - S_g_old;
            diff_w = S_w - S_w_old;
            if abs(diff_g) > max_diff_g
                S_g = S_g_old + max_diff_g * sign(diff_g);
                diff_g = max_diff_g;
            end
            if abs(diff_w) > max_diff_w
                S_w = S_w_old + max_diff_w * sign(diff_w);
                diff_w = max_diff_w;
            end
            S_o = 1-S_g-S_w;
            if S_g_old > eps
                rel_diff_g = diff_g/S_g_old;
            else % occurs when pressure drops below the bubble point
                rel_diff_g = 1;
            end
            rel_diff_w = diff_w/S_w_old;
            if abs(diff_g) > tol_abs || ...
                    abs(diff_w) > tol_abs || ...
                    abs(rel_diff_g) > tol_rel || ...
                    abs(rel_diff_w) > tol_rel
                repeat = 1;
            else
                repeat = 0;
            end
            sat = [S_g,S_o,S_w];
    end
end
n_iter = iter_2;

% % Reality check:
% if S_g < S_gc
%     S_g = S_gc;
% end
% if S_g > 1-S_or-S_wi
%     S_g = 1-S_or-S_wi;
% end
% if S_w < S_wi
%     S_w = S_wi;
% end
% if S_w > 1-S_or
%     S_w = 1-S_or;
% end
% S_o = 1-S_g-S_w;
% sat = [S_g,S_o,S_w];
