function [n_iter,sat] = saturations_analytical(h,k,oil,p,q_sc,r,rel,rho_sc,start,T_R)
% function [n_iter,sat] = saturations_analytical(h,k,oil,p,q_sc,r,rel,rho_sc,start,T_R)
%
% Computes the saturations S_g, S_o and S_w in three-phase gas/oil/water flow for a given
% pressure p and radial distance r, in a reservoir penetrated by a vertical well producing from
% a cylindrical drainage area under steady-state conditions.
%
% h = reservoir height, m
% n_iter = number of iterations, -
% k = absolute permeability, m^2
% oil = parameter to select black oil model or volatile oil table, -  
%   oil = 1: black oil; parameters computed with the aid of Standing correlations
%   oil = 2: black oil; parameters computed with the aid of Glaso correlations
%   oil = 3: volatile oil; parameters read from file 'vol_oil_table_01'
% p = pressure, Pa
% q_sc = [q_g_sc,q_o_sc,q_w_sc], m^3/s
%   q_g_sc = gas flow rate at standard conditions, m^3/s.
%   q_o_sc = oil flow rate at standard conditions, m^3/s.
%   q_w_sc = water flow rate at standard conditions, m^3/s. Not relevant.
% r = radial distance, measured from the well to the reservoir boundary, m
% rel = [k_rg_0,k_ro_0,k_rw_0,n_g,n_og,n_ow,n_w,S_gc,S_or,S_wi]
%   k_rg_0 = end-point relative permeability to gas, -
%   k_ro_0 = end-point relative permeability to oil, -
%   k_rw_0 = end-point relative permeability to water, -
%   n_g = Corey exponent for gas, -
%   n_og = Corey exponent for oil in gas-oil flow, -
%   n_ow = Corey exponent for oil in oil-water flow, -
%   n_w = Corey exponent for water, -
%   S_gc = critical gas saturation, -
%   S_or = residual oil saturation, -
%   S_wi = immobile water saturation, -
% rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc], kg/m^3
%   rho_g_sc = gas density at standard conditions, kg/m^3.
%   rho_o_sc = oil density at standard conditions, kg/m^3.
%   rho_w_sc = water density at standard conditions, kg/m^3. Not relevant.
% sat = [S_g,S_o,S_w]
%   S_g = gas saturation, -
%   S_o = oil saturation, -
%   S_w = water saturation, -
% start = [S_g_start,S_w_start]
%   S_g_start = starting value for gas saturation
%   S_w_start = starting value for water saturation
% T_R = reservoir temperature, deg. C
%
% JDJ, 27-01-05, last revised 27-12-13

% Densities and flow rates at standard conditions:
rho_g_sc = rho_sc(1); % gas density, kg/m^3
rho_o_sc = rho_sc(2); % oil density, kg/m^3
q_g_sc = q_sc(1); % gas flow rate, m^3/s
q_o_sc = q_sc(2); % oil flow rate, m^3/s

% GOR:
R_sb = q_g_sc/q_o_sc; % R_sb = R_go, m^3/m^3

% Compute local properties:
[q,~] = local_q_and_rho(oil,p,q_sc,R_sb,rho_sc,T_R);
q_g = q(1); % local gas rate, m^3/s
q_o = q(2); % local oil rate, m^3/s
q_w = q(3); % local water rate, m^3/s
mu_g = gas_viscosity(p,rho_g_sc,T_R); % local gas viscosity, Pa s
mu_o = oil_viscosity(p,R_sb,rho_g_sc,rho_o_sc,T_R); % local oil viscosity, Pa s
mu_w = water_viscosity; % local water viscosity, Pa s
mu = [mu_g,mu_o,mu_w];
v_g = q_g/(2*pi*h*r); % local superficial gas velocity, m/s
v_o = q_o/(2*pi*h*r); % local superficial oil velocity, m/s
v_w = q_w/(2*pi*h*r); % local superficial water velocity, m/s
v = [v_g,v_o,v_w];
S_wi = rel(10);

% Compute saturations with Newton Raphson iteration:
n_iter = 0;
small = eps; % cut off value for two/three-phase flow
S_g = start(1);
S_w = start(2);
if abs(q_g) < small % two-phase oil/water flow
    multi = 1;
    S_g = 0;
    if abs(q_w) < small % single-phase oil flow
        multi = 0;
        S_g = 0;
        S_o = 1 - S_wi;
        S_w = S_wi;
        sat = [S_g,S_o,S_w];     
        return
    end
else
    if abs(q_w) < small % two-phase gas/oil flow
        multi = 2;
        S_w = S_wi;
    else % three-phase gas/oil/water flow
        multi = 3;
    end
end

%%%%
k_rg_0 = rel(1);
k_ro_0 = rel(2);
n_g = rel(4);
n_og = rel(5);
S_gc = rel(8);
S_or = rel(9);
S_g_star = (1+((k_ro_0*mu_g*v_g)/(k_rg_0*mu_o*v_o))^(-2/(n_g+n_og)))^-1;
S_g = S_gc + S_g_star * (1-S_gc-S_or-S_wi);
S_w = S_wi;
S_o = 1-S_g-S_w;
%%%%

% % Reality check:
% if S_g < S_gc
%     S_g = S_gc;
% end
% if S_g > 1-S_or-S_wi
%     S_g = 1-S_or-S_wi;
% end
% if S_w < S_wi
%     S_w = S_wi;
% end
% if S_w > 1-S_or
%     S_w = 1-S_or;
% end
% S_o = 1-S_g-S_w;
sat = [S_g,S_o,S_w];
