function sat = saturations_gas_oil(oil,p,q_sc,rel,rho_sc,T_R)
% function sat = saturations_gas_oil(oil,p,q_sc,rel,rho_sc,T_R)
%
% Computes approximate saturations for two-phase gas-oil flow in a reservoir penetrated by a
% vertical well producing from a cylindrical drainage area under steady-state conditions.
%
% oil = parameter to select black oil model or volatile oil table, -  
%   oil = 1: black oil; parameters computed with the aid of Standing correlations
%   oil = 2: black oil; parameters computed with the aid of Glaso correlations
%   oil = 3: volatile oil; parameters read from file 'vol_oil_table_01'
% p = pressure, Pa
% q_sc = [q_g_sc,q_o_sc,q_w_sc], m^3/s
%   q_g_sc = gas flow rate at standard conditions, m^3/s.
%   q_o_sc = oil flow rate at standard conditions, m^3/s.
%   q_w_sc = water flow rate at standard conditions, m^3/s. Not relevant.
% rel = [k_rg_0,k_ro_0,k_rw_0,n_g,n_og,n_ow,n_w,S_gc,S_or,S_wi]
%   k_rg_0 = end-point relative permeability to gas, -
%   k_ro_0 = end-point relative permeability to oil, -
%   k_rw_0 = end-point relative permeability to water, -
%   n_g = Corey exponent for gas, -
%   n_og = Corey exponent for oil in gas-oil flow, -
%   n_ow = Corey exponent for oil in oil-water flow, -
%   n_w = Corey exponent for water, -
%   S_gc = critical gas saturation, -
%   S_or = residual oil saturation, -
%   S_wi = immobile water saturation, -
% rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc], kg/m^3
%   rho_g_sc = gas density at standard conditions, kg/m^3.
%   rho_o_sc = oil density at standard conditions, kg/m^3.
%   rho_w_sc = water density at standard conditions, kg/m^3. Not relevant.
% sat = [S_g,S_o,S_w]
%   S_g = gas saturation, -
%   S_o = oil saturation, -
%   S_w = water saturation, -
% T_R = reservoir temperature, deg. C
%
% JDJ, 27-12-13, last revised 27-12-13

% Retrieve data:
q_g_sc   = q_sc(1); 
q_o_sc   = q_sc(2); 
rho_g_sc = rho_sc(1); 
rho_o_sc = rho_sc(2); 
k_rg_0   = rel(1);
k_ro_0   = rel(2);
n_g      = rel(4);
n_og     = rel(5);
S_gc     = rel(8);
S_or     = rel(9);
S_wi     = rel(10);

% Compute local properties:
R_sb = q_g_sc/q_o_sc; % R_sb = R_go, m^3/m^3
[q,~] = local_q_and_rho(oil,p,q_sc,R_sb,rho_sc,T_R);
q_g = q(1); % local gas rate, m^3/s
q_o = q(2); % local oil rate, m^3/s

% Compute approximate saturations:
mu_g = gas_viscosity(p,rho_g_sc,T_R); % local gas viscosity, Pa s
mu_o = oil_viscosity(p,R_sb,rho_g_sc,rho_o_sc,T_R); % local oil viscosity, Pa s
S_g_star = (1+((k_ro_0*mu_g*q_g)/(k_rg_0*mu_o*q_o))^(-2/(n_g+n_og)))^-1;
S_g = S_gc + S_g_star * (1-S_gc-S_or-S_wi);
S_w = S_wi;
S_o = 1-S_g-S_w;
sat = [S_g,S_o,S_w];
