function [ahd,inc,azi,nor,eas,tvd] = evaluate_survey(fid)
% function [ahd,inc,azi,nor,eas,tvd] = evaluate_survey(fid)
%
% Reads a survey file in ASCII format that should contain three columns with along-hole depth
% (m), inclination (deg) and azimuth (deg). Evaluates the survey file using the minimum
% curvature method and computes an array of coordinates expressed as Northing (m), Easting (m)
% and true-vertical depth (m). 
% 
% ahd = along-hole depth (m)
% inc = inclination (rad)
% azi = azimuth (rad)
% nor = northing (m)
% eas = easting (m)
% tvd = true-vertical depth (m)
%
% JDJ, 19-08-04, last revised 23-11-16

% Read in survey data:
[sur_deg, n] = fscanf(fid,'%g %g %g',[3 inf]);
m = n/3; % number of survey points
sur_deg = sur_deg'; % store as column vector
ahd = sur_deg(:,1); % along-hole-depth, m
inc_deg = sur_deg(:,2); % inclination, deg
azi_deg = sur_deg(:,3); % azimuth, deg

% Convert to radians:
inc = inc_deg*pi/180;
azi = azi_deg*pi/180;

% Initialization:
x_vec = zeros(m,3);
e = zeros(m,3);
e_C = zeros(m,3);
Delta_s = zeros(m,1);
C = zeros(m,1);
gamma = zeros(m,1);
e(1,1) = sin(inc(1))*cos(azi(1));
e(1,2) = sin(inc(1))*sin(azi(1));
e(1,3) = cos(inc(1));

for i = 2:m
    % 1) Direction vectors:
    e(i,1) = sin(inc(i))*cos(azi(i));
    e(i,2) = sin(inc(i))*sin(azi(i));
    e(i,3) = cos(inc(i));

    % 2) Chord direction vectors:
    e_C(i-1,:) = (e(i-1,:)+e(i,:))/norm(e(i-1,:)+e(i,:));

    % 3) Along-hole depth increments:
    Delta_s(i-1) = ahd(i)-ahd(i-1);

    % 4) Opening angles:
    inp = e(i-1,:)*e(i,:)'; % inner product
    if inp > 1 % check for round-off errors
        inp = 1;
    end
    gamma(i-1) = acos(inp);

    % 5) Chord lengths:
    if gamma(i-1) < eps % straight hole
        C(i-1) = Delta_s(i-1);
    else % curved hole
        C(i-1) = 2*(Delta_s(i-1)/gamma(i-1))*sin(gamma(i-1)/2);
    end
        
    % 6) Survey points:
    x_vec(i,:) = x_vec(i-1,:)+C(i-1)*e_C(i-1,:);
end

nor = x_vec(:,1);
eas = x_vec(:,2);
tvd = x_vec(:,3);