% Script file system_01
%
% Computes the pressure drop over a simple system consisting of a near-well bore area, a
% tubing, a well head choke and a flow line. The analysis node is at the top of the tubing.
%
% JDJ, 23-10-12, last revised 29-05-17

clear variables
close all

% ---------------------------------------------------------------------------------------------
% Input data:
% ---------------------------------------------------------------------------------------------
alpha_fl = from_deg_to_rad(90); % flow line inclination, rad
alpha_w  = 0; % wellbore inclination, rad
av = 0; % parameter to select reservoir pressure, -
        % av = 0: use reservoir pressure at external boundary
        % av = 1: use average reservoir pressure
choke_model = 1; % Gilbert (1954)
d_fl = 0.10053; % inside diameter of a 4.5 inch flow line, m
d_w = 0.07600; % inside diameter of a 3.5 inch, 9.30 lbs/ft tubing, m
d_ch = from_in_to_m(40/64); % inside diameter of a 40/64 inch choke, m
e_fl = 30e-6; % flow line roughness, m
e_w = 30e-6; % tubing roughness, m
f_w = 0.0; % water cut, - 
fluid = 4; % multi-phase gas-oil-water flow, Mukherjee and Brill correlation, -
h = 20; % reservoir hight, m
k = 0.6e-13; % permeability, m^2
k_ro_0 = 0.9; % end-point relative permeability to oil, -
l_fl = 500; % flow line length, m
n_pt = 50; % number of points in plot, -
oil = 1; % black oil; parameters computed with the aid of Standing correlations, -
p_R = 24e6; % reservoir pressure, Pa
p_sep = 0.60e6; % separator pressure, Pa 
q_o_sc_max = -2.5e-3; % maximum oil rate, m^3/s 
r_e = 800; % radius external drainage boundary, m
r_w = 0.1778; % wellbore radius, m
R_sb = 50; % solution GOR at bubble point, m^3/m^3 
rho_g_sc = 0.95; % gas density at standard conditions, kg/m^3
rho_o_sc = 850;  % oil density at standard conditions, kg/m^3
rho_w_sc = 1000; % water density at standard conditions, kg/m^3
S = 0; % skin, -
semi = 0; % use steady-state conditions
s_tot = 3000; % total along-hole well depth, m 
T_sep = 60; % separator temperature, deg. C
T_tf = 70; % tubing head temperature, deg. C
T_wf = 120; % bottomhole temperature, deg. C
% ---------------------------------------------------------------------------------------------
% End of input data
% ---------------------------------------------------------------------------------------------

% Compute auxiliary variables:
R_go = R_sb; % no free gas 
k_o = k * k_ro_0; % effective permeability to oil
rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc];
delta_q_o_sc = q_o_sc_max/n_pt; % oil rate increment, m^3/s

% Initialize output:
results = zeros(n_pt,3);

warning off
for i=1:n_pt
    q_o_sc = i * delta_q_o_sc; % oil rate, m^3/s 
    q_g_sc = R_go * q_o_sc; % gas rate, m^3/s
    q_w_sc = (f_w/(1-f_w)) * q_o_sc; % water rate, m^3/s
    q_sc = [q_g_sc,q_o_sc,q_w_sc];
    
    % 1) Compute upstream performance curve:
    % 1.1) Compute pressure drop over the reservoir:
    p_wf = res_oil_simp(av,h,k_o,oil,p_R,q_sc,r_e,r_w,rho_sc,semi,S,T_wf); % BHP
    % 1.2) Compute pressure drop over the wellbore: 
    p_tf_us = pipe(alpha_w,d_w,e_w,fluid,oil,p_wf,q_sc,rho_sc,s_tot,0,T_wf,T_tf); % THP
    results(i,1)= -q_o_sc; % flow rate
    results(i,2)= p_tf_us; % upstream pressure

    % 2) Compute downstream performance curve:
    % 2.1) Compute pressure drop over the flow line:
    p_fl = pipe(alpha_fl,d_fl,e_fl,fluid,oil,p_sep,q_sc,rho_sc,0,l_fl,T_sep,T_tf); % FLP  
    % 2.2) Compute pressure drop over the choke: 
    p_tf_ds = choke_multi_phase_simp(choke_model,d_ch,p_fl,q_sc); % THP
    results(i,3)= p_tf_ds; % downstream pressure 
end
warning on

% Compute intersection:
% Makes use of Matlab function fzero to search for the zeros of a function fitted through
% points that represent differences between the upstream and down stream pressure drop curves.
difference = fit(results(:,1),results(:,2)-results(:,3),'linearinterp');
options.Display = 'off'; % supresses error meassages in case of no intersection
q_o_sc_start = 1e-3; % starting point for search, m^3/s
[q_o_sc,~,exitflag] = fzero(difference,q_o_sc_start,options);
results_us_fit = fit(results(:,1),results(:,2),'linearinterp'); 
p_tf = results_us_fit(q_o_sc);

% Create plot:
plot(results(:,1)*1e3,results(:,2)/1e6,'LineWidth',1);
hold on
plot(results(:,1)*1e3,results(:,3)/1e6,'-r','LineWidth',1);
legend('Upstream','Downstream','Location','NorthEast')
if exitflag > 0 % intersection present 
    plot(q_o_sc*1e3,p_tf/1e6,'o','MarkerSize',6,'MarkerEdgeColor','k','MarkerFaceColor','k')
    plot([q_o_sc*1e3 q_o_sc*1e3],[0.5 p_tf/1e6],'-k','LineWidth',1)
    plot([0 q_o_sc*1e3],[p_tf/1e6 p_tf/1e6],'-k','LineWidth',1)
end
hold off
axis([0 2.5 0.5 3.5])
grid on  
xlabel('Oil Flowrate,\it -q_{o,sc}\rm (10^{-3} m^3/s)')
ylabel('FTHP,\it p_{tf}\rm (MPa)')

