% Script file system_03_part_2
%
% Creates a plot of the required gas compressor discharge pressure as a function of choke
% diameter for given values of the lift gas rate and the wellbore pressure at the depth of the
% gas lift valve.         
%
% JDJ, 11-01-14, last revised 29-05-17

clear variables
close all

% ---------------------------------------------------------------------------------------------
% Input data:
% ---------------------------------------------------------------------------------------------
% Production data:
p_4 = 9.38e6; % wellbore pressure at the depth of the gas lift valve, Pa
p_7_av = 18e6;% available gas compressor discharge pressure, Pa
q_g_sc_lift = 0.10; % lift gas rate, m^3/s
% Note: positive value because of sign convention for injection and production (gas is
% injected through the gas lift line) 

% Gas choke data:
C_d = 0.75; % choke discharge coefficient, -
% For choke diameter d_ch, see plot data below.

% Fluid data:
fluid = 2; % single-phase gas, -
oil = 2; % arbitrary value; required by pipe.m
rho_g_sc = 0.93; % gas density at standard conditions, kg/m^3
rho_o_sc = 845;  % oil density at standard conditions, kg/m^3
rho_w_sc = 1070; % water density at standard conditions, kg/m^3
c_p = 0.25; % specific heat capacity of gas at constant pressure, J/(kg K)
c_v = 0.20; % specific heat capacity of gas at constant volume, J/(kg K)

% Well data:
s_gl = 1600; % gas lift setting depth, m
alpha_t = 0; % tubing inclination, rad
d_c_i = 0.154; % inside diameter of a 6-5/8 inch, 24 lbs/ft casing, m
d_t_o = 0.102; % outside diameter of a 4.0 inch, 11 lbs/ft tubing, m
e_c = 30e-6; % casing roughness, m
e_t = 30e-6; % tubing roughness, m

% Gas lift line and riser data:
L_gl = 1000; % gas lift line length, m
alpha_gl = from_deg_to_rad(90); % gas lift line inclination, rad
d_gl = 0.048; % inside diameter of a 2.375 inch gas lift, m
e_gl = 30e-6; % gas lift line roughness, m
L_gr = 120; % gas lift riser length, m
alpha_gr = 0; % gas lift riser inclination, rad
d_gr = d_gl; % inside diameter gas lift riser, m
e_gr = e_gl; % gas lift riser roughness, m

% Gas lift valve data (modelled as restriction):
C_d_glv = 0.9; % gas lift valve discharge coefficient, -
d_glv = from_in_to_m(0.25); % gas lift valve opening, m

% Temperature data:
T_4 = 92; % oil @ gas lift valve temperature, deg. C
T_7 = 50; % gas compressor discharge temperature, deg. C
T_8 = 45; % gas choke exit temperature, deg. C
T_9 = 45; % gas riser exit temperature, deg. C
T_10 = 15; % gas tubing head temperature, deg. C
T_11 = 25; % gas @ gas lift valve temperature, deg. C

% Plot data: 
n_pt = 50; % number of points in plot, -
d_ch_min = from_in_to_m(4/64); % minimum choke diameter, m
d_ch_max = from_in_to_m(16/64); % maximum choke diameter, m

% ---------------------------------------------------------------------------------------------
% End of input data
% ---------------------------------------------------------------------------------------------

% Compute auxiliary variables:
L_top = s_gl; % distance between well head and gas lift valve, m
d_eq = (d_c_i^2 - d_t_o^2 ) / (d_c_i + d_t_o); % equivalent annulus diameter, m
gamma = c_p/c_v; % ratio of specific heat capacities, -
delta_d_ch = (d_ch_max - d_ch_min) / n_pt; % choke diameter increment, m
rho_sc = [rho_g_sc,rho_o_sc,rho_w_sc];
q_sc = [q_g_sc_lift,0,0];

% Initialize output:
results = zeros(n_pt,2);

for i=1:n_pt
    d_ch = d_ch_min + i * delta_d_ch; % choke diameter, m 
    % 1) Compute pressure drop over the gas lift valve (modelled as a restriction):
    p_11 = choke_gas(C_d_glv,d_glv,gamma,p_4,q_sc,rho_sc,T_11);
    % 2) Compute pressure drop over the annulus:
    p_10 = pipe(alpha_t,d_eq,e_c,fluid,oil,p_11,q_sc,rho_sc,L_top,0,T_11,T_10);
    % 3) Compute pressure drop over the gas lift line:
    p_9 = pipe(alpha_gl,d_gl,e_gl,fluid,oil,p_10,q_sc,rho_sc,L_gl,0,T_10,T_9);
    % 4) Compute pressure drop over the lift gas riser:
    p_8 = pipe(alpha_gr,d_gr,e_gr,fluid,oil,p_9,q_sc,rho_sc,L_gr,0,T_9,T_8);
    % 5) Compute pressure drop over the choke:
    p_7 = choke_gas(C_d,d_ch,gamma,p_8,q_sc,rho_sc,T_7); 
    results(i,1)= d_ch; % choke diameter, m
    results(i,2)= p_7; % required compressor discharge pressure
end

% Performance plots at compressor:

% Pressure as function of choke diameter in m:
plot(results(:,1)*1e3,results(:,2)/1e6,'LineWidth',1);
axis([d_ch_min*1e3 d_ch_max*1e3 5 30])
grid on  
xlabel('Choke Diameter,\it d_{ch}\rm mm')
ylabel('Compressor Discharge Pressure,\it p_7\rm (MPa)')

% Pressure as function of choke diameter in 1/64th in:
figure
d_ch_min_FU = 64*from_m_to_in(d_ch_min);
d_ch_max_FU = 64*from_m_to_in(d_ch_max);
plot(64*from_m_to_in(results(:,1)),results(:,2)/1e6,'LineWidth',1);
axis([d_ch_min_FU d_ch_max_FU 5 30])
grid on  
xlabel('Choke Diameter,\it d_{ch}\rm (1/64^{th} in)')
ylabel('Compressor Discharge Pressure,\it p_7\rm (MPa)')
